function [sobj, raw, offsetlist] = calcSpectra(obj,varargin)
    % Calculate the power spectra of an ts_data object
    %         spectra = calcSpectra(obj, ...)
    %   [spectra,raw,offsetlist] = calcSpectra(obj, ...)
    % Outputs:
    %   spectra             the ts_spectra object
    %   raw                 a "raw" (unaveraged) version of the spectra and
    %                       coherences that can be input to subsequent calls
    %						to ts_spectra to calculate spectra over multiple
    %						ts_data objects.
    %
    % Inputs:
    %   obj               an ts_data object
    %	field,value pairs	optional parameters to modify how the spectrum
    %						is calculated. (default values are shown in square
    %						brackets)
    % _________________________________________________________________________
    %	field		shortfield	value
    % 	'start'         'st'  seconds from start of data to start spectra [0]
    % 	'length'        'ln'  seconds of data to analyze  [to end of data]
    % 	'windowlength'  'wl'  seconds for each FFT window length [2048/samprate]
    % 	'taper'         'tp'  window taper: 'prol1pi' or 'prol4pi' ['prol4pi']
    % 	'offsetlist'    'ol'  list of offsets in seconds for the start of each FFT window
    % 	'removeoutliers''ro'  acceptable ratio from the median spectral energy (will
    %                           throw out all spectra (up to 10%) outside of this
    %                           ratio) [no rejection]
    %                           Note: iterates several times, will never throw
    %                           out more than 10% per iteration (in case of
    %                           outliers that mess up the median)
    % 	'badoffsets'   'bo'   Nx2 matrix of offset blocks NOT to use
    %                         each row is [start_secs end_secs]
    % 	'badbuffer'    'bb'   extend each bad section by this many seconds [0]
    %                         if positive, adds to beginning and end of bad
    %                                   blocks
    %                         if negative, adds to end of bad blocks
    % 	'subtractxf'   'sx'   ts_xferfun to use when subtracting one channel from another
    % 	'rawspectra'   'rs'   input raw spectra from a previous ts_spectra
    % 	                     (rawspectra)
    % GRANDFATHERED: same as above, but in samples, not seconds:
    % 	'startaddress','seglength','windowsize','badaddresses','badbuffer'
    % _________________________________________________________________________
    %
    % The window length will be rounded up to the value giving a power of 2
    % samples
    % if windowoffsets are not specified, FFT windows will be even spaced N
    %	samples apart, where N==windowlength for a prol1pi taper and
    %	N=0.7*windowlength for a prol4pi taper
    %
    debug=0;
    
    sobj=[];raw=[];soffsetlist=[];
    if ~obj.issynchronized(),
        error('cannot calculate spectra for non-synchronized data');
    end
    
    showdlg = ts_get('dialogs');
    
    ts_debug(1,'Starting...');
    
    datalen = obj.channel(1).nSamples;
    sr = obj.channel(1).samprate;
    
    % Set defaults
    startaddr = 1;		seglen=0;		ws = 2048;
    taper= 'prol4pi';
    oldtimeoffsets= [];	addrlist= [];	badbuff= 0;		raw= [];
    removeoutliers=0;	offsetlist=[];
    
    % "New" variables: sr=samprate, ws=windsize, al=addresslist, bl=badlist
    fields={'name', 'shortname', 'info'                                                , 'command'                   ,'othernames'};
    S={	'start'        ,'st','start time (seconds from start of data [0])'             , 'startaddr=calcStartAddr(val,sr);' ,{'startat'}; ...
        'length'       ,'le','length of data to use (seconds [to end of data])'        , 'seglen=round(val*sr);'         ,{'spectlength'}; ...
        'windowlength' ,'wl','FFT window length (seconds, [500])'            , 'ws = calcWindSize(val*sr,sr);',{'windsize','windowlen','windlen'}; ...
        'taper'        ,'ta','window taper (''prol1pi'' or ''prol4pi'' [''prol4pi''])' , 'taper = val;'              ,{'windowtype'}; ...
        'offsetlist'   ,'ol','offsets for start of each FFT window (array of seconds)' , 'addrlist=round(val*sr)+1;' ,{}; ...
        'removeoutliers','ro','remove spectral outliers (threshold ratio [inf])'       , 'removeoutliers=val;'       ,{}; ...
        'badoffsets'   ,'bo','offsets NOT to use [numerical list, or ts_seqlist]'        , 'badoffsets=val;'           ,{}; ...
        'badbuffer'     ,'bb','extend each bad offset or window by this many seconds [0]' , 'badbuff=round(val*sr);'    ,{}; ...
        'subtractxf'   ,'sx','subtract coherent noise  (transfer function)'            , 'subxf=val;'                ,{}; ...
        'rawspectrum'  ,'rs','input raw spectra from a previous ts_spectra (rawspecta)', 'oldtimeoffsets=checkRaw(val); raw=val;' ,{'rawspect'}; ...
%        'startaddress' ,'sa','GRANDFATHERED:'                                          , 'sa = val;'                 ,{'startaddr'}; ...
%        'seglength'    ,'sl','GRANDFATHERED: length of data to use (samples)'          , 'seglen = val;'             ,{'seglen'}; ...
%        'windowsize'   ,'ws','GRANDFATHERED: FFT window length in (samples)'           , 'ws = calcWindSize(val,sr);',{}; ...
%        'badbuffer'    ,'bu','GRANDFATHERED: samples to subtract from ''good'' times'  , 'badbuff=val;'              ,{'badbuff','badbuffer'}; ...
%        'badaddresses' ,'ba','GRANDFATHERED: list of addresses NOT to use'             , 'badaddrs=val; '            ,{'badaddrs'} ...
        };
    optiontable=cell2struct(S,fields,2);
    %namelist={optiontable.name};
    
    %disp('a');  % DEBUG
    % read/do options
    if length(varargin)>1,
        for i = 1:2:length(varargin);
            field=lower(varargin{i});
            val=varargin{i+1}; %#ok<NASGU>
            cmd=findCmd(field,optiontable);
            if isempty(cmd),
                ts_warning('Unknown option ''%s''.  Valid options are:',field);
                for j=1:length(optiontable),
                    op=optiontable(j);
                    fprintf('%16s (''%s''): %s\n',sprintf('''%s''',op.name),op.shortname,op.info);
                end
                error('Quitting');
            else
                eval(cmd);
            end
        end
    end
    %disp('b');  % DEBUG
    
    if startaddr < 1, startaddr=1; end
    if startaddr+seglen-1 > datalen, seglen = datalen-startaddr+1; end
    if seglen==0; seglen= datalen-startaddr+1; end
    endaddr = startaddr+seglen-1;
    if endaddr > datalen, error('End time > data length'); end
    if startaddr > endaddr-ws+1,
        ts_warning('The first window goes beyond the end address:\n\tstart (%g) + length (%g) > %g', ...
            (startaddr-1)/sr,ws/sr,(endaddr-1)/sr);
    end
    %disp('c');  % DEBUG
    
    while ws > seglen, ws = ws/2; end
    
    switch taper
        case 'prol4pi',
            windstep = ceil(0.7*ws);
            windfunc = @prol4pi;
        case 'prol1pi',
            windstep = ws;
            windfunc = @prol1pi;
        otherwise, 		error(['Unknown window taper: ' taper]);
    end
    %disp('d');  % DEBUG
    
    if exist('badoffsets','var');	% get addresses by avoiding a list of bad times
        if ~isnumeric(badoffsets)
            error('badoffsets must be an Nx2 numerical matrix');
        end
        badaddrs=offsetblocks_to_addresses(badoffsets,sr);
        addrlist = calcAddrList(obj,badaddrs,ceil(badbuff*sr), ...
                                   startaddr,endaddr,ws,windstep);
    elseif isempty(addrlist),
        addrlist = startaddr:windstep:endaddr-ws+1;
    end
    if debug,
        %disp('e');  % DEBUG
        addrlist    % DEBUG
    end
    mywind = windfunc(ws);
    mywind = mywind(:)';                       % Make row vector
    taperbias = (sum(abs(mywind).^2))/ws;
    freqstep = sr/ws;
    freqs = freqstep*(1:ws/2);
    numfreqs = length(freqs);
    
    oldRaw=raw;
    calcSpectra=1;
    
    %% Calculate "raw" spectra and cross-spectra
    while calcSpectra,  % Loop over all address, calculating indiv spectra and cross-spectra
        if isempty(addrlist)
            ts_verbose(1,'No spectra calculated!');
            return;
        end
        
        numtotalwinds = length(addrlist) + length(oldtimeoffsets);
        
        %Zero out spec and coh arrays
        numchans = obj.nChannels;
        if isempty(raw),
            for i = 1:numchans,
                raw.spec(i,:) = zeros(1,ws/2);
                for j = i:numchans,
                    raw.xspec(i,j,:) = zeros(1,1,ws/2);
                end
            end
        elseif isempty(raw.xspec),
            for i = 1:numchans,
                if isempty(raw.spec)
                    raw.spec(i,:) = zeros(1,ws/2);
                end
                for j = i:numchans,
                    raw.xspec(i,j,:) = zeros(1,1,ws/2);
                end
            end
        end
        
        % Calculate FFTs, add up spectra, cross-spectra
        str = sprintf('Calculating spectra: %d %d-pt windows',length(addrlist),ws) ;
        if length(addrlist) > 1 && showdlg,
            h_waitwind = waitbar(0,str);
        end
        alreadySaidInterp=0;
        ts_verbose(1,str);
        nrg=zeros(numchans,length(addrlist)); % Energy in each spectrum, used to look for outliers
        for iwind = 1:length(addrlist),
            addr = addrlist(iwind);
            iaddr = addr:addr+ws-1;
            ffres=zeros(numchans,numfreqs);
            for i = 1:numchans
                data = obj.channel(i).data(iaddr);
                data = data - mean(data);
                windowedData = data .* mywind;
                %		temp = fft(windowedData/ws);
                temp = fft(windowedData);
                ffres(i,:) = temp(2:ws/2+1)/sqrt(taperbias);	% Address 1 is DC value
                ffres(i,:) = ffres(i,:) ./ response(obj.channel(i).metadata.response,freqs);
            end
            % IF the transfer functions are provided, SUBTRACT influence from other channels
            if exist('subxf','var');
                for i = 1:subxf.nChannels,
                    dchan = subxf.channel(i).drivechan;
                    rchan = subxf.channel(i).respchan;
                    xffreqs = subxf.freqs;
                    xf = subxf.channel(i).data;
                    %xfgood = get(subxf,'gooddata',i);
                    xfgood = subxf.channel(i).gooddata;
                    interpit=0;
                    if (length(xffreqs) ~= length(freqs)),   interpit=1;
                    elseif xffreqs ~= freqs,                 interpit=1;
                    end
                    if interpit,
                        if ~alreadySaidInterp,
                            ts_verbose(1,'Interpolating transfer function');
                            alreadySaidInterp=1;
                        end
                        subber = interp1(xffreqs,xf,freqs);
                        gooddata = interp1(xffreqs,xfgood,freqs);
                        gooddata = (gooddata == 1);	% Only use xfs where the coherence is
                        % definitely greater than signif level
                        subber = subber .* gooddata;
                    else
                        subber = xf .* xfgood;
                    end
                    ffres(rchan,:) = ffres(rchan,:) - subber.*ffres(dchan,:);
                end
            end % exist('subxf')
            for i = 1:numchans,
                onespect=abs(ffres(i,:).*abs(ffres(i,:)));
                nrg(i,iwind)=sum(onespect);
                raw.spec(i,:) = raw.spec(i,:) + onespect;
                for j = i+1:numchans,
                    raw.xspec(i,j,:) = raw.xspec(i,j,:) + reshape(conj(ffres(i,:)).*ffres(j,:),1,1,numfreqs);
                end
            end
            if length(addrlist) > 1 && showdlg,
                waitbar(iwind/length(addrlist));
            end
        end % for iwind=1:length(addrlist),
        calcSpectra=0;	% Already done
        % REMOVE OUTLIERS
        if ~exist('iGoodWinds','var') && removeoutliers,
            maxReject=0.1;  % Reject at most this proportion in each iteration
            goodWinds=ones(1,length(addrlist));
            iGoodWinds=1:length(addrlist);
            nLastGoodWinds=length(addrlist);
            while 1,
                basenrg=median(nrg(:,iGoodWinds),2);
                for i=1:numchans,
                    ratios = nrg(i,:)./basenrg(i);
                    goodTemp=ratios<removeoutliers & ratios>1/removeoutliers;
                    if length(find(goodTemp))<(1-maxReject)*nLastGoodWinds,    % only reject 10 maxi
                        ratios=ratios+1./ratios;    % Normalize to >1
                        for j=1:nLastGoodWinds*maxReject   % Reject 10%
                            [~,iy]=max(ratios);
                            ratios(iy)=0;
                        end
                        goodTemp=ratios>0;
                    end
                    goodWinds=goodWinds & goodTemp;  % must be good on all channels to hold window
                end
                iGoodWinds=find(goodWinds);
                nNewGoodWinds=length(iGoodWinds);
                if nNewGoodWinds == nLastGoodWinds, break;
                else
                    ts_verbose(1,'Threw away %d spectra whose energy were more(less) than %gx greater(less) than the median value(s) of:',...
                        nLastGoodWinds-nNewGoodWinds, removeoutliers);
                    ts_verbose(1,'%g ',basenrg);
                    ts_verbose(2,'All energies:');
                    ts_verbose(2,'%g ',nrg(:,iGoodWinds));
                    nLastGoodWinds=nNewGoodWinds;
                end
            end
            fprintf('removeoutliers(%g) threw away %d of %d windows: ', ...
                removeoutliers,length(addrlist)-length(iGoodWinds),length(addrlist));
            if length(iGoodWinds) < length(addrlist),
                addrlist=addrlist(iGoodWinds);
                raw=oldRaw;  % Just go back to the previously selected spectra????
                calcSpectra=1;
                fprintf('recalculating spectra\n');
            else
                fprintf('\n');
            end
        end
        if length(addrlist) > 1 && showdlg,
            close(h_waitwind);
        end
    end  % while calcSpectra: "raw" spectra/cross-spectra calculation
 
    %% Verify that there are enough windows to do something
    if numtotalwinds < 2,	% If less than two windows, coherences are useless
        for i = 1:numchans,
            if numtotalwinds < 1;	% If less than one window, spect are useless
                raw.spec(i,:) = NaN*ones(1,ws/2);
            end
            for j = i+1:numchans,
                raw.xspec(i,j,:) = NaN*ones(1,ws/2);
            end
        end
    end
    
    %% Calculate spectra and coherencies from the raw (simple summation) spectra and cross-spectra
    %========================================================================
    sampint = 1/sr;
    multfac = 2*sampint/(numtotalwinds*ws); % Bendata&Piersol 1986 eqs 11.100 & 11.102
    spectdata=zeros(numchans,numfreqs);
    coherdata=zeros(numchans,numchans,numfreqs);
    for i = 1:numchans,
        spectdata(i,:) = raw.spec(i,:)*multfac;
        coherdata(i,i,:) = ones(1,1,numfreqs);
        for j = i+1:numchans,
            coherdata(i,j,:) = raw.xspec(i,j,:) ./ ...
                reshape(sqrt(raw.spec(i,:).*raw.spec(j,:)),1,1,numfreqs);
            coherdata(j,i,:) = conj(coherdata(i,j,:));
        end
    end
    
    %% Don't pass on raw data if there aren't enough to work with
    if numtotalwinds < 2,	% If less than two windows, coherences are useless
        raw.xspec = [];
        if numtotalwinds < 1;	% If less than one window, spect are useless
            raw.spec = [];
        end
    end
    
    %% Final prettying up
    offsetlist=(addrlist-1)/sr;
    timeoffsets= offsetlist;    % OK since data are synchronized
    csl=sqrt(2/length(offsetlist));    % coherency 95% significancy level
    %sinfo=zeros(1,numchans);
    for i = 1:numchans
        sinfo(i)=ts_spectra_channelinfo(obj.channel(i).metadata.fullname, obj.channel(i).metadata.response.inUnits);
    end
    timeoffsets = [oldtimeoffsets timeoffsets];  % Combine old and new timeoffsets (BUT WHERE ARE OLD RAW COMBINED???)
    sobj=ts_spectra(freqs,spectdata,coherdata,ws,sr,taper,csl,obj.channel(1).starttime,timeoffsets,sinfo);
    raw.timeoffsets = timeoffsets;
    %ctr = 0;
end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function cmd=findCmd(field,ot)
    for i=1:length(ot),
        tester = {ot(i).name,ot(i).shortname,ot(i).othernames{:}};
        if any(strcmp(field,tester)), cmd=ot(i).command; return; end
    end
    cmd=[]; 
end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function sa=calcStartAddr(starttime,samprate) %#ok<DEFNU>
    if isnumeric(starttime),	% seconds since start of data
        sa = round(starttime*samprate)+1;
    elseif isa(starttime,'ts_datetime'),
        ts_verbose(0,'calculating start point based on channel 1 start time')
        sa = round(starttime*samprate)+1;
    else
        error('''start'' value must be a number (seconds) or an ts_datetime object');
    end
end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ws = calcWindSize(ws,sr) %#ok<DEFNU>
    if log2(ws) ~= round(log2(ws)),
        ws = 2^(ceil(log2(ws)));
        ts_verbose(1,'increased window length to %g seconds to make samples a power of 2',ws/sr);
    end
end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %===========================================================================
function badaddrs=offsetblocks_to_addresses(badoffsets,samplerate)
    %
    [M,N]=size(badoffsets);
    if N ~=2,
        error('offset block matrix does not have two columns')
    end
    badaddrs=[];
    for i=1:M
        b=floor(badoffsets(i,1)*samplerate)+1;
        e= ceil(badoffsets(i,2)*samplerate)+1;
        badaddrs=[badaddrs b:e];
    end
    badaddrs=remove_duplicates(badaddrs);
end

function addrlist = calcAddrList(obj,badaddrs,filtbuffer,minaddr,maxaddr,windsize,windstep)
    % CALCWINDADDRS returns possible start addresses for a set of data windows,
    %		given the constraints on bad data and min and max addresses
    %
    % Usage: addrs = calcWindAddrs(obj,badaddrs,filtbuffer,minaddr,maxaddr,windsize,windstep)
    %   OUTPUT:
    %	addrlist =   List of starting addresses [numerical vector]
    %   INPUT:
    %	badaddrs =   Addresses to skip                 [numeric]
    %	filtbuffer = Samples to skip at the beginning of data and after each bad window
    %                (to avoid edge effects of a causal filter)
    %                [integer]
    %	minaddr =    Address at which to start windows [integer]
    %	maxaddr =    Maximum address for windows       [integer]
    %	windsize =   Size of the window in data points [integer]
    %	windstep =   Default spacing between windows.  [integer]
    %                If not defined, set to windsize
    debug=0;
    if nargin == 4, windstep = windsize; end
    nS=obj.channel(1).nSamples;
    
    % Make possibleaddrs an array of all data addresses
    possibleaddrs=1:nS;
    
    % Set "bad" addresses to zero
    possibleaddrs(possibleaddrs<minaddr | possibleaddrs>maxaddr) = 0;
    if filtbuffer > 1,
        badaddrs=addbuffer(badaddrs,filtbuffer,'after');		% Add filtbuffer to end of each badaddress window
    end
    badaddrs=badaddrs(badaddrs>0 & badaddrs<=nS);
    possibleaddrs(badaddrs) = 0;
        
    % Make a list of starting addresses based on possibleaddrs
    ia=0; addrlist=[]; addr = 1;
    while addr+windsize<=nS,
        addr = addr + find(possibleaddrs(addr:end)>0,1,'first')-1;
        if addr+windsize<=nS,
            if ~any(possibleaddrs(addr:addr+windsize-1)==0),
                ia=ia+1;
                addrlist(ia)=addr; %#ok<AGROW>
                addr=addr+windstep;
            else
                addr=addr+find(possibleaddrs(addr:end)==0,1,'first');
            end
        end
    end
    %disp('cal_4');   % DEBUG
    if debug,
        addrlist % DEBUG
    end
end
    

function bad_out = addbuffer(badaddrs,buffer,direction)
    % Add filtbuffer to each badaddress
    % Inputs:
    %   badaddrs: list of bad addresses
    %   buffer: number of addresses to extend beyond to each bad address
    %   direction: on which side to add the before
    %           'before': addresses smaller than the badaddress
    %           'after' : addresses greater than the badaddress
    %           'both'  : on both sides of the bad address
    % Output:
    %   bad_out : new list of bad adresses
    if ~exist(where,'var')
        direction='after';
    end
    bb=buffer; ba=buffer;
    if ~strcmp('case','both')
        switch direction,
            case 'before'
                ba=0;
            case 'after'
                bb=0;
            otherwise
                error('unknown direction: %s',direction)
        end
    end
    % Make a list of all possible bad addresses (including overlaps)
    new_a=[];
    for a = badaddrs
        new_a=[new_a a-bb:a+ba];
    end
    bad_out=remove_duplicates(new_a);
end

function out_vect=remove_duplicates(in_vect)
    in_vect=sort(in_vect);
    out_vect=in_vect(1);
    ia=1; 
    old_val=out_vect;
    for val = in_vect(2:end),
        if val ~= old_val,
            ia=ia+1;
            out_vect(ia)=val;
            old_val=val;
        end
    end
end
            
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function oldtimeoffsets = checkRaw(val)	%#ok<DEFNU> % Make sure val is a raw spectrum
    if isstruct(val),
        if length(fieldnames(val))==3 && isfield(val,'spec') ...
                && isfield(val,'coh') && isfield(val,'timeoffsets'),
            raw = val;
            oldtimeoffsets = raw.timeoffsets;
        else error(['raw has bad fields:'  ...
                sprintf(' %s',fieldnames(val))]);
        end
    elseif isempty(val)
        oldtimeoffsets=[];
    else
        error('raw is not a structure');
    end
end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function w = prol1pi(n)
    % PROL1PI(N) returns the N-point 1-pi prolate spheroidal window.
    
    %Converted from C code by W Crawford, 4-17-99
    fOnE = ones(1,n);
    x = (2*(1:n)-fOnE)/n;
    u = (2*fOnE-x) .*(x);
    w =	sqrt(2.) * (((((((((( ...
        5.3476939016920851e-11*u        + 2.2654256220146656e-9*fOnE) .*u + ...
        7.8075102004229667e-8*fOnE) .*u + 2.1373409644281953e-6*fOnE) .*u + ...
        4.5094847544714943e-5*fOnE) .*u + 7.0498957221483167e-4*fOnE) .*u + ...
        7.7412693304064753e-3*fOnE) .*u + 5.5280627452077586e-2*fOnE) .*u +...
        2.2753754228751827e-1*fOnE) .*u + 4.3433904277546202e-1*fOnE) .*u + ...
        2.2902051859068017e-1*fOnE);
end

function w = prol4pi(n)
    % PROL4PI(N) returns the N-point 4-pi prolate spheroidal window.
    
    %Converted from C code by W Crawford, 9-12-89
    fOnE = ones(1,n);
    x = (2*(1:n)-fOnE)/n;
    u = (2*fOnE-x) .*(x);
    w =	sqrt(2./.508125548147497) * ((((((((((((((((((((( ...
        2.6197747176990866e-11*u        + 2.9812025862125737e-10*fOnE) .*u + ...
        3.0793023552299688e-9*fOnE) .*u + 2.8727486379692354e-8*fOnE) .*u + ...
        2.4073904863499725e-7*fOnE) .*u + 1.8011359410323110e-6*fOnE) .*u + ...
        1.1948784162527709e-5*fOnE) .*u + 6.9746276641509466e-5*fOnE) .*u + ...
        3.5507361197109845e-4*fOnE) .*u + 1.5607376779150113e-3*fOnE) .*u + ...
        5.8542015072142441e-3*fOnE) .*u + 1.8482388295519675e-2*fOnE) .*u + ...
        4.8315671140720506e-2*fOnE) .*u + 1.0252816895203814e-1*fOnE) .*u + ...
        1.7233583271499150e-1*fOnE) .*u + 2.2242525852102708e-1*fOnE) .*u + ...
        2.1163435697968192e-1*fOnE) .*u + 1.4041394473085307e-1*fOnE) .*u + ...
        5.9923940532892353e-2*fOnE) .*u + 1.4476509897632850e-2*fOnE) .*u + ...
        1.5672417352380246e-3*fOnE) .*u + 4.2904633140034110e-5*fOnE);
end