function filename = writeMSEED(tdata,outfname,encodingFormat)
% Write a ts_data object to a MiniSEED data file
% Usage:
% 	fName = writeMSEED(tdata) % names the file using the start time & channel #
%	fName = writeMSEED(tdata,outfilename) uses outfilename, appends '.msd'
%	
% The encoding format will be standard 32-bit.  Steim1 compression is also
% available by changing encodingFormat to 10 at the top of the M-file, but
% runs MUCH (>10x) slower!

% W Crawford 5/2009

if ~exist('format','var'),
    encodingFormat= 10;  % 2=24-bit integers
                    % 3=32-bit integers
                    % 10=Steim1
end


%% Standard beginning for ts_* functions
ts_debug(1,'Starting'); tic; 

%% Set up SEED parameters
reclen=4096;        % Recommended seed record length (bytes)
datastart=64;       % Byte # at which actual data starts
dataQual='D';       % Indeterminate data quality
locID='00';         % Location identifier
% Activity flag: No calib, time correction applied, no events, no leap seconds,
actflag=bitset(0,1,0)+bitset(0,2,1)+bitset(0,3,0)+bitset(0,4,0)+bitset(0,5,0)+bitset(0,6,0)+bitset(0,7,0);
ioflag=0;
dqflag=0;   % Data quality flag, change if there are data problems


%% Choose file name and open file
if exist('outfname','var'),
    filename=[outfname '.msd'];
else
    filename = sprintf('%s.%s.%s.sac', ...
                 strptime(tdata.channel(1).starttime,'%y%j.%H%M'), ...
                 strtrim(tdata.channel(1).metadata.station));
end
ts_verbose(1,'Writing MSEED file: %s ',filename);
fid = fopen(filename,'w','ieee-be');    % This seems to be necessary for headers and blockettes
if fid==-1,
	error('File %s could not be opened for writing',filename);
end

%% Write Data
for ich=1:tdata.nChannels,
    %% Get channel information
    ch=tdata.channel(ich);
    network= ch.metadata.network;
	station= ch.metadata.station;
%	instnum= get(tdata,'chinstnum',ich);
%	origchnum= tdata.channel(ich).fileinfo.readchans;
	chname = ch.metadata.component;
	stime = ch.starttime;	% Record start time
    samprate = ch.samprate; 
    numsamps = ch.numsamps;
	data = ch.data;
    isamp=1;
    srfact=samprate;    % Samples/second (<0 would be seconds/sample
    if rem(samprate,1), error('samp rate not an integer, will need to be more clever'); end
    srmult=1;           % samprate multiplier (if <0 is a divisor)
    
    %% Create the channel's data header string:
    if length(station)>5, station=station(1:5); end
    if length(locID)>2, locID=locID(1:2); end
    if length(chname)>3, chname=chname(1:3); end
    if length(network)>2, network=network(1:2); end
    headstr=sprintf('%06d%s %-5s%-2s%-3s%-2s',ich,dataQual,station,locID,chname,network);
    if length(headstr) ~=20,
        error('data header string is %d bytes long, should be 20',length(headstr));
    end
    ts_verbose(1,'SEED data record header starts with: ''%s''\n',headstr);
    
    %% Write each channel as consecutive data records
    lastVal=0;
    while isamp < numsamps, % Write individual records making up a channel
        numBytes=reclen-64;
        % Create the data part of the record
        [writeBlock,leftover,writesamps]=prepData(data(isamp:numsamps),encodingFormat,numBytes,lastVal);
        
        recioflag=ioflag;
        if isamp==1,
            recioflag=bitset(recioflag,4,1);    % Start of time series flag
        end
        if isamp+writesamps==numsamps,
            recioflag=bitset(recioflag,5,1);	% End of time series flag
        elseif isamp+writesamps > numsamps+1,
            error('isamp+writesamps > numsamps+1! (%d+%d > %d+1)',isamp,writesamps,numsamps);
        end

        %% Write Fixed Section of Data Header
        fwrite(fid,headstr,'char');   % Sequence number
        fwrite(fid,stime.year,'uint16');    % Start year
        fwrite(fid,stime.jday,'uint16');    %   Day of Year (Jan 1 is 1)
        fwrite(fid,stime.hour,'uint8');     %   Hours of Day (0-23)
        fwrite(fid,stime.minute,'uint8');   %   Minutes of Day (0-59)
            isec=floor(stime.second);
            subsecs=floor(10000*(stime.second-isec));
        fwrite(fid,isec,'uint8');           %	Seconds of day (0-59, 60 for leap secs)
        fwrite(fid,0,'uint8');              %	Unused for data (required for alignment)
        fwrite(fid,subsecs,'uint16');       %   .0001 seconds (0-9999)
        fwrite(fid,writesamps,'uint16');    % Number of samples in record
        fwrite(fid,srfact,'uint16');        % Sample rate factor
        fwrite(fid,srmult,'uint16');        % Sample rate multiplier
        fwrite(fid,actflag,'uint8');        % Activity flags
        fwrite(fid,recioflag,'uint8');      % I/O and clock flags
        fwrite(fid,dqflag,'uint8');         % Data quality flags
        fwrite(fid,1,'uint8');              % Number of blockettes that follow (before data)
        driftCorr=stime-ch.metadata.instrumentTime(stime);
        fwrite(fid,floor(driftCorr*10000),'int32');      % Time correction already applied to data
        fwrite(fid,datastart,'uint16');     % Beginning of data 
        fwrite(fid,48,'uint16');             % First blockette (right after this 48-byte header)
        %% Write Data Only SEED Blockette
        fwrite(fid,1000,'int16');           % Blockette type - 1000
        fwrite(fid,0,'int16');              % Byte number of next blockette
        fwrite(fid,encodingFormat,'int8');	% encoding format
        fwrite(fid,1,'int8');               % Byte swapping order (0 = little-endian)
        fwrite(fid,log2(reclen),'int8');    % Record length
        fwrite(fid,0,'int8');               % Reserved
        % Pad to byte 64
        fwrite(fid,'        ','char');

        %% Write data
        fwrite(fid,writeBlock,class(writeBlock));
        % Pad record
        fwrite(fid,leftover,class(leftover));
        
        lastVal=data(isamp+writesamps-1);   % for Steim compression
        isamp=isamp+writesamps;
        % To get exact time of next record, make new station with time zero
        % at last GMT time and correct drift rate, then calculate the GMT
        % time corresponding to the offset time
        temp=ch.metadata;
        temp.timezero=stime;
        stime=stime+writesamps/samprate;    % starttime of next record if no clock drift
        stime=temp.GPSTime(stime);      % drift corrected starttime
    end % while isamp < numsamps
end % for ich=1:nChannels
fclose(fid);
%% Standard end for ts_* functions
ts_debug(1,'Exiting after %.1f seconds\n',toc); 
end % end of function


function  [writeBlock,leftover,writesamps]=prepData(data,encodingFormat,numBytes,lastVal)
% numBytes is the number of bytes available to write to
% lastValue is the last data value in the preceding record.  Used only by
%           Steim.  You should set to zero the first time through.
switch encodingFormat,
    case 0, error('Data format: <ASCII Text>      not implemented');
    case 1, error('Data format: <16-bit integers> not implemented');
    case 2, % 24-bit integers
        sampsperrec=floor(numBytes/3);
        if length(data) < sampsperrec,
            writesamps=length(data);
        else
            writesamps=sampsperrec;
        end
        writeBlock=cast(data(1:writesamps),'int24');
        nLeft=numBytes-writesamps*3;
        leftover=zeros(1,nLeft,'int8');
    case 3, % 32-bit integers
        sampsperrec=floor(numBytes/4);
        if length(data) < sampsperrec,
            writesamps=length(data);
        else
            writesamps=sampsperrec;
        end
        writeBlock=cast(data(1:writesamps),'int32');
        nLeft=numBytes-writesamps*4;
        leftover=zeros(1,nLeft,'int8');
    case 4, error('Data format: <IEEE floating point> not implemented');
    case 5, error('Data format: <IEEE double precision floating point> not implemented');
    case 10, % Steim1 compression
        if length(data) > numBytes, data=data(1:numBytes); end % At most efficient, Steim 1 will store 1 sample/byte
        data=int32(data);
        mexfile=['writesteim.' mexext()];
        if exist(mexfile,'file')==3,
            [writeBlock,writesamps] = writesteim(data,numBytes,lastVal,1);
        else
            [writeBlock,writesamps] = makeSteim1(data,lastVal,numBytes);
        end
        leftover=zeros(1,numBytes-4*length(writeBlock),'uint8');
    case 11, % Steim2 compression
        if length(data) > numBytes*2, data=data(1:numBytes*2); end % At most efficient, Steim 2 will store 2 samples/byte
        data=int32(data);
        mexfile=['writesteim.' mexext()];
        if exist(mexfile,'file')==3,
            [writeBlock,writesamps] = writesteim(data,numBytes,lastVal,2);
            leftover=zeros(1,numBytes-4*length(writeBlock),'uint8');
        else
            error('Data format: <Steim2 compression> not implemented in M-file');
        end
    case 12, error('Data format: <GEOSCOPE Multiplexed 24-bit> not implemented');
    case 13, error('Data format: <GEOSCOPE Multiplexed 16-bit gain ranged, 3 bit exponent> not implemented');
    case 14, error('Data format: <GEOSCOPE Multiplexed 16-bit gain ranged, 4 bit exponent> not implemented');
    case 15, error('Data format: <US National Network compression> not implemented');
    case 16, error('Data format: <CDSN 16 bit gain ranged> not implemented');
    case 17, error('Data format: <Graefenberg 16 bit gain ranged> not implemented');
    case 18, error('Data format: <IPG - Strasbourg 16 bit gain ranged> not implemented');
    case 19, % Steim3 compression
        if length(data) > numBytes*4, data=data(1:numBytes*4); end % Just a guess, don't know maxium Steim3 efficiency
        data=int32(data);
        mexfile=['writesteim.' mexext()];
        if exist(mexfile,'file')==3,
            [writeBlock,writesamps] = writesteim(data,numBytes,lastVal,3);
            leftover=zeros(1,numBytes-4*length(writeBlock),'uint8');
        else
            error('Data format: <Steim3 compression> not implemented in M-file');
        end
    otherwise, error('Data format #%d <Unknown Format> not implemented',encodingFormat);
end
end

%% Function makeSteim1
function [writeBlock,writesamps] = makeSteim1(data,lastVal,numBytes)
global DEBUG

    d=diff([int32(lastVal) data]); id=1; nd=length(d);
    if DEBUG>1,
        fprintf('diff='); fprintf('%3d ',d(1:16)); fprintf('\n');
    end
    [str,maxsize,endian]=computer;

    nframes=floor(numBytes/64); % Steim compression is made up of 64-byte frames
    writeBlock=zeros(1,16*nframes,'int32'); i_w=1;
    %% One 64-Byte (16-word) frame at a time
    for i_f=1:nframes,
        code=uint32(0); % Holds compression information in 2-bit nibbles
        frame=zeros(1,16,'int32');
        dStart=2;
        if i_f==1,
            dStart=4;
        end
        if id<=nd,
            for i=dStart:16,
                if id+3 <= nd,
                    dblock=d(id:id+3);
                    if all(dblock>=-128) && all(dblock<=127),
                        code=bitset(code,32-2*i+1);   % 01b
                        %fprintf('%8d ',dblock); fprintf('\n');
                        %fprintf('%08X ',typecast(dblock,'uint32')); fprintf('\n');
                        t=cast(d(id:id+3),'int8');
                        frame(i)=typecast(t,'int32'); % could be byte-swapped, use swapbytes??
                        if endian=='L',
                            frame(i)=swapbytes(frame(i)); 
                        end
                        %frame(i)
                        id=id+4;
                        continue;
                    end
                end
                if id+1 <= nd,
                    dblk=d(id:id+1);
                    if all(dblk>=-32768) && all(dblk<=32767) 
                        code=bitset(code,32-2*i+2);                    %10b
                        t=cast(dblk,'int16');

                        if endian=='L',
                            frame(i)=typecast(swapbytes(t),'int32');
                            frame(i)=swapbytes(frame(i)); 
                        else
                            frame(i)=typecast(t,'int32');
                        end
                        id=id+2;
                        continue;
                    end
                end
                if id <=nd,
                    code=bitset(code,32-2*i+1);code=bitset(code,32-2*i+2);  %11b
                    frame(i)=cast(d(id),'int32');
                    id=id+1;
                end
            end
        end

        frame(1)=code;
        writeBlock(i_w:i_w+15) = frame;
        i_w=i_w+16;
        if DEBUG>1 && i_f==1,
            fprintf('frame %d:',i_f); 
            fprintf('\ncode=%08X=',code);
            for i=1:16,fprintf('%8d ',bitand(3,bitshift(code,-32+2*(i)))); end
            fprintf('\nwords    ='); fprintf('%08X ',typecast(frame,'uint32')); fprintf('\n\n');
        end
    end
    writesamps=id-1;
    writeBlock(2)=data(1);
    writeBlock(3)=data(writesamps);
end