function filename = writeSAC(tdata,outfname)
% Write a ts_data object to a SAC data file or files
% writeSAC(tdata) writes a SAC data file for each tdata channel
% 	fName = ts_writeSAC(tdata) creates standard SAC filenames using the start time & channel #
%	fName = ts_writeSAC(tdata,outfilename) uses outfilename, appends '.sac'
%           (if tdata has multiple channels, will also insert the channel name
%           in each filename)
%	

% header names & values come from the PASSCAL program segy2sac.c (v98.133)
%	and its header sac.h
%
% W Crawford 3/1999

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% Standard beginning for ts_* functions %%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ts_debug(1,'Starting...'); tic;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% First get universal (channel-independent) parameters
ITIME = 1;
IB = 9;
TRUE = 1;
FALSE = 0;
DEFNUM = -12345;
DEFSTR8 = '-12345  ';
DEFSTR16 = '-12345          ';
%FLT_MAX = 3.402823466E+38;
floatdef = 'float32';
longdef = 'int32';
chardef = 'uchar';
numchans = tdata.nChannels;
%evtime = get(tdata,'eventtime');
%[evlat,evlon,evdepth,evwdepth] = get(tdata,'eventloc');

oldfilenames={};
for ich = 1:numchans,
	ts_debug(1,'channel %d',ich);
    chan=tdata.channel(ich);
    samprate = chan.samprate;
    numsamps = chan.nSamples;
	stime = chan.starttime;	% data start time
	evtoffset = 0; %evtime - stime;		% seconds
	stlat=chan.metadata.lat;
    stlon=chan.metadata.lon;
    stelev=chan.metadata.elev;
    stwdepth=chan.metadata.wdepth; 
	network= chan.metadata.network;
	stationname=chan.metadata.station;
	chname = chan.metadata.component;
	data = chan.data;

	if exist('outfname','var'),
        [path,name,ext]=fileparts(outfname);
        if ~strcmp(lower(ext),'.sac'),
            name=[name ext];
            ext='.sac';
        end
		if numchans==1, 
            filename=[fullfile(path,name) ext];
		elseif numchans>1, 
			filename=sprintf('%s.%s%s',fullfile(path,name),strtrim(chname),ext);
		end
    else
        writeTime=round(stime); % Round to the nearest second
		filename = sprintf('%s.%s.%s.sac', ...
							writeTime.char('%y%j.%H%M'), ...
							strtrim(stationname),strtrim(chname));
		if strmatch(filename,oldfilenames,'exact'),
			filename = strrep(filename,'.sac',sprintf('%d.sac',ich));
		end
		oldfilenames{ich}=filename;
	end
	ts_verbose(0,'Writing SAC file %s',filename); % Always write (unless ordered to shut up)
	fid = fopen(filename,'w','ieee-be');
	if fid==-1,
		error('File %s could not be opened for writing',filename);
	end
	ts_debug(1,'Writing header');
	% Write the SAC header
	fwrite(fid,1/samprate,floatdef);				% float   delta
	fwrite(fid,min(data),floatdef);					% float   depmin
	fwrite(fid,max(data),floatdef);					% float   depmax
	fwrite(fid,DEFNUM,floatdef);					% float   scale
	fwrite(fid,DEFNUM,floatdef);					% float   odelta
	fwrite(fid,0.0,floatdef);						% float   b
	fwrite(fid,numsamps/samprate,floatdef);			% float   e
	fwrite(fid,DEFNUM,floatdef);					% float   o
	fwrite(fid,evtoffset,floatdef);					% float   a
	fwrite(fid,DEFNUM,floatdef);					% float   internal1
	fwrite(fid,DEFNUM*ones(1,10),floatdef);			% float   t0 - t9
	fwrite(fid,DEFNUM,floatdef);					% float   f
	fwrite(fid,DEFNUM*ones(1,10),floatdef);			% float   resp0 - resp9
	fwrite(fid,stlat,floatdef);						% float   stla
	fwrite(fid,stlon,floatdef);						% float   stlo
	fwrite(fid,stelev,floatdef);					% float   stel
	fwrite(fid,stwdepth,floatdef);					% float   stdp
	fwrite(fid,0,floatdef);						% float   evla
	fwrite(fid,0,floatdef);						% float   evlo
	fwrite(fid,0,floatdef);                     % float   evel
	fwrite(fid,0,floatdef);                     % float   evdp
	fwrite(fid,DEFNUM,floatdef);					% float   unused1;
	fwrite(fid,DEFNUM*ones(1,10),floatdef);			% float   user0 - user9
	fwrite(fid,DEFNUM,floatdef);					% float   dist
	fwrite(fid,DEFNUM,floatdef);					% float   az
	fwrite(fid,DEFNUM,floatdef);					% float   baz
	fwrite(fid,DEFNUM,floatdef);					% float   gcarc
	fwrite(fid,DEFNUM,floatdef);					% float   internal2
	fwrite(fid,DEFNUM,floatdef);					% float   internal3
	fwrite(fid,DEFNUM,floatdef);					% float   depmen
	fwrite(fid,DEFNUM,floatdef);					% float   cmpaz
	fwrite(fid,DEFNUM,floatdef);					% float   cmpinc
	fwrite(fid,DEFNUM*ones(1,11),floatdef);			% float   unused2 - unused22
	fwrite(fid,stime.year,longdef);					% long    nzyear
	fwrite(fid,stime.jday,longdef);					% long    nzjday
	fwrite(fid,stime.hour,longdef);					% long    nzhour
	fwrite(fid,stime.minute,longdef);					% long    nzmin
	fwrite(fid,floor(stime.second),longdef);				% long    nzsec
	msecs = floor(1000* (stime.second-floor(stime.second)) );
	fwrite(fid,msecs,longdef);						% long    nzmsec
	fwrite(fid,6,longdef);							% long    internal4	(LLL something)
	fwrite(fid,0,longdef);							% long    internal5	(LLL something)
	fwrite(fid,0,longdef);							% long    internal6	(LLL something)
	fwrite(fid,numsamps,longdef);					% long    npts;
	fwrite(fid,DEFNUM,longdef);						% long    internal7
	fwrite(fid,DEFNUM,longdef);						% long    internal8
	fwrite(fid,DEFNUM*ones(1,3),longdef);			% long    unused13 - unused15
	fwrite(fid,ITIME,longdef);						% long    iftype
	fwrite(fid,DEFNUM,longdef);						% long    idep
	fwrite(fid,IB,longdef);							% long    iztype
	fwrite(fid,DEFNUM,longdef);						% long    unused16
	fwrite(fid,DEFNUM,longdef);						% long    iinst
	fwrite(fid,DEFNUM,longdef);						% long    istreg
	fwrite(fid,DEFNUM,longdef);						% long    ievreg
	fwrite(fid,DEFNUM,longdef);						% long    ievtyp
	fwrite(fid,DEFNUM,longdef);						% long    iqual
	fwrite(fid,DEFNUM,longdef);						% long    isynth
	fwrite(fid,DEFNUM*ones(1,10),longdef);			% long    unused17 - unused26;
	fwrite(fid,TRUE,longdef);						% long    leven
	fwrite(fid,FALSE,longdef);						% long    lpspol	(LLL something)
	fwrite(fid,DEFNUM,longdef);						% long    lovrok
	fwrite(fid,TRUE,longdef);						% long    lcalda	(LLL something)
	fwrite(fid,FALSE,longdef);						% long    unused27	(LLL something)
	tstr = sprintf('%-8s',deblank(stationname));
	fwrite(fid,tstr(1:8),chardef);					% char    kstnm[8]
	fwrite(fid,DEFSTR16,chardef);					% char    kevnm[16];
	fwrite(fid,DEFSTR8,chardef);					% char    khole[8];
	fwrite(fid,DEFSTR8,chardef);					% char    ko[8];
	fwrite(fid,DEFSTR8,chardef);					% char    ka[8];
	for i=1:10,fwrite(fid,DEFSTR8,chardef);end		% char    kt0[8] - kt9[8];
	fwrite(fid,DEFSTR8,chardef);					% char    kf[8]
	for i=1:3,fwrite(fid,DEFSTR8,chardef);end		% char    kuser0[8] - kuser2[8]
	ts_debug(1,'chname=%s',chname);
	tstr = sprintf('%-8s',chname);
	fwrite(fid,tstr(1:8),chardef);					% char    kcmpnm[8]
	tstr = sprintf('%-8s',network);
	fwrite(fid,tstr(1:8),chardef);					% char    knetwk[8]
	fwrite(fid,DEFSTR8,chardef);					% char    kdatrd[8]
	fwrite(fid,DEFSTR8,chardef);					% char    kinst[8]
	
	% WRITE OUT DATA
	ts_debug(1,'Writing data');
	count = fwrite(fid,data,floatdef);
	fclose(fid);
	if count ~= numsamps,
		error('Specified %.0f samples, wrote %.0f',numsamp,count);
	end
	ts_debug(1,'Done');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%% Standard end for ts_* functions %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ts_debug(1,'Exiting after %.1f seconds',toc);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end % end of function
