function [sgram]=calcSpectrogram(obj,windowlen,stepsize,varargin)
% Calculate single-channel spectrogram
% Usage:
%	sgram = calcSpectrogram(obj,windowlen,stepsize [,options]);
% Input:
%	obj         ts_data_channel object
%	windowlen  Length (seconds) for each spectra window
%	stepsize   number of seconds in each spectra calculation
%	options: optional field,value pairs
%		'starttime' time or seconds offset at which to start spectrogram
%					A negative offset is counted back from the end of data [0]
%		'endtime'   time (ts_datetime) or seconds offset at which to end spectrogram
%					A 0 or negative offset is counted back from the end of the data [0].
%		'taper'		taper to use for each window: ('prol1pi' | ['prol4pi'])
% Output:
%   sgram:   the spectrogram (a ts_spectrogram object)

% 5/2006 Wayne Crawford

%

%% Set up variables and read in arguments
ts_verbose(1,'Calculate/plot a spectrogram');
if ~exist('windowlen','var')
    windowlen=1024/obj.samprate;    % default window length is 1024 samples
end
if ~exist('stepsize','var')
    stepsize=5*windowlen;           % default step size is 5 windows
end
ts_verbose(0,'Will calculate spectra every %s',ts_datetime.seconds2str(stepsize));

dl = obj.nSamples;
st=0; et=dl/obj.samprate;		% Start and end addresses
taper='prol4pi';

if exist('varargin','var'),
	for i=1:2:length(varargin),
		field = varargin{i};
		value = varargin{i+1};
		switch lower(field),
			case 'starttime',
				st = value;
				if isa(st,'ts_datetime');
					st=st-obj.starttime;
				end
			case 'endtime',
				et = value;
				if isa(et,'ts_datetime');
					temp=et;
					et=et-obj.starttime;
					ts_verbose(1,'Changing end time to %s (%d seconds offset)\n',char(temp),et);
                else
					if et<=0, et=dl-et; end
					ts_verbose(1,'Changing end time to %d seconds offset\n',et);
				end
			case 'taper',
				taper = value;
			otherwise,
				error([field ': unknown field'])
		end
	end
end

ists = st:stepsize:et-stepsize+1;

%% Quiet down ts_spectra runs
DIALOGS = ts_get('dialogs');
if DIALOGS, ts_set('dialogs',0); end	% Don't let spectra make a waitbar, this will do it's own.
verboselevel = ts_get('verbose');
if verboselevel==1, ts_verbose(0); end	% Make spectra be a little quieter

%% Prepare ts_spectrogram object
% Run a spectrum once to get # of frequencies
mobj=ts_data(obj);  % create a ts_data object so that we can calc spect.
sd=mobj.calcSpectra('length',stepsize,'windowlength',windowlen);
sgram=ts_spectrogram(obj.metadata.fullname,sd.sinfo(1).units,sd.starttime,stepsize,length(ists),sd.freqs);

%% Calculate spectrogram
str = sprintf('Calculating %d spectra for spectrogram',length(ists)) ;
if length(ists) > 1 && DIALOGS,
	h_waitwind = waitbar(0,str);
elseif verboselevel>-1,
	fprintf('%s',str);
end
j=0;
for ist=ists,
	j=j+1;
	sd=mobj.calcSpectra('start',ist,'length',stepsize,'windowlen',windowlen, ...
					'taper',taper);
	sgram.data(j,:)=sd.spectdata;
	if length(ists) > 1 && DIALOGS,
		waitbar((j+1)/length(ists));
    elseif verboselevel>-1
		fprintf('.');
	end
end

%% SET DIALOGS AND VERBOSE BACK TO ORIGINAL VALUES
if length(ists) > 1 && DIALOGS,
	close(h_waitwind);
	ts_set('dialogs',1);
elseif verboselevel>-1
	fprintf('Done\n');
end
ts_verbose(verboselevel);

return
