function [axhand,linehand] = plot(obj,start,plotlength,figureprops,xlabelit,varargin)
    %Plot one channel of time series data
    % plot(obj,start,length,figureprops,xlabelit,options) plots data versus time
    %	by default, plots the entire data
    %
    % Inputs:
    %   obj        is the ts_data_channel object to plot
    %   start [default=0] is a date and time (ts_datetime or character string) or a number
    %           (seconds from the beginning of the data)
    %   length [default=0] is the number of seconds of data to plot (0 means
    %           plot to the end of data
    %   figureprops   is a ts_data_figureproperties object containing information about plotting
    %               if it's not specified, uses the value obtained using ts_get('figureprops')
    %   xlabelit   print the x label (1)
    %
    %	options are field, value pairs specified as in ts_data_figureproperties/set, plus:
    %       field:      value:                              Explanation
    %       'highlight' {slist,color,channel,text,line}     Highlight part of the data
    %                   where
    %                   slist =		Nx2 matrix of time windows to highlight
    %                               each row: [start_seconds end_seconds]
    %                   color =		highlight color
    %                   text =      text in upper right corner ['']
    %                   line =      Bounding box linetype [''];
    
    ts_debug(1,'Starting');
    
    % FILL ANY EMPTY INPUT VARIABLES
    if ~exist('start','var'); start=0; end
    if ~exist('plotlength','var'); plotlength=0; end
    if ~exist('figureprops','var'); figureprops=ts_get('figureprops'); end
    if ~exist('xlabelit','var'); xlabelit=1; end
    
    [start,plotlength,nodressing,hilite,obj] = verifyParms(start,plotlength,obj,varargin);
    
%     if ischar(start)
%         start=ts_datetime(start);
%     elseif ~isnumeric(start) && ~isa(start,'ts_datetime')
%         error('start is a %s: neither a datetime nor a number',class(start));
%     end
%     if ischar(plotlength)
%         plotlength = ts_datetime.str2seconds(plotlength);   % if a string, convert to seconds
%     end
%     
%     % READ OPTIONAL PARAMETERS
%     i = 1; hilite=[];
%     nodressing=0;   % For overlapped plots, do not add new ticks, grid, etc
%     ts_debug(1,'length(varargin)=%d',length(varargin));
%     while i < length(varargin),
%         field = varargin{i};
%         value = varargin{i+1};
%         i = i+2;
%         ts_debug(1,'field,value=''%s'',''%s''',field, value)
%         switch field,
%             case {'nodressing'}
%                 nodressing=value;
%             case {'highlight','highlights','hilite','hilites'}
%                 if ~isempty(value),
%                     if ~isa(value,'cell'), value={value}; end
%                     hilite.slist = value{1};
%                     if ischar(hilite.slist),
%                         hilite.slist=ap_slist(hilite.slist);
%                     end
%                     if ~isa(hilite.slist,'ts_seqlist'), error('Bad first element in highlights'); end
%                     lh=length(value);
%                     if lh >= 2, hilite.color=value{2}; else hilite.color='y'; end
%                     if lh >= 3, hilite.chan=value{3};  else hilite.chan=':'; end
%                     if lh >= 4, hilite.text=value{4};  else hilite.text='';  end
%                     if lh >= 5, hilite.line=value{5};  else hilite.line='';  end
%                 end
%             case 'figureprops',
%                 obj.figureprops=value;
%             otherwise,
%                 eval(['obj.subplot.' field '=value;']);
%         end % switch
%     end % while
%     
%     
    nS = obj.nSamples;
    samprate = obj.samprate;
    
    if isnumeric(start), cps=num2str(start); else cps=char(start); end
    ts_debug(1,'start=%s, plotlength=%.1fs',cps,plotlength);
    plotSamps = plotlength*samprate;
    refStartAddr = obj.calcAddress(start);
    if plotSamps < 1, plotSamps = nS-refStartAddr+1; end % "0" length means plot to end of data
    
    pi=obj.subplotprops;
    
    switch figureprops.decimtype,
        case 'regular', decim = round(figureprops.regdecim);
        case 'smart', decim = ceil(plotSamps/figureprops.smartdecim);
        otherwise, decim = 1;
    end
    
    
    fontsize =	figureprops.fontsize;
    linehand = [];
    
    if isa(start,'ts_datetime'),
        startaddr = calcAddress(obj,start);
    else
        startaddr = obj.calcAddress(start);	% FORCES ALL CHANNELS TO USE SAME START OFFSET
    end
%    leftStamp = sprintf('%d',refStartAddr);
%    rightStamp = sprintf('%d',refStartAddr+plotSamps-1);
    
    demux = 	pi.demultiplex;
    yrange = pi.yrange;
    calcyrange = isnan(yrange);
    miny = pi.ymin;
    maxy = pi.ymax;
    
    endaddr = startaddr+plotSamps-1;
    if startaddr > endaddr, error('starting address is greater than end address!'); end
    idata = round(startaddr:decim:endaddr);
    timearray = idata/samprate;
    timearray = timearray - min(timearray);
    %timerange = max(timearray) - min(timearray);
    minx=min(timearray); maxx=max(timearray);
    if isempty(figureprops.xticks)
        [xticks,xticklabels] = calcXTicks(minx,maxx);    % Not time synchronized/aligned, use relative times
        xlabelstr='Relative times';
    else
        xticks=figureprops.xticks;
        xticklabels=figureprops.xticklabels;
        xlabelstr=figureprops.xlabel;
    end
    if decim > 1,
        xlabelstr = [xlabelstr sprintf(' (Decimated by %d)',decim)];
    end
    
    % Get data, putting NaNs where there is no data (WCC 6/2005)
    nbef = length(find(idata<1));
    naft = length(find(idata>nS));
    %idata = idata(find(idata>=1 & idata <=numsamps));
    idata = idata((idata>=1) & (idata <=nS));
    %fprintf('%d...%f\n',idata(1),idata(end))
    %nS
    data = [nan*ones(1,nbef) obj.data(idata) nan*ones(1,naft)];
    
    filtparms = obj.metadata.response;
    switch pi.removegain,
        case 'on',
            units = filtparms.inUnits;
            data = data / filtparms.gain;
        otherwise,
            units = filtparms.outUnits;
    end
    %currtime = obj.calcTime(startaddr);
    
    ptimearray = timearray;
    
    % Calculate y range first, so we can use it for hilites
    if calcyrange,
        if isnan(miny), miny=min(data); end
        if isnan(maxy), maxy=max(data); end
        yrange=maxy-miny;
        if yrange <=0, yrange = 2; miny=miny-1; end
    else
        if yrange <=0, yrange = 2; end
        if isnan(miny), miny = mean(data)-yrange/2; end
    end
    maxy = miny+yrange;
    if miny ~=0,
        if abs((maxy-miny)/miny)<1e-9, maxy = 1.01*miny; end	% handles large miny
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%
    %% PLOTTING
    %%%%%%%%%%%%%%%%%%%%%%%%%%
    % Hilites first, so that they stay behind data
    if ~nodressing
        if ~isempty(hilite),
            if (hilite.chan) % Put a higlight on this channel
                for ih = 1:length(hilite.slist),
                    if (hilite.slist.starts(ih) <= endaddr/samprate) && (hilite.slist.ends(ih) >= startaddr/samprate),
                        boxx = [hilite.slist.starts(ih)*[1 1] hilite.slist.ends(ih)*[1 1] hilite.slist.starts(ih)]-startaddr/samprate;
                        boxy = [maxy miny*[1 1] maxy*[1 1]];
                        if ~isempty(hilite.color); h=fill(boxx,boxy,hilite.color);
                            set(h,'LineStyle','none'); hold on, end
                        if ~isempty(hilite.line), plot(boxx,boxy,hilite.line); hold on, end
                        if ~isempty(hilite.text),
                            text(hilite.slist.ends(ih) - (startaddr/samprate),maxy,hilite.text, ...
                                'HorizontalAlignment','right',...
                                'VerticalAlignment','top', ...
                                'FontSize',fontsize);
                        end
                    end
                end
            end
        end
    end
    
    % PLOT DATA
    if demux == 0,
        h = plot(ptimearray,data);
        marker = pi.marker;
        if strcmp(marker,'.'), markersize = 8; else markersize = 6; end
        set(h,'LineStyle',pi.linetype,'Marker', marker,'MarkerSize',markersize);
        linehand = [linehand; h];
    else
        colors = 'kbrgymc';
        for j = 0:demux-1,
            color = colors(rem(j,7)+1);
            isubdata = find(rem(idata,demux*ones(size(idata)))-j==0);
            h = plot(ptimearray(isubdata),data(isubdata),color); hold on
            set(h,'LineStyle',pi.linetype,'Marker', pi.marker);
            linehand = [linehand; h]; %#ok<AGROW>
        end
        hold off
    end
    %
    set(gca,'FontSize',fontsize);
    
    % ADD "DRESSING" (TICKS, ETC)
    if ~nodressing,
        if (isnan(miny) && isnan(maxy)), miny=-100; maxy=100; end
        axis([minx maxx miny maxy])
        set(gca,'XTick',xticks,'Layer','top');
%         text(0.01,0.99,leftStamp, 'Units','normalized', ...
%             'VerticalAlignment','top','HorizontalAlignment','left','Fontsize',8);
%         text(0.99,0.99,rightStamp ,'Units','normalized', ...
%             'VerticalAlignment','top','HorizontalAlignment','right','Fontsize',8);
        ylabelstr = [obj.metadata.station '.' obj.metadata.component];
        ylabelstr = sprintf('%s\n%s',obj.metadata.station,obj.metadata.component);
        if ~isempty(units), ylabelstr = [ylabelstr ' (' units ')']; end
        ts_debug(2,'ylabelstr="%s"',ylabelstr);
        ylabel(ylabelstr,'FontSize',fontsize,'Interpreter','None');
        yticks = get(gca,'YTick');
        labels=cell(length(yticks),1);
        for j=1:length(yticks),	labels{j} = sprintf('%6.5g',yticks(j)); end
        set(gca,'YTickLabel',labels);
        switch pi.grid
            case 'xy', grid on;
            case 'x', set(gca,'XGrid','on'), set(gca,'YGrid','off');
            case 'y', set(gca,'YGrid','on'), set(gca,'XGrid','off');
            otherwise, grid off
        end
    end
    
    % ADD X-AXIS LABELS
    if xlabelit,
        set(gca,'XTickLabel',xticklabels);
        xlabel(xlabelstr)
    else
        set(gca,'XTickLabel',[]);
    end
    
    axhand = gca;
    
    ts_debug(1,'Ending');
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [xticks,ticklabels] = calcXTicks(xmin,xmax)
    
    xrange = xmax-xmin;
    if xrange < 15;
        multfactor = 1;		divunits = 'seconds';
        basestep = 10^floor(log10(xrange));
        basesteps = basestep*[1 .5 .2 .1 .05];
    elseif xrange < 1.5*60;
        multfactor = 1;		divunits = 'seconds';
        basesteps = [30 15 10 5 3 1.5 1];
    elseif xrange < 1.5*60*60;
        multfactor = 60;		divunits = 'minutes';
        basesteps = [30 15 10 5 2 1 .5 .25];
    elseif xrange < 1.5*60*60*24;
        multfactor = 60*60;		divunits = 'hours';
        basesteps = [12 6 3 1.5 1 .5 .25];
    else
        multfactor = 60*60*24;		divunits = 'days';
        basestep = 10^floor(log10(xrange/multfactor));
        basesteps = basestep*[1 .5 .25 .1 .05];
    end
    % Step down into whatever units we're working in
    xrange = xrange/multfactor;
    istep=1;
    step = basesteps(istep);
    while xrange/step < 5 && istep<(length(basesteps)), % If less than 4 ticks, increase #
        istep=istep+1;
        step = basesteps(istep);
    end
    % now step back up and create the ticks
    step = step*multfactor;
    tickmin = step * ceil(xmin/step);
    tickmax = step * floor(xmax/step);
    xticks = tickmin:step:tickmax;
    
    switch divunits,
        case 'seconds',
            tvals=xticks;
            tchar='s';
        case 'minutes',
            tvals=xticks/60;
            tchar='m';
        case 'hours',
            tvals=xticks/3600;
            tchar='h';
        case 'days',
            tvals=xticks/86400;
            tchar='d';
    end
    
    formstr = sprintf('%%g%s',tchar);
    ticklabels=cell(length(tvals),1);
    for i = 1:length(tvals),
        ticklabels{i}= sprintf(formstr,tvals(i));
    end
end

function [start,plotlength,nodressing,hilite,obj] = verifyParms(start,plotlength,obj,vin)
    hilite=[];
    
    if ischar(start)
        start=ts_datetime(start);
    elseif ~isnumeric(start) && ~isa(start,'ts_datetime')
        error('start is a %s: neither a datetime nor a number',class(start));
    end
    if ischar(plotlength)
        plotlength = ts_datetime.str2seconds(plotlength);   % if a string, convert to seconds
    end
    
    % READ OPTIONAL PARAMETERS
    i = 1;
    nodressing=0;   % For overlapped plots, do not add new ticks, grid, etc
    ts_debug(1,'length(vin)=%d',length(vin));
    while i < length(vin),
        field = vin{i};
        value = vin{i+1};
        i = i+2;
        ts_debug(1,'field,value=''%s'',''%s''',field, value)
        switch field,
            case {'nodressing'}
                nodressing=value;
            case {'highlight','highlights','hilite','hilites'}
                if ~isempty(value),
                    if ~isa(value,'cell'), value={value}; end
                    hilite.slist = value{1};
                    if ischar(hilite.slist),
                        hilite.slist=ap_slist(hilite.slist);
                    end
                    if ~isa(hilite.slist,'ts_seqlist'), error('Bad first element in highlights'); end
                    lh=length(value);
                    if lh >= 2, hilite.color=value{2}; else hilite.color='y'; end
                    if lh >= 3, hilite.chan=value{3};  else hilite.chan=':'; end
                    if lh >= 4, hilite.text=value{4};  else hilite.text='';  end
                    if lh >= 5, hilite.line=value{5};  else hilite.line='';  end
                end
            case 'figureprops',
                obj.figureprops=value;
            otherwise,
                eval(['obj.subplot.' field '=value;']);
        end % switch
    end % while
end