classdef ts_data_channel
    % PRIMARY TISKIT CLASS: Store and manipulate one time series 
    %
    % Each channel can have its own start time and sampling rate.
    % Data are uniformly spaced along the time axis
    % In Tiskit, multiple channels are combined in a ts_data object
    %
    % 
    %
    % ts_data_channel properties:
    %   samprate      - Samples per second                    (double)
    %   starttime     - Time of first sample                  (ts_datetime)
    %   data          - Data storage                          (double vector)
    %   metadata      - Includes station, component ...       (ts_data_channel_metadata)
    %   info            - Alias for "metadata"
    %   sourcefile    - Information about source data file    (ts_file)
    %   subplotprops  -                                       (ts_figureproperties_subplot)
    %   sampinterval  - READONLY: Interval between samples    (double)
    %                            (seconds)  ==1/samprate;
    %   nSamples      - READONLY: Number of samples in data   (double)
    %   nSeconds      - READONLY: Number of seconds in data   (double)
    %   endtime       - READONLY: time of last sample         (ts_datetime)
    %                             (NOT DRIFT CORRECTED)
    %   endtime_dc    - READONLY: time of last sample         (ts_datetime)
    %                             (DRIFT CORRECTED)
    % 
    % ts_data_channel methods:
    %   Creator:
    %      ts_data_channel      - Creator
    %   Data modifiers:
    %      demean          - Remove mean of data
    %      detrend         - Remove mean plus trend
    %      filter          - Butterworth filter
    %      resample        - Resample data
    %      trueAmplitude   - Correct for instrument response
    %   Information:
    %      calcNumSamps    - Calculate the samples in a given number of seconds, hours, etc
    %      calcOffset      - Calculate time offset between two ts_data_channel objects  
    %      calcTime        - Calculate the time at a given sample number
    %      isempty         - Is data vector is empty?
    %      isConsecutive   - Returns 1 if two data channels are consecutive in time
    %      isNotConsecutive - Returns 1 if two ts_data_channel objects are NOT consecutive
    %      overLapSeconds  - Return overlap between two channels, in seconds
    %      overLapSamples  - Gives overlap between two channels, in samples
    %   Transformation
    %      cat             - Concatenate two ts_data_channel objects
    %      cut             - Return a section of the data
    %      calcSpectrogram - Make a spectrogram
    %   Redefine Matlab methods:
    %      plot            - Plot a data channel
    %
    % EXAMPLE
    % sps=40; t=0:1/sps:100; x=sin(t/5);
    % d=ts_data_channel(x,sps);
    % c.metadata.station='Napoli';
    % c.metadata.component='BHZ';
    % c.starttime='2007/01/01 00:00:00';
    % plot(c);
    % c
    
    properties        
        samprate=[];                            % Samples per second      
        starttime=ts_datetime;                  % Exact time of first sample       
        data=[];                                % Data storage (double vector)        
        metadata=ts_data_channel_metadata();    % Includes station, component ...         
        subplotprops = ts_get('subplotprops');  % (ts_figureproperties_subplot object)
    end
    properties (Dependent=true)
        
        sourcefile     % Information about source data file
        sampinterval   % Interval between samples (seconds)  ==1/samprate;
        nSamples       % Number of samples in data
        nSeconds       % Number of seconds in data
        endtime        % Time of last sample in channel (NOT DRIFT CORRECTED)
        endtime_dc     % Time of last sample in channel (DRIFT CORRECTED)
        info           % Alias for "metadata"
    end
    properties (Access=private)
        PrivateSourcefile=ts_file();    % Private so that can set other properties
                                        % when setting sourcefile.  Dependent
                                        % property "sourcefile" is its public face
    end
    
    %% CREATOR METHOD
    %
    %   obj=ts_data_channel(data,samprate)
    %   obj=ts_data_channel(data,ts_file)
    %
    methods
        function obj=ts_data_channel(data,arg2)
            % Create a ts_data_channel object
            %
            % Usage:
            %   obj=ts_data_channel(data,samprate) % Sets only data and sampling rate
            %       -or-
            %   obj=ts_data_channel(data,ts_file)  % Copies over all the parameters in the ts_file
            %                                 % ts_file.readchans must be an integer,  which
            %                                 % tells ts_data_channel which metadata to use
            %
            % Example
            %      tchan = ts_data_channel(sin([1:200]/20),10);
            ts_init ;	% Sets up global variables
            if nargin==0; return; end   % Handle zero-argument case
            if nargin~=2, error('ts_data_channel() needs two arguments, the data and the sampling rate'); end
            if isnumeric(arg2) && length(arg2)==1,
                obj.samprate=arg2;
            elseif isa(arg2,'ts_file');
                obj.sourcefile=arg2;  % set.sourcefile also sets the related obj properties
            else
                error('samprate must be a single numerical value or ts_file object');
            end
            if isnumeric(data)
                [M,N]=size(data);
                if M==1, obj.data=data;
                elseif N==1, obj.data=data';
                else error('data is %dx%d, must be a row or column vector',M,N);
                end
            else
                error('data is not numeric: type %s',class(data));
            end
        end
        
        %% METHODS THAT MODIFY THE DATA
        %
        %   filter(obj,ftype,npoles,lfreq,hfreq) % Butterworth filter
        %   resample(obj,newsamprate)            % Resample data
        %   trueAmplitude(obj,minfreq)           % Correct for instrument response
        %   demean(obj)                          % Remove mean of data
        %   detrend(obj,N)                       % Remove mean plus linear (or higher order) trend
        %
        obj = filter(obj,ftype,npoles,lfreq,hfreq); % DEFINED IN FILE
        [obj,leftover] = resample(obj,newsamprate); % DEFINED IN FILE
        c = xcorr(a,b, option);                     % DEFINED IN FILE
        function obj = trueAmplitude(obj,minfreq)
            %Convert data to true amplitude
            %
            % Uses the object's stored instrument response
            % Usage:
            %	obj = trueAmplitude(obj);
            %	 -or-
            %	obj = trueAmplitude(obj,minfreq);
            %
            %	Sets new instrument response to unity
            %	Convolves the data with the inverse fft of the filter parameters.
            %	If minfreq isn't specified, the ifft is calculated for frequencies
            %   down to 1/2000 of the sampling rate.
            
            % 2/2006 Wayne Crawford
            
            if nargout ==0, error('Must have an output argument'); end
            ts_verbose(1,'Converting data to True Amplitude.');
            sr = obj.samprate;
            datalen = obj.nSamples;
            if exist('minfreq','var'); numFreqs=ceil((sr/2)/minfreq);
            else numFreqs=1000; end
            
            freqs = (1:numFreqs)*(sr/(2*numFreqs));
            oldResp = obj.metadata.response;
            d = obj.data;
            
            % Calculate the convolution operator to apply to the channel
            filtresp = oldResp.response(freqs).^(-1);
            filtresp2side = [0 filtresp fliplr(conj(filtresp(1:end-1)))];
            ifilter = (fftshift(ifft(filtresp2side)));	% Time-domain equivalent of the transfer function
            filtlen = length(ifilter);
            offset = filtlen/2;
            iadds = offset+1:datalen+offset;
            
            % Remove mean and trend from original data. Reduces low freq noise
            x=1:length(d); p=polyfit(x,d,1); d = d-polyval(p,x);
            
            truAmpl = conv(d,ifilter);		% Result is length(drive0)+length(ifilter)-1
            truAmpl = truAmpl(iadds)-mean(truAmpl(iadds));	% Extract only the original times
            
            obj.data = real(truAmpl);
            units = oldResp.inUnits;
            obj.metadata.response=ts_response('PZ',units,units,1,[],[]);
            
            ts_verbose(1,'Done');
        end %trueAmplitude
        function obj = demean(obj)
            % Remove mean of data
            obj.data=obj.data-mean(obj.data);
        end
        function obj = detrend(obj,N)
            % Remove mean and linear (or higher order) trend
            % Usage: obj=detrend(obj)   % removes mean and linear trend
            %        obj=detrend(obj,N) % removes trend up to order N
            %        obj=detrend(obj,1) % same as detrend(obj)
            if ~exist('N','var'); N=1; end
            x=1:length(obj.data);
            P=polyfit(x,obj.data,N);
            obj.data=obj.data-polyval(P,x);
        end
        %% ACTION METHODS
        %
        %   plot(obj,...)                     % Plot data
        %   calcNumSamps(obj,plotLength)      % Calculate the samples in a given number of seconds, hours, etc
        %   calcSpectrogram(obj,windowlen,stepsize,...) % Make a spectrogram
        %   cut(obj,start,len,startcriteria,endcriteria)% Return a section of the data
        %   cat(obj,objB)                     % Concatenate two data channels
        %
        function [numSamps,suffix] = calcNumSamps(obj,plotLength)
            %Calculate the samples in a given number of seconds, hours, etc
            %
            % Usage: numSamps = calcnSamples(tdata,plotLength)
            % Input:
            %	tdata	  ts_data object
            %	plotLength character array with a number terminated by 's' (seconds),
            %	          'm' (minutes), 'h' (hours) or 'd' (days). If there is no
            %	          termination, the value is assumed to be in seconds
            %				If a scalar, assumed to be in seconds
            %
            %	Doesn't check to make sure that we don't go beyond the end of data
            %
            %	returns -1 if the input is screwy
            %
            % Output:
            %   the number of samples
            suffix = '';
            sampfreq = obj.samprate;
            if isa(plotLength,'char'),
                [A] = sscanf(plotLength,'%f %c');
                if length(A) > 1,
                    switch char(A(2)),
                        case 'm', numSamps = round(A(1) * sampfreq*60); suffix = 'm';
                        case 'h', numSamps = round(A(1) * sampfreq*3600); suffix = 'h';
                        case 'd', numSamps = round(A(1) * sampfreq*86400); suffix = 'd';
                        case 'p', numSamps = round(A(1)); suffix = 'p';
                        case 's', numSamps = round(A(1) * sampfreq);	 suffix = 's';
                        otherwise, ts_warning ('bad plot Length'); numSamps = -1;
                    end
                else numSamps = round(A(1))* sampfreq;
                end
            else numSamps = plotLength* sampfreq;
            end
            ts_debug(1,'ts_data/calcNumSamps: %s = %d samps',plotLength,numSamps);
        end
        [axhand,linehand] = plot(obj,varargin); % DEFINED IN FILE
        [sgrams,info]=calcSpectrogram(tdata,windowlen,stepsize,varargin); % MAKE A SPECTROGRAM
        obj=cut(obj,start,len,startcriter,endcriter); % DEFINED IN FILE
        function obj=cat(obj,obj2)
            % Concatenate two data channels
            %   fails if the offset between the data is more than the data length
            %   in the first objet
            %disp('ENTERING CAT')
            maxsampratediff=1e-4;   % Maximimum allowable difference in sample rate
            epsilon=1e-2;           % Maximum gap/overlap to completely ignore (in samples)
            if abs(obj.samprate - obj2.samprate)/obj.samprate > maxsampratediff,
                error('Cannot concatenate objects with sampling rates off by more than %g%%',100*maxsampratediff);
            end
            if obj2.starttime==ts_datetime(),    % time not set
                obj2.starttime=calcTime(obj,1,obj.nSamples+1);
            end
            if obj2.starttime <= obj.starttime
                error('The second data starts BEFORE the first!')
            end
            timea = calcTime(obj,obj.nSamples);
            timeb = calcTime(obj2,1);
            timediff = timeb-timea;
            dgap = timediff*obj.samprate - 1;
            % Inform user of gap/overlap if greater than epsilon samples
            if abs(dgap)>epsilon,
                if dgap > 0,
                    ts_verbose(0,'%g sample excessive gap between data',dgap);
                elseif dgap<0,
                    ts_verbose(0,'%g sample overlap between data',-dgap);
                end
            end
            dgap = round(dgap);
            if dgap> obj.nSamples, error('Gap between data is more than nSamples (%d)',obj.nSamples); end
            
            % Concatenate data
            totsamps = obj.nSamples+obj2.nSamples+dgap;
            newdata = zeros(1,totsamps);	% Predefine vector, already has zeros for gap (if any)
            istart2=totsamps-obj2.nSamples+1;
            % Put first data at beginning
            newdata(1:obj.nSamples) = obj.data;
            % Put second data at appropriate position
            newdata(istart2 : totsamps) = obj2.data;
            
            obj.data=newdata;
        end
        %% INFORMATION METHODS
        %
        %   isempty(obj)                      % Returns 1 if data vector is empty
        %   overLapSeconds(obj,objB)          % Returns overlap between two channels, in seconds
        %   overLapSamples(obj,objB)          % Gives overlap between two channels, in samples
        %   isConsecutive(obj,objB,tolerance) % Returns 1 if two data channels are consecutive
        %   isNotConsecutive(obj,objB)        % Returns 1 if two data channels are NOT consecutive
        %   cat(obj,objB)                     % Concatenate two data channels
        %
        function out=isempty(obj)
            % Returns 1 if the data array is empty
            out=isempty(obj.data);
        end
        function out=overLapSeconds(objA,objB)
            % returns offset in seconds between perfect data overlap
            % objA is assumed to precede objB
            %   Positive values mean the start of objB is later than expected
            %   Negative values mean the start of objB is earlier than expected
            out=objB.starttime-objA.endtime_dc-objA.sampInterval;
        end
        function out=overLapSamples(objA,objB)
            % returns offset in samples from perfect data overlap
            % objA is assumed to precede objB
            %   Positive values mean the start of objB is later than expected
            %   Negative values mean the start of objB is earlier than expected
            % NaN means objA and objB have different sample rates, or
            % 	objB starts before objA
            if objA.samprate~=objB.samprate,
                ts_warning('the two channels have different sample rates');
                out=NaN;
                return;
            end
            if objB.startime < objA.starttime
                ts_warning('the second channel starts before the first');
                out=NaN;
                return;
            end
            out=overLapSeconds(objA,objB)*objA.samprate;
        end
        
        function out=isConsecutive(obj,obj2,tolerance)
            % Returns 1 if data have same samplerate and obj2 starts 1
            % sample after obj1
            % tolerance: tolerance for gap (in samples) [0.5]
            
            if ~exist('tolerance','var'); tolerance = 0.5; end
            if tolerance < 0, error('tolerance cannot be less than 0'); end
            if tolerance > 100, ts_warning('you chose a tolerance > 100 samples!'); end
            % Returns 1 if data are consecutive in time, 0 if they are not or we can't tell (e.g., if starttime wasn't set)
            if obj.starttime==ts_datetime() || obj2.starttime==ts_datetime()    % Not consecutive if time is unset
                ts_verbose(0,'At least one object starttime is not a datetime, can not determine if consecutive');
                out=0; return;
            end
            if abs(obj.samprate-obj2.samprate)/obj.samprate >0.01, out=0; return; end  % Not consecutive if diff samp rates
            timediff = calcTime(obj2,1)-calcTime(obj,obj.nSamples);
            dgap = timediff*obj.samprate;    % # of data samples off in gap
            if dgap~=1,
                ts_verbose(2,'%g-sample gap between data',dgap);
            end
            out = abs(dgap-1)<tolerance;
        end
        
        function out=isNotConsecutive(obj,obj2)
            % Returns 1 if data are not consecutive in time, 0 if they are or we can't tell
            if obj.starttime==ts_datetime() || obj2.starttime==ts_datetime()
                out=0; return;
            end
            out=~isConsecutive(obj,obj2);
        end
        
        %% REPLACEMENT METHODS
        %
        %   end(obj,k)                        % Implements 'end' in .() notation
        %   char(obj,padding);                % Returns character string describing the object
        %   disp(obj)                         % Called when you type the object without a semicolon
        %
        function iend = end(obj,k)
            %Implement use of 'end' in .() notation
            if k == 1,
                iend = obj.nChannels;
            else
                iend = obj.nSamples;
            end
        end
        function otxt=char(obj,padding)
            % Make character string describing the object
            % padding pads the left side, to give an offset
            if ~exist('padding','var'), padding=0; end
            pformat=sprintf('%%%ds',padding);
            pad=sprintf(pformat,'');
            
            otxt=      sprintf('%s%15s: %g\n',                 pad,'samprate',obj.samprate);
            otxt=[otxt sprintf('%s%15s: %s (''%s'' object)\n',pad,'starttime',char(obj.starttime),class(obj.starttime))];
            otxt=[otxt sprintf('%s%15s: %d\n',           pad,'nSamples',obj.nSamples)];
            otxt=[otxt sprintf('%s%15s: ''%s'' object\n',pad,'info',class(obj.metadata))];
            otxt=[otxt sprintf('%s\n',                 char(obj.metadata,15+padding))];
            otxt=[otxt sprintf('%s%15s: ''%s'' object\n',pad,'sourcefile',class(obj.sourcefile))];
            %otxt=[otxt sprintf('%s',                   '',char(obj.sourcefile,15+padding))];
        end
        function disp(obj)
            % Displays object properties when object is typed without semicolon
            disp(char(obj));
        end
        
        % SET/GET METHODS
        function obj = set.samprate(obj,val)
            if length(val)>1, error('samprate must be single numeric value'); end
            obj.samprate=val;
            obj.metadata.component=ts_data_channel.changeComp(obj.metadata.component,val);  % Change component name if necessary
        end
        function obj = set.starttime(obj,val)
            val=ts_datetime(val);   % Converts strings, verifies the rest
            obj.starttime=val;
        end
        function obj = set.data(obj,val)
            if ~isnumeric(val), error('value must be numeric'); end
            [M,N]=size(val);
            if M~=1,
                if N==1, val=val';  % transpose if it was a column matrix
                else error('value must be a row or column vector');
                end
            end
            obj.data=val;
        end
        function obj = set.metadata(obj,value)
            if ~isa(value,'ts_data_channel_metadata')
                error('value is not a ts_data_channel_metadata object')
            end
            obj.metadata=value;
        end
        function obj = set.subplotprops(obj,val)
            if ~isa(val,'ts_figureproperties_subplot'), 
                error('value must be a ts_figureproperties_subplot object'); 
            end
            obj.subplotprops=val;
        end
        
        % SET/GET METHODS (FOR DEPENDENT VARIABLES)
        function sfile = get.sourcefile(obj)
            sfile=obj.PrivateSourcefile;
        end
        function obj = set.sourcefile(obj,finfo)
            % Reads ts_file into obj.sourcefile and directly copies metadata
            % MATLAB doesn't like me coping directly into another property
            if ~isa(finfo,'ts_file'); error('value must be a ts_file object'), end
            i=finfo.readchans;
            if length(i)~=1,
                error('when setting a ts_data_channel''s sourcefile parameter, the passed ts_file readchans should be a scalar');
            end
            obj.PrivateSourcefile=finfo;
            %finfo   % DEBUG
            
            %if ~isempty(finfo.samprate); obj.samprate=finfo.samprate; end
            if ~isempty(finfo.ch_metadata(i)); obj.metadata=finfo.ch_metadata(i); end
        end
        function out =  get.sampinterval(obj)
            out=1/obj.samprate;
        end
        function ns =  get.nSamples(obj)
            ns=length(obj.data);
        end
        function ns =  get.nSeconds(obj)
            ns=obj.nSamples/obj.samprate;
        end
        function out = get.endtime(obj) % Time of last sample (not drift corrected)
            out=obj.starttime+(obj.nSamples-1)/obj.samprate;
        end
        function out = get.endtime_dc(obj) % Drift corrected time of last sample
            out=obj.GPSTime(obj.endtime);
        end
        function obj =  set.info(obj,value)
            obj.metadata=value;
        end
        function out =  get.info(obj)
            out=obj.metadata;
        end
    end % Public methods
    
    %% STATIC METHODS
    % Called using ts_data_channel.methodname
    %
    %   calcOffset(timeOffset,driftRate,samprate)  
    %
    methods (Static=true) % Does not require an object argument. Call using ts_data_channel.methodname
        function addrOffset= calcOffset(timeOffset,driftRate,samprate)
            % Return the address offset equalling a given time offset
            %  , for the specified drift rate.
            %
            % Usage: caddrOffset= calcAddrOffset(timeOffset,driftRate,samprate)
            %
            % Inputs:
            %	timeOffset: time (in seconds from current position)
            %	driftRate is the instrument drift rate:   (instElapsed-GPSElapsed)/GPSElapsed
            %   sampRate is the instrument sampling rate (samples/second)
            % Output:
            %	addrOffset: the address offset from the current posision
            obj=ts_data_channel;
            obj.metadata.driftRate=driftRate;
            tz=ts_datetime();
            obj.metadata.timezero=tz;
            %mytime=;
            
            instTimeOffset = obj.driftCorrect(tz+timeOffset,'GPStoInst')-tz; % Convert to UNCORRECTED seconds
            %instTimeOffset = ts_datatime.driftCorrecSecs(timeOffset,driftRate,'backwards'); % Convert to UNCORRECTED seconds
            addrOffset = round(instTimeOffset*samprate);
        end
        c= xcorr_static(a,b,option);   % DEFINED IN SEPARATE FILE
            % The old Matlab Signal Toolbox XCORR function
        cmp=changeComp(cmp,newsamprate)
        
    end % static methods
    
    %% PRIVATE METHODS
    % Accessed by class members only.  Use only if you are changing a
    % TiSKit class
    %
    %   obj=cut_addr(obj,istart,iend) 
    %   currtime = calcTime(obj,address)
    %   [address,delay] = calcAddress(obj,time)
    %   addrList = makeAddressList(obj,addressString,addBuffer,option)
    %
    methods (Access=private) % Access by class methods only
        obj=cut_addr(obj,istart,iend)  % DEFINED IN FILE
        function currtime = calcTime(obj,address)
            % Calculate the time at a given address on a given channel
            %
            % currtime = calcTime(obj,address);
            % where address the address of the current sample if we assume that address 1
            %   equals the starttime
            %
            % Assumes that obj.starttime corresponds to the CORRECT (drift
            % corrected) time at address 1
            
            % NEW WAY USES TS_FILE (WCC 02/2011)
            insttime=obj.starttime+(address-1)/obj.samprate;
            temp=obj.metadata;
            temp.timezero=obj.starttime;
            currtime=temp.GPSTime(insttime);
            
        end
        function [address,delay] = calcAddress(obj,time,comparator)
            % Return the data address closest to the specified time
            %
            % Usage: [address, delay] = calcAddress(obj,time,comparator);
            % Input:
            %   time    ts_datetime object giving the time to look for (will drift correct)
            %			-or-
            %			scalar giving seconds from start of data (won't drift correct)
            %   comparator ['='] specifies which address to select:
            %           '=':  the closest address [default]
            %           '>':  the closest address AFTER the time
            %           '>=': the closest address EQUAL TO or AFTER the time
            %           '<':  the closest address BEFORE the time
            %           '<=': the closest address EQUAL TO or BEFORE the time
            %
            % Output:
            %   address  the data address (first address is 1)
            %	delay    (time at the address) - (requested time) [seconds]
            %
            %  Note: the address may be outside of the current file/record!
            %
            % Corrects for the clock drift (SHOULD I REALLY DO THIS HERE?)
            
            ts_debug(1,'Starting...');
            if ~exist('comparator','var'),
                comparator='=';
            end
            if isa(time,'ts_datetime')
                temp=obj.metadata;
                temp.timezero=obj.starttime;
                insttime=temp.instrumentTime(time);
                toffset=insttime-obj.starttime;
            else
                toffset=time;
            end
            
            srate = obj.samprate;
            faddress=1 + toffset*srate;
            switch comparator,
                case '>',
                    address=floor(faddress+1);
                case '>=',
                    address=ceil(faddress);
                case '<',
                    address=ceil(faddress-1);
                case '<=',
                    address=floor(faddress);
                case '=',
                    address=round(faddress);
                otherwise,
                    error('Unknown comparator ''%s'', use ''='', ''>'', ''>='', ''<'' or ''<=''', comparator);
            end
            %address = 1 + floor(faddoffset+.5);
            delay = (address - faddress)/srate;
            ts_debug(1,'Ending, address=%d, delay=%g...',address,delay);
        end
        
        function addrList = makeAddressList(obj,addressString,addBuffer,option)
            %MAKEADDRESSLIST GRANDFATHERED Convert a Matlab address string to numbers (should be private?)
            %
            % Usage: addrList = makeaddresslist(obj,addressString,addBuffer)
            %
            % Takes a string of addresses in Matlab format and convert it to numerical values.
            % Optionally, adds a buffer to the back end of the addresses (for causal filtering)
            % This is the only smart option, because you could just use the numerical
            % array otherwise (except that the string list is shorter and takes into
            % account the data length)
            %
            % For example, if addressString = '1:1000 10000:40000 60000:end',
            % length(obj)=100000 and addBuffer=0, then addrList will be
            %	[1:1000 10000:40000 60000:100000];
            % for the same inputs, but addBuffer=1000, then addrList will be
            %	[1:2000 9000:41000 59000:100000];
            %
            %	option[0]:	0, output results as a numerical array
            %				1, output results as an array of cell arrays containing
            %					{startaddress, length}
            
            if ~exist('addBuffer','var'), addBuffer=0; end
            if ~exist('option','var'), option=0; end
            
            addressString = strrep(addressString,'end',num2str(obj.nSamples));
            if addBuffer > 0, addressString = ['1 ' badaddrs]; end	% Will fill out to 1:addBuffer below
            iElem=0;
            if ~isempty(addressString)
                if addBuffer > 0 || option==1,
                    A=textscan(addressString,'%s');
                    addressString = [];
                    for i=1:length(A{1}),
                        T=A{1}{i};
                        if ~isempty(T),
                            k = strfind(T,':');
                            if isempty(k),		% If just a single value
                                minaddr = str2double(T);
                                maxaddr = str2double(T)+addBuffer;
                                if option==1,
                                    addrList{iElem} = {minaddr,maxaddr-minaddr+1}; %#ok<*AGROW>
                                end
                            else
                                B = sscanf(T,'%d:%d');
                                minaddr = B(1);
                                maxaddr = B(2)+addBuffer;
                            end
                        end
                        if option==1,
                            iElem=iElem+1;
                            addrList{iElem} = {minaddr,maxaddr-minaddr+1};
                        else
                            addressString = [addressString sprintf(' %d:%d',minaddr,maxaddr)];
                        end
                    end
                end
            else
                if option==1,
                    addrList = {};
                end
            end
            if option==0,
                eval(['addrList = [' addressString '];']);
            end
        end
    end % Private methods
end % classdef