classdef ts_data_figureproperties < ts_figureproperties
    % Subclass of ts_figureproperties for data figures
    %  see "help ts_figureproperties" for most of the properties and methods
    %  This subclass adds time series plot-specific properties
    %
    % ts_data_figureproperties properties:
    %   title        - Figure title                                     (string)
    %   bottomoffset - Offset of bottom plot from figure bottom         (double)
    %                 (in points) [0]
    %   overlay      - If/how to overlay plots:                         (double)
    %                      0=no overlay
    %                     -1=overlay with common scale
    %                     -2=overlay, each autoscaled (time plots only)
    %                   dely=overlay, each autoscaled and offset by
    %                        dely of full scale (time plots only)
    %   yscale       - y-axis scale, valid values are:                  (string)
    %                     'default', 'linear', 'log', 'db'
    %   fontsize     - Font size [10]                                   (integer)
    %   xticks       - Specify x ticks, empty uses default              (vector)
    %   xticklabels  - Specify x tick labels, empty uses default        (cell vector)
    %   xlabel       - x label, empty uses default                      (string)
    %   plotasmatrix - Organize plots in a matrix (COHERENCE ONLY?)     (boolean)
    %                  [0]
    %   nSubplots    - Number of subplots                               (integer)
    %   subplot      - Vector of subplot parameters                     (ts_figureproperties_subplot)    
    %   decimtype    - Type of decimation to use:                       (string)
    %                          'smart' or 'regular'
    %   smartdecim   - Maximum # of points for smart decimation         (integer)
    %   regdecim     - Regular decimation factor                        (integer)
    %
    % ts_data_figureproperties methods:
    %   ts_data_figureproperties - Creator
    %   addtitle            - Add a title to the plot (called by plot routines)
    %   calcplotpos         - Calculate the position for a subplot
    %
    % EXAMPLES:
    % fp=ts_data_figureproperties
    % fp.nSubplots=5
    % fp.calcplotpos(fp.nSubplots,1)
    % fp.calcplotpos(fp.nSubplots,2)
    % fp.calcplotpos(fp.nSubplots,2,'allrows')
    
    properties
        decimtype='smart';  % Type of decimation: 'smart' or 'regular'
        smartdecim=10000;   % Smart decimation factor: Show up X points
        regdecim=16;        % Regular decimation factor: show every X points
%         start = 0;      % REMOVE???  TIME ONLY: Can be scalar (seconds), char (seconds, minutes, etc), or ts_datetime
%         length = 0;     % REMOVE???? TIME ONLY: int = seconds (0 means rest of data), char can be seconds, minutes, etc
end
    
    %% METHODS
    methods
        function obj=set.decimtype(obj,value)
            if isa(value,'char');
                switch lower(value),
                    case {'smart','regular'},
                        obj.decimtype = lower(value);
                    otherwise, ts_warning('decimtype unchanged: should be "smart" or "regular"');
                end
                obj.decimtype=value;
            else
                error('value must be char');
            end
        end
        function obj=set.smartdecim(obj,value)
            if isnumeric(value), obj.smartdecim=value;
            else error('value must be numeric');
            end
        end
        function obj=set.regdecim(obj,value)
            if isnumeric(value), obj.regdecim=round(value);
            else error('value must be numeric');
            end
        end
        %% DISPLAY AND CHAR METHODS
        function otxt=char(obj,padding)
            % Make character string describing the object
            % padding pads the left side, to give an offset
            if ~exist('padding','var'), padding=0; end
            pformat=sprintf('%%%ds',padding);
            pad=sprintf(pformat,'');
            
            otxt=char@ts_figureproperties(obj,padding);
            otxt=[otxt sprintf('\n%s%20s: ''%s''\n',  pad,'decimtype',obj.decimtype)];
            otxt=[otxt sprintf('%s%20s: %d\n',  pad,'smartdecim',obj.smartdecim)];
            otxt=[otxt sprintf('%s%20s: %d\n',  pad,'regdecim',obj.regdecim)];
        end
        
        %% OTHER METHODS
    end % methods
end % classdef

