function [tdata,dfile] = readSUDS(dfile,startAt,readLen)
    %READSUDS Read SUDS-format time series data into an ts_data object
    % tdata = readSUDS(theFile) reads the entire file into an ts_data object
    % tdata = readSUDS(theFile,startAt,length) does the same thing
    %
    %	theFile is a ts_file object
    %	Ignores startAt and readLen, leaves to calling function ts_file.read
    %	

    %%%%%%%% START ALL ts_* WITH THIS LINE
    ts_debug(1,'Starting\n'); tic;
    %%%%%% START ts_file.read* that use startAt and readLen with these lines
    %if ~exist('startAt','var'), startAt=0; end
    %if ~exist('readLen','var'), readLen=0; end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % READ ALL OF THE DATA
    tdata=readSUDSFile(dfile);
    
    % OUTPUT INFO ON START AND END OF DATA
    startedAt=get(tdata,'chstarttime',1);
    endedAt=startedAt+(get(tdata,'numsamps')-1)/tdata.samprate;
    dfile.custom={ startedAt, endedAt};
return;


function tdata=readSUDSFile(dfile)
    % Put in your function to read the data file
    % All of the file information should be in dfile
    % If there are special parameters that need to be specified for your
    % file format, they should be stored in a cell array in dfile.custom,
    % and their format described in the top comments.
    
    fname =	dfile.filename;
    ts_verbose(1,'Reading SUDS file %s',fname);
    
    % SUDS globals
    global MUXCHANS
    MUXCHANS=1;
    
    fid=fopen(fname,'rb');
    dfile.nChannels=1;
    while ~EOF,
        [dfile,data_length]=suds_read(fid,dfile);
        if data_length>0,
            chanrange=dfile.nChannels : dfile.nChannels+MUXCHANS-1;
            data(chanrange,:)=suds_read_data(fid,dfile,data_length);
            dfile.nChannels=dfile.nChannels+MUXCHANS;
        end
    end

    % If the file format needs custom parameters, they should be in
    % dfile.custom{}.  Check here to make sure they are and output an error
    % message explicitly saying what is missing if they are not.

    % checkCustom()

    % YOUR CODE GOES HERE
    % Put the data into a float32 row array (one channel per row) named "data".
    % Put the time of the first sample in the ts_time object "starttime"

    tdata = ts_data(data,samprate,dfile);
    tdata=set(tdata,'chstarttime',1,startTime);
return

function data=suds_read_data(fid,dfile,data_length)
    global MUXCHANS
    global DATATYPE
    global NUMSAMPS
    global BLOCKSIZE
    
    switch DATATYPE,
        case 's', 
            data=fread(fid,data_length,'uint16');
            %error('Can''t read 12 bit unsigned stored as short int, 0 to 4096');
        case 'q',
            data=fread(fid,data_length,'int16');
            %error('Can''t read 12 bit signed stored as short int, -2048 to 2048');
        case 'u',
            data=fread(fid,data_length,'uint16');
        case 'i',
            data=fread(fid,data_length,'int16');
        case '2'
            data=fread(fid,data_length,'int32');    % 24-bit signed stored as long
        case 'l'
            data=fread(fid,data_length,'int32');    % 32-bit signed stored as long
        case 'r'
            error('Can''t read 12 bit data, 4 lsb time stored as short int');
        case 'f'
            data=fread(fid,data_length,'float');    % 32-bit IEEE real
        case 'd'
            data=fread(fid,data_length,'double');    % 32-bit IEEE real
        case 'c'
            error('Can''t read complex');
        case 'v'
            error('Can''t read vector');
        case 'r'
            error('Can''t read tensor');
        otherwise, error('Unknown data type: ''%s''',DATATYPE);
    end
            
            
return

function [dfile,data_length]=suds_read(fid,dfile)
    % SUDS_DESCRIPTRACE IS NECESSARY (SUDS MANUAL SAYS STATIONCOMP, TOO):
    
    sync=fread(fid,'char'); if sync ~= 'S', error('lost sync!'); end
    machine=fread('char');  if machine ~= '6', error('not Intel x86 format: %c',machine); end
    struct_type=fread('int16');
    struct_length=fread('int32');
    data_length=fread('int32');
    startpos=ftell(fid);
    switch struct_type,
        case 0, ts_verbose(0,'structure=0! Ignoring');
        case 1, % STAT_IDENT
            dfile=suds_read_STATIDENT(fid,dfile);
        case 2, ts_verbose(0,'Ignoring STRUCTTAG structure');
        case 3, ts_verbose(0,'Ignoring TERMINATOR structure');
        case 4, ts_verbose(0,'Ignoring EQUIPMENT structure');
        case 5, ts_verbose(0,'Ignoring STATIONCOMP structure');
        case 6, % MUXDATA
            [dfile,samprate]=suds_read_MUXDATA(fid,dfile);
        case 7, % DESCRIPTRACE
            [dfile,samprate]=suds_read_DESCRIPTRACE(fid,dfile);
        case 8: ts_verbose(0,'Ignoring LOCTRACE structure');
        case 9, ts_verbose(0,'Ignoring CALIBRATION structure');
        case 10, ts_verbose(0,'Ignoring FEATURE structure');
        case 11, ts_verbose(0,'Ignoring RESIDUAL structure');
        case 12, ts_verbose(0,'Ignoring EVENT structure');
        case 13, ts_verbose(0,'Ignoring EV_DESCRIPT structure');
        case 14, ts_verbose(0,'Ignoring ORIGIN structure');
        case 15, ts_verbose(0,'Ignoring ERROR structure');
        case 16, ts_verbose(0,'Ignoring FOCALMECH structure');
        case 17, ts_verbose(0,'Ignoring MOMENT structure');
        case 18, ts_verbose(0,'Ignoring VELMODEL structure');
        case 19, ts_verbose(0,'Ignoring LAYERS structure');
        case 20, ts_verbose(0,'Ignoring COMMENT structure');
        case 21, ts_verbose(0,'Ignoring PROFILE structure');
        case 22, ts_verbose(0,'Ignoring SHOTGATHER structure');
        case 23, ts_verbose(0,'Ignoring CALIB structure');
        case 24, ts_verbose(0,'Ignoring COMPLEX structure');
        case 25, ts_verbose(0,'Ignoring TRIGGERS structure');
        case 26, ts_verbose(0,'Ignoring TRIGSETTING structure');
        case 27, ts_verbose(0,'Ignoring EVENTSETTING structure');
        case 28, ts_verbose(0,'Ignoring DETECTOR structure');
        case 29, ts_verbose(0,'Ignoring ATODINFO structure');
        case 30, ts_verbose(0,'Ignoring TIMECORRECTION structure');
        case 31, ts_verbose(0,'Ignoring INSTRUMENT structure');
        case 32, ts_verbose(0,'Ignoring CHANSET structure');
        otherwise,  ts_verbose(0,'Ignoring unknown structure=%d',struct_type);
    end
    endpos=ftell(fid);
    if (endpos-startpos) < struct_length,
        fseek(fid,struct_length - (endpos-startpos),'cof');
    elseif (endpos-startpos) > struct_length
        error('the structure you just read took up more space than its length (problem in your code)')
    end
return;

function dfile=suds_read_STATIDENT(fid,dfile)
    ts_verbose(1, 'Reading structure STATIDENT (station identifier)');
    ichan=dfile.nChannels;
    dfile.chan(ichan).metadata.network=fread(fid,4,'char');
    dfile.chan(ichan).metadata.station=fread(fid,5,'char');
    dfile.chan(ichan).metadata.component=fread(fid,1,'char');
    temp=freqd(fid,1,'int16');  % Instrument type    
return;

function [dfile,samprate]=suds_read_MUXDATA(fid,dfile)
    ts_verbose(1, 'Reading structure MUXDATA (multiplexed data info)');
    dfile.network=fread(fid,4,'char');
    dfile.starttime=ts_time('1970/1/1')+fread(fid,1,'double');
    temp=fread(fid,1,'int16');                  % Minutes to add to GMT time to get local time
    global MUXCHANS=fread(fid,1,'int16');       % number of channels
    samprate=fread(fid,1,'float');        % samples per second
    global DATATYPE=fread(fid,1,'char');
    temp = fread(fid,1,'char');                 % Descriptor: g=good, t=telemetry noise, c=calib...    
    temp = fread(fid,1,'int16');                % Spare...    
    global NUMSAMPS = fread(fid,1,'int32');     % length: samples in trace    
    global BLOCKSIZE = fread(fid,1,'int32');    % # of demultiplexed samples per
                                                % channel if data is partially demultiplexed   
return;

%%% REQUIRED FIELD
function [dfile,samprate]=suds_read_DESCRIPTRACE(fid,dfile)
    startpos=ftell(fid);
    ts_verbose(1, 'Reading structure DESCRIPTRACE (trace description)');
    dfile=suds_read_STATIDENT(fid,dfile);   % Station ID (network, name, component, type)
    ichan=dfile.nChannels;
    dfile.chan(i).starttime=ts_time('1970/1/1') + fread(fid,1,'double');
    temp = fread(fid,1,'int16');            % Minutes to add to GMT time to get local time
    global DATATYPE=fread(fid,1,'char');
    temp = fread(fid,1,'char');             % Descriptor: g=good, t=telemetry noise, c=calib...
    temp = fread(fid,1,'int16');            % digi_by: agency code who digitized records
    temp = fread(fid,1,'int16');            % processed: processing done on this waveform
    global NUMSAMPS = fread(fid,1,'int32'); % length: samples in trace
    samprate = fread(fid,1,'float');  % rate: samples per second
    temp = fread(fid,1,'float');            % mindata: minimum value of data
    temp = fread(fid,1,'flaot');            % maxdata: maximum value of data
    temp = fread(fid,1,'float');            % avenoise: avg value of 1st 200 samples
    temp = fread(fid,1,'int32');            % numclip: # of clipped datapoints
    temp = fread(fid,1,'double');           % time_correct: correction to add to begintime
    if temp~=0, warning('ignoring time correction!'); end
    temp = fread(fid,1,'float');            % rate_correct: rate correction to add to rate
    if temp~=0, warning('ignoring rate correction!'); end
    endpos=ftell(fid)
    if
    
return;


function [startAddress,endAddress]=calcCutAddrs(tdata)
    numsamps = tdata.nSamples;
    samprate = tdata.samprate;

    if isa(startAt,'ts_datetime'),
        timeOffset = startAt-get(tdata,'chstarttime',1);
        startAddress = round(timeOffset*samprate +1);
    elseif startAt~=0,
        startAddress = round(startAt*samprate +1);
    else
        startAddress = 1;
    end
    endAddress = numsamps-startAddress+1;
    if  readLen~=0,
        endAddress = startAddress+(readLen*samprate) - 1;
    end
return;
