
function [tdata,dfile] = readLCPO(dfile,startAt,readLen)
    % READLCPO Read L-CHEAPO format OBS data into an ts_data object
    % Usage: tdata = readLCPO(dfile,startAt,readLen)
    %  -or - [tdata,dfile] = readLCPO(dfile,startAt,readLen)
    %
    % Inputs:
    %   dfile:          A ts_file object.
    %   startAt [0]:	if a scalar, start that many seconds into the file
    %                       if an ts_datetime, start at given time
    %   readLen[0]:     number of seconds to read (if 0, read one day's data)
    %
    % Output:
    %	tdata		a ts_data object.
    %	dfile       the input ts_file object with parameters privateFirstTime, privateLastTime,
    %                   and custom modified (custom contains numBits)
        
    %%%%%%%% START ALL ts_file.read* WITH THESE LINES
    ts_debug(1,'Starting'); tic;
    if ~exist('startAt','var'), startAt=0; end
    if ~exist('readLen','var'), readLen=0; end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    ts_verbose(1,'Reading LCHEAPO file %s',dfile.filename);
    fid = fopen(dfile.filename,'rb','ieee-be');
    if fid==-1, error('fopen() failed to open LCPO file %s',dfile.filename); end
    
    [lcDirStart,lcDirCount,sampRate,numChans,lcDescript,lcDataType] = getlchead(fid);
    [lcDirectory] = getlcdir(fid,lcDirStart,lcDirCount);
    firstBlock = lcDirectory(1,1);
    firstLCTime = lcDirectory(1,2);
    firstTime = toTSTime(firstLCTime); %,lcDataType);
    
    if readLen==0,
        ts_warning('lcheapo files can be very long, limiting to 1 hour of data');
        readLen=3600;
    end
    numSamps = round(readLen*sampRate);
    ts_debug(1,'firstTime = %s, samprate=%d',char(firstTime),sampRate);
    if isa(startAt,'ts_datetime'),	% Read in at given time
        [startBlk,nBlks2Rd,sampSkip,totalOffset] = calcLCFilePos(lcDataType,startAt,firstTime,firstBlock,sampRate,numChans,numSamps);
        ts_debug(1,'Requested startTime %s => addr Offset %d', char(startAt), totalOffset);
    else						% Read in at a given number of seconds from the beginning
        [startBlk,nBlks2Rd,sampSkip,totalOffset] = calcLCFilePos(lcDataType,startAt+firstTime,firstTime,firstBlock,sampRate,numChans,numSamps);
        ts_debug(1,'Requested start Offset %d seconds => addr Offset %d', startAt, totalOffset);
    end
    startTime = firstTime+totalOffset/sampRate;
    ts_debug(1,['Requested instrument startTime is ' char(startTime)]);
    
    [DATA,blocktime,iblock]=getlcdata(fid,startBlk,numChans,nBlks2Rd,lcDataType);
    if isempty(blocktime), ts_warning('No time checking: Couldn''t find a valid time in any of the data blocks');
    else
        secsPerBlock=calcSampsPerBlock(lcDataType)/sampRate;
        ts_debug(2,'blocktime=%s, iblock=%d, offset from datastart=%.2fs',char(blocktime),iblock,(iblock-1)*secsPerBlock-sampSkip/sampRate)
        %ts_debug(1,'Will skip in %d samples (%.2f seconds)',sampSkip,sampSkip/sampRate);
        obtainedStartTime=blocktime-(iblock-1)*secsPerBlock + sampSkip/sampRate;
        ts_debug(1,['Obtained instrument startTime is ' char(obtainedStartTime)]);
        fclose(fid);
        if abs(startTime-obtainedStartTime) > 1/sampRate,
            warning('Data was offset by more than one sample from the start time\nRequested=%s, obtained=%s: difference=%.1f seconds\nDATE/TIME COULD BE WRONG!!!!', ...
                char(startTime),char(obtainedStartTime),abs(startTime-obtainedStartTime));
        end
    end
    DATA=DATA(:,sampSkip+1:sampSkip+numSamps);  %%% GET ONLY DATA WE WANT FROM THESE BLOCKS (SKIP SAMPSKIP SAMPLES)
    
    % STUFF DATA INTO ts_data OBJECT
    %lcDescript
    for i=1:length(dfile.ch_metadata),
        dfile.ch_metadata(i).information = lcDescript;
    end
    dfile.nChannels=numChans;
    %nameHdr = sprintf('%d:',dfile.instnum);
    readchans = dfile.readchans;
    if isempty(readchans), readchans=1:numChans; end
    dfile.readchans=readchans;
    ts_debug(1,['readchans = [' sprintf('%d ',readchans) ']']);
    %dfile.timezero= startTime; % BALDERDASH!
    tdata = ts_data(DATA,sampRate,dfile);
    %startTime      % DEBUG
    tdata=tdata.setstarttimes(startTime);
    %dfile,tdata     % DEBUG
    % WE CAN'T FORCE CHANNEL NAMING, BECAUSE BBOBS SPOBS USE DIFFERENT
    %  CH      OBS-2C     OBS-4C     OBS-BB
    %   1                 Hyd         X (E)
    %   2                 X           Y (N)
    %   3      ---        Y           Z
    %   4      ---        Z           DPG
    
    % siteFirstTime = toTSTime(lcDirectory(1,2));%,lcDataType);
    % siteLastTime = toTSTime(lcDirectory(end,2));%,lcDataType);
    if lcDataType==0,	numBits=16;
    elseif lcDataType==2, numBits=24;
    end
    dfile.privateFirstTime=toTSTime(lcDirectory(1,2));
    dfile.privateLastTime=toTSTime(lcDirectory(end,2));
    % dfile.privateLastTime=ts_datetime('2010/12/23 05:00:00');    % temporary for BBMMC4_13
    % ts_warning('readLCPO is hard-coding the instrument end time for site Lc1 (BBMMC4_13_New), don''t use with another!!!!!!!!!');
    dfile.custom={numBits};
    
    ts_debug(1,'%s: Exiting after %.1f second\ns',toc);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function mytime = toTSTime(lcDirTime) %,lcDataType);
    %%% The following is for Y2K problems with 16-bit instruments.  I don't understand it, so I took it out
    %%% PLOT START TIME: FIX FOR Y2K PROBLEM ON 16 BIT INSTRUMENTS BY MAPPING YEARS 2000+ to 1972+ FOR 16 BIT INSTRUMENTS
    % if lcDataType==0  % 16-bit
    %	if lcDirTime.YY < 72; lcDirTime.YY = lcDirTime.YY+72; end
    %elseif lcDataType==2, %24 bit instrument, y2k compliant
    %	if lcDirTime.YY >= 72; lcDirTime.YY = lcDirTime.YY-72; end
    %end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %[y,m,d,h,mi,s] = datevec(lcDirTime,lcDataType);
    [y,m,d,h,mi,s] = datevec(lcDirTime);
    % Convert to real years!, not this "after 2000" crap!  (are 16-bit just in years since 1972?)
    y=y+2000;
    mytime = ts_datetime(y,m,d,h,mi,s);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function nSpBlk = calcSampsPerBlock(lcDataType)
    if lcDataType==0,	% 16-bit
        nSpBlk = 249;
    elseif lcDataType==2, % 24-bit
        nSpBlk = 166;
    else
        error('Unknown L-CHEAPO data type: %d',lcDataType);
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [startBlk,nBlks2Rd,sampSkip,totalOffset] = calcLCFilePos(lcDataType,startTime,firstTime,firstBlock,sampRate,numChans,numSamps)
    % returns the sample at or just after the requested time.  Important not to
    % take a sample that starts a fraction of a second before in case you are
    % cutting into hourly/daily blocks to feed to another program, because
    % other programs don't always handle date times well
    % (e.g. call "2007/12/31 23:59:59.9999999999"  2007/12/31 instead of 2008/01/01
    
    nSpBlk = calcSampsPerBlock(lcDataType);
    %%% CALCULATE STARTING RECORD TO READ FROM
    %
    sampSkip = ceil(sampRate*(startTime-firstTime));	% SAMPLES TO SKIP FROM FIRST BLOCK
    nBlkSkip =numChans*floor(sampSkip/nSpBlk);	% BLKS TO SKIP:
    startBlk = nBlkSkip+firstBlock;				% STARTING BLOCK NUMBER ON FILE
    sampSkip = sampSkip-nBlkSkip*nSpBlk/numChans; % NUMBER OF SAMPLES TO SKIP IN THE BLOCK
    nBlks2Rd=ceil((sampSkip+numSamps)/nSpBlk);	% NUMBER OF BLKS TO READ
    
    totalOffset = sampSkip+nBlkSkip*nSpBlk/numChans;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%-----------------------------------------------------------------------
% function getlchead.m
% Version 1.0, October 2000
%
% Written by Kerry Key
% Scripps Institution of Oceanography
%
% Summary: Reads in header parameters from an LCHEAPO binary data file
%          First open file using matlab command fopen.m to get input
%          argument FID: FID = fopen(filename,'r','b'). Make sure to use
%          the 'b' argument as this lets matlab know the byte ordering of
%          the file, making the code usable on both PC's,Macs and Unix boxes.
%
% Usage:   [DIR_START,DIR_COUNT,SAM_RATE,NUM_CHANS,...
%          DESCRIPTN,DATA_TYPE] = getlchead(FID);
%
% Inputs:  FID        - file indentifier from fopen.m
%
% Outputs: DIR_START  - start block of directory listing
%          DIR_COUNT  - number of directory entries
%          SAM_RATE   - instrument sampling frequency (in Hz)
%          NUM_CHANS  - Number of channels recorded by LCHEAPO
%          DESCRIPTN  - User description of instrument deployment
%          DATA_TYPE  - 0 = 16-bit, 2 = 24-bit data
%
%-----------------------------------------------------------------------------

function [DIR_START,DIR_COUNT,SAM_RATE,NUM_CHANS,DESCRIPTN,DATA_TYPE] = getlchead(FID)
    
    BLK_SIZE=512;
    
    %Read in file header:
    fseek(FID,2*BLK_SIZE,'bof'); % Header is in block 2
    HEADER=fread(FID,BLK_SIZE,'uint8'); %read one block
    
    %Decode Header:
    i = 12;
    DIR_START = HEADER(i+4) + 256*(HEADER(i+3) + ...
        256*(HEADER(i+2) + 256*HEADER(i+1)));
    i = i + 4;
    DIR_SIZE  = HEADER(i+4) + 256*(HEADER(i+3) + ...
         256*(HEADER(i+2) + 256*HEADER(i+1)));
    i = i + 4;
    % DIR_BLOCK = HEADER(i+4) + 256*(HEADER(i+3) + ...
    %     256*(HEADER(i+2) + 256*HEADER(i+1)));
    i = i + 4;
    DIR_COUNT = HEADER(i+4) + 256*(HEADER(i+3) + ...
        256*(HEADER(i+2) + 256*HEADER(i+1)));
    i = i + 4;
    % SLOW_DATA = HEADER(i+4) + 256*(HEADER(i+3) + ...
    %     256*(HEADER(i+2) + 256*HEADER(i+1)));
    i = i + 4;
    % SLOW_SIZE = HEADER(i+4) + 256*(HEADER(i+3) + ...
    %     256*(HEADER(i+2) + 256*HEADER(i+1)));
    i = i + 4;
    % SLW_CRBLK = HEADER(i+4) + 256*(HEADER(i+3) + ...
    %     256*(HEADER(i+2) + 256*HEADER(i+1)));
    i = i + 4;
    % SLW_CRBYT = HEADER(i+4) + 256*(HEADER(i+3) + ...
    %     256*(HEADER(i+2) + 256*HEADER(i+1)));
    i = i + 4;
    % LDAT_STBK = HEADER(i+4) + 256*(HEADER(i+3) + ...
    %     256*(HEADER(i+2) + 256*HEADER(i+1)));
    i = i + 4;
    % LOG_DATSZ = HEADER(i+4) + 256*(HEADER(i+3) + ...
    %     256*(HEADER(i+2) + 256*HEADER(i+1)));
    i = i + 4;
    % LDCUR_BLK = HEADER(i+4) + 256*(HEADER(i+3) + ...
    %     256*(HEADER(i+2) + 256*HEADER(i+1)));
    i = i + 4;
    % LDCUR_BYT = HEADER(i+4) + 256*(HEADER(i+3) + ...
    %     256*(HEADER(i+2) + 256*HEADER(i+1)));
    i = i + 4;
    % DAT_START = HEADER(i+4) + 256*(HEADER(i+3) + ...
    %     256*(HEADER(i+2) + 256*HEADER(i+1)));
    i = i + 4;
    % DISK_NUMB = HEADER(i+2) + 256*HEADER(i+1);
    i = i + 2;
    PROG_VERS = char(HEADER(i+1:i+4)');
    i = i + 10;
    DESCRIPTN = char(HEADER(i+1:i+80)');
    i = i + 80;
    SAM_RATE  = HEADER(i+2) + 256*HEADER(i+1);
    i = i + 2;
    % START_CHN = HEADER(i+2) + 256*HEADER(i+1);
    i = i + 2;
    NUM_CHANS = HEADER(i+2) + 256*HEADER(i+1);
    i = i + 2;
    % SLWD_RATE = HEADER(i+2) + 256*HEADER(i+1);
    i = i + 2;
    % SLWST_CHN = HEADER(i+2) + 256*HEADER(i+1);
    i = i + 2;
    % NSLW_CHNS = HEADER(i+2) + 256*HEADER(i+1);
    i = i + 2;
    DATA_TYPE = HEADER(i+2) + 256*HEADER(i+1);
    % i = i + 2;
    % DISK_SIZE = HEADER(i+2) + 256*HEADER(i+1);
    % i = i + 2;
    % RAM_SIZE  = HEADER(i+2) + 256*HEADER(i+1);
    % i = i + 2;
    % NUM_WINWS = HEADER(i+2) + 256*HEADER(i+1);
    % i = i + 2;
    %Rest ofblock reserved for future use.
    
    % FIX FOR 24 bit sampling rates:
    if SAM_RATE==31 || SAM_RATE == 32;
        SAM_RATE=31.25;
    elseif SAM_RATE==62;
        SAM_RATE=62.5;
    end
    
    % Debugging info:
    ts_debug(1,'   Program Version:             %-4s',PROG_VERS);
    ts_debug(1,'   Experiment Description:      %s',DESCRIPTN);
    ts_debug(1,'   Sample Rate (Hz):            %-4.0f',SAM_RATE);
    ts_debug(1,'   Number of Channels:          %-4.0f',NUM_CHANS);
    ts_debug(1,'   Directory Start Block:       %-6.0f',DIR_START);
    ts_debug(1,'   Max # of Directory Entries:  %-8.0f',DIR_SIZE);
    ts_debug(1,'   # of Directory Entries:      %-8.0f',DIR_COUNT);
    ts_debug(1,'   Data type (0=16, 2=24 bit):  %-2.0f',DATA_TYPE);
end

%%%%%%%%%%%% END OF HEADER READ %%%%%%%%%%%%%%%%

%------------------------------------------------------------------------
% function getlcdir.m
% Version 1.0, October 2000.
%
% Written by Kerry Key
% Scripps Institution of Oceanography
%
% Summary: Reads in directory from an LCHEAPO binary data file.
%          First use getlchead.m then this routine, then getlcdata.m
%
% Usage:   [DIRECTORY]=getlcdir(FID,DIR_START,DIR_COUNT)
%
% Inputs:  FID       - file identifier from matlab command fopen.m
%          DIR_START - block number of first directory entry
%          DIR_COUNT - number of directory entries
%
% Outputs: DIRECTORY - Directory entries [block# Timestamp] with
%                      Timestamp in matlab's "datenum" format
%
%-----------------------------------------------------------------------

function [DIRECTORY]=getlcdir(FID,DIR_START,DIR_COUNT)
    
	BLK_SIZE=512;
    
    %%%%%%%%%%%% FILE DIRECTORY READ %%%%%%%%%%%%%%%
    %%% Go to DIR_START block:
    fseek(FID,BLK_SIZE*DIR_START,'bof');
    
    %%% Read into all entries into BUFFER matrix:
    BUFFER = fread(FID,[32 DIR_COUNT],'uint8');
    
    %%% Extract info:
    IMS       = BUFFER(1,:)*256 + BUFFER(2,:);
    SS   = BUFFER(3,:) + IMS/1000;
    MM   = BUFFER(4,:);
    HH     = BUFFER(5,:);
    DD      = BUFFER(6,:);
    MO    = BUFFER(7,:);
    YR     = BUFFER(8,:);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    NBLOCK    = ((BUFFER(9,:)*256 + BUFFER(10,:))*256 + ...
        BUFFER(11,:))*256  + BUFFER(12,:);
    % LENGTHREC = ((BUFFER(13,:)*256 + BUFFER(14,:))*256 + ...
    %     BUFFER(15,:))*256  + BUFFER(16,:);
    IRATE     = BUFFER(17,:)*256 + BUFFER(18,:);
    NBLOCKS   = BUFFER(19,:)*256 + BUFFER(20,:);
    
    %%% Print directory to screen:
    TEMP=[1:length(NBLOCK);NBLOCK;YR;MO;DD;HH;MM;SS;IRATE;NBLOCKS];
    ts_debug(2,'    I       BLOCK#  YR/MO/DD_HH:MM:SS.SS RATE  NBLOCKS')
    ts_debug(2,'%5d %12.0f  %02.0f/%02.0f/%02.0f_%02.0f:%02.0f:%05.2f %4.0f %6.0f\n',TEMP);
    %%% Save Block number and time (as datenum, can reverse with datevec.m)
    %%% into DIRECTORY array:
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    DATE=datenum(YR,MO,DD,HH,MM,SS);
    DIRECTORY = [NBLOCK', DATE'];
    
end
%%%%%%%%%%%%  END OF DIRECTORY READ %%%%%%%%%%%%%%%%%%%%%%%%%

%-----------------------------------------------------------------------------
% function getlcdata.m
%-----------------------------------------------------------------------------
% Version 1.1, October 2001.  Added return statement for when eof reached
% Version 1.0, October 2000.
%
% Written by: Kerry Key
% Scripps Institution of Oceanography
%
% Summary: Gets blocks of LCHEAPO data from a binary data file
%          Open file using fopen, then use getlchead.m and getlcdir.m to get input
%          paramaters for this function
%
% Usage:   [DATA] = getlcdata(FID,STARTBLK,NCH,NBLKS,DATA_TYPE)
%
% Inputs:  FID        - file identifier
%          STARTBLK   - first block in data file to be read
%                      (in disk format as listed in directory)
%          NCH        - number channels in data file
%          NBLKS      - number of blocks per channel to read
%          DATA_TYPE  - 0 for 16 bit, 2 for 24-bit intruments
%
% Output:  DATA       - a NCH x NBLKS*BLKSZ array(BLKSZ: 166 or 249 (24,16 bit)
%
%-----------------------------------------------------------------------------

function [DATA,blocktime,iblock,STATUS]=getlcdata(FID,STARTBLK,NCH,NBLKS,DATA_TYPE)
    
    BLK_SIZE=512;
    
    %%% Set the file position indicator to the start of the requested data blocks
    fseek(FID,BLK_SIZE*STARTBLK,'bof');
    %%% Skip data block headers (14 bytes of info)
    BUFFER = fread(FID,14,'uint8');
    %%% Extract info:
    iblock=1;
    blocktime=extractTimeHeader(BUFFER(1:8));
    if isempty(blocktime)
        ts_warning('First data block had bad time, searching for good time');
        iblock=15;   % Kludge for 2009/12/19 2200 on LSc1
        while isempty(blocktime) && iblock*NCH<NBLKS,    % Sometimes header has an error
            fseek(FID,BLK_SIZE*(STARTBLK+iblock*NCH),'bof');
            iblock=iblock+1;
            BUFFER = fread(FID,14,'uint8');
            blocktime=extractTimeHeader(BUFFER(1:8));
        end
    end
    if (iblock >1)
        if iblock*NCH<NBLKS
            ts_verbose(0,'Found good time in block %d (block group %d): %s', ...
                1+iblock*NCH,iblock,char(blocktime));
        else
            ts_warning('Never found a good time!');
        end
    end
    %NSAMPS=BUFFER(11,:)*256 + BUFFER(12,:);
    %ts_debug(1,'block #%d time=%s, nsamps=%d',iblock,char(blocktime),NSAMPS);
    STATUS=fseek(FID,BLK_SIZE*STARTBLK+14,'bof');    % Redundant?
    %%% Check to see if the end of datafile has been reached, warn if so
    if STATUS~=0
        errordlg('Fseek failed in getlcdata, end of file?','File read error');
        return
    end
    
    %%% Read in the data blocks, 16- and 24-bit versions:
    % 16-bit data read:
    if DATA_TYPE==0; %DATA_TYPE is 16/24 bit flag
        [TEMP,count]=fread(FID,[249 NBLKS*NCH],'249*bit16',112);
        %Above: 112 is number of bits skipped at reading each 249*bit16 data (the data block header)
        % Check to see if end of datafile reached
        if count ~= 249*NBLKS*NCH
            errordlg('Fread failed in getlcdata, end of file?','File read error');
            STATUS = 1;
            return
        end
        % Reshape TEMP into NCH x 249*NBLKS DATA array
        DATA=zeros(NCH,249*NBLKS);
        for i= 1:NCH
            DATA(i,:)=reshape(TEMP(:,i:NCH:NBLKS*NCH),1,249*NBLKS);
        end
        
        %%% 24-bit data read:
    elseif DATA_TYPE==2;
        [TEMP, count]=fread(FID,[166,NBLKS*NCH],'166*bit24',14*8);
        if count~=166*NBLKS*NCH
            errordlg('Fread failed in getlcdata, end of file?','END OF FILE');
            STATUS = 1;
            return
        end
        %%% RECURSIVE READ IS SLOW, REPLACE BY ALL AT ONCE
        %     TEMP = zeros(166,NBLKS*NCH);
        %     for i=1:NBLKS*NCH,
        %         [tmp, count]=fread(FID,166,'bit24');
        %         if count~=166
        %             errordlg('Fread failed in getlcdata, end of file?','END OF FILE');
        %             STATUS = 1;
        %             return
        %         end
        %         TEMP(:,i) = tmp;
        %         %count
        %         STATUS=fseek(FID,14,'cof');
        %     end
        
        DATA=zeros(NCH,166*NBLKS);
        for i=1:NCH
            DATA(i,:)=reshape(TEMP(:,i:NCH:NBLKS*NCH),1,166*NBLKS);
        end
        %%% End of DATA_TYPE if statement
    end
end
%---------------------------------------------------------------------------------------------

function thetime=extractTimeHeader(BUFFER)
    thetime=[];
    SS   = BUFFER(3) + (BUFFER(1)*256 + BUFFER(2))/1000;
    MM   = BUFFER(4);
    HH   = BUFFER(5);
    DD   = BUFFER(6);
    MO   = BUFFER(7);
    YR   = BUFFER(8);
    if (SS>=0 && SS<=60 && MM>=0 && MM<60 && HH>=0 && HH<24 ...
            && DD>0 && DD<32 && MO>0 && MO<13)
        thetime=ts_datetime(2000+YR,MO,DD,HH,MM,SS);
    end
    ts_verbose(2,'Header time components = %02dy %02dm %02dd %02dH %02dM %02dS %5.0fms',...
        YR,MO,DD,HH,MM,BUFFER(3),BUFFER(1)*256+BUFFER(2));
end
