function [tdata,f] = readDB(format,basedir,startAt,readLen,varargin)
    % Read time-series data from multiple files into a ts_data object
    % Usage:
    %   tdata = ts_file.readDB(format,basedir,startAt,readLen,options)
    %			-or-
    %   [tdata,dfile] = ts_file.readDB(format,basedir,startAt,readLen,options);
    %
    % Reads in datafiles in a database corresponding to the requested times and also
    % 	by what network, station(s) or component(s) you want
    %
    % Input:
    % format: 'SDS':  data are in daily miniSEED files in the SeisComp3 data structure
    %         'XXXX:fname': data are in a format 
    %             'XXXX' is a format that TiSKit knows (e.g, 'SAC', 'AH',...)
    %                 type 'ts_file.listFormats()' for a list of formats)
    %             'fname' is a formatted string that can include the following
    %                 fields :
    %                     %y  4-digit year
    %                     %m  2-digit month
    %                     %d  2-digit day
    %                     %j  3-digit julian day
    %                     %H  2-digit hour
    %                     %M  2-digit minutes
    %                     %S  2-digit seconds
    %                     %n  network
    %                     %s  station
    %                     %c  component
    %                     %l  location
    % basedir: The directory in which the database is rooted
    % startAt: Start of data to read:
    %                 Either a character string like '2008/01/31 12:00'
    %				  or a ts_datetime object
    % length:  Number of seconds to read [default=3600]
    % options: Strings specifying additional parameters for the data to take from the database
    %               written as 'parameter=value':
    %                   Paramters that restrict the files to read. 
    %                    Only read files with PARAMETER=VALUE in the 
    %                    filename.  VALUE may contain wildcards '*'
    %                       'network'
    %                       'component'
    %                       'location'
    %                       'station'
    %                   Parameters that specficy how the files are distributed
    %                    in time
    %                       'granularity': Offset between each file in seconds [86400]
    %                       'offset':      Offset of each file from the 
    %                                        "logical" beginning (in
    %                                        seconds) [0]
    %                                        Examples of "logical" beginnings: 
    %                                        Granularity  Beginnings
    %                                        1 day        even days
    %                                        N hours      even days
    %                                        1 hour       even hours
    %                                        1 minute     even minutes
    %
    %           Examples:
    %               tdata=readDB('SAC:%y/%m/%y-%m-%d_%H.sac','.','2007/08/01',3600);
    %               % reads one hour from daily SAC files in year/month directories
    %           
    %               tdata=readDB('SAC:%y/%m/%y-%m-%d_%H.sac','.','2007/08/01',3600,'offset=43200')
    %               % reads one hour of SAC data, daily SAC files start at 12h each day
    %
    %               tdata=readDB('SDS','/MySDS','2007/08/01',3600)
    %               % reads one hour of data from an SDS database
    
    %
    % W Crawford 4/2012
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%% Standard beginning for ts_* functions %%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    ts_debug(1,'Starting'); tic;             %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    debug=0;
    
    if nargin<4,
        error('You must specify the format, base directory, start time and length');
    end
    
    if strcmp(format,'SDS');
        format='MSEED:%y/%n/%s/%c.D/%n.%s.%l.%c.D.%y.%j';
    end
    
    network='*';
    station='*';
    location='*';
    component='*';
    granularity=86400;
    offset=0;
    for i=1:length(varargin),
        C=regexp(varargin{i},'=','split');
        param=C{1};
        value=C{2};
        switch param
            case {'network','station','location','component'}
                eval(sprintf('%s=''%s'';',param,value));
            case {'granularity','offset'};
                str2double(value);	% Simple check: should give error if value is not a number
                eval([varargin{i} ';']);
            otherwise
                error('"%s" is not a valid parameter',param);
        end
    end
    
    C=regexp(format,':','split');
    format=C{1};
    fname=C{2};


    %% HANDLE TIME SYMBOLS IN FILENAME
    startAt=ts_datetime(startAt);
        % Old version, assumed every file starts and ends at the exact specified times
        %[fnames,startAts,readLens]=getTimeNames(fname,startAt,readLen,granularity,offset); 
    % New version, assumes that start and end times can be off by up to
    % slop seconds
    slop=0.05*granularity;  % Assumes files can start/end slop seconds too early/late
    fnames=getTimeNames(fname,startAt,readLen,granularity,offset,slop); 
    ts_verbose(2,'File list returned by getTimeNames');
    for i=1:length(fnames),
%        ts_verbose(2,'%2d: %s %s %g',i,fnames{i},char(startAts{i}),readLens{i});
        ts_verbose(2,'%2d: %s',i,fnames{i});
    end
    
    %% FOR EACH SET OF DATA FILES REQUIRED BY THE TIME RANGE
        firstRead=1; %startGap=0;
    for ifn=1:length(fnames), % Looping over files by TIME
        basefname=fnames{ifn};
        ts_debug(1,'Time-based file name= %s',fullfile(basedir,basefname));

        %% HANDLE STATION/NETWORK/COMPONENT SYMBOLS
        fname=expandParameters(basefname,network,station,location,component);
        

        %% CHECK FOR EXISTENCE OF INPUT FILE, EXPAND TO LIST OF FILES IF NECESSARY
        % Force basedir to be an absolute path (because getFileList returns abs paths)
        currdir=pwd; cd(basedir); basedir=pwd; cd(currdir);
        
        %% READ INDIVIDUAL COMPONENTS WITH SAME FILENAME TIMES
        filenames=getFileList(fullfile(basedir,fname));
        for i=1:length(filenames),
            ts_debug(1,'  Expanded file name= %s',filenames{i});
            %fprintf('  Expanded file name= %s\n',filenames{i});
            f=ts_file(filenames{i},format);
            %[td,f]=f.read(startAts{ifn},readLens{ifn});
            [td,f]=f.read(0,0);
            if td.nChannels==1,
                %[n,s,l,c]=getStationParms(filenames{i},fullfile(basedir,basefname));            
                [n,s,l,c]=getStationParms(filenames{i},fullfile(basedir,basefname));            
                if ~isempty(n), td.channel(1).metadata.network=n; end
                if ~isempty(s), td.channel(1).metadata.station=s; end
                if ~isempty(l), td.channel(1).metadata.location=l; end
                if ~isempty(c), td.channel(1).metadata.component=c; end
            else
                ts_warning('Did not try to determine station parameters from file/pathname, because there are multiple channels in the file');
            end
            ch=td.channel(1);
            if debug,
                fprintf('DEBUG: channel %s, %s to %s ... ',ch.info.fullname,ch.starttime,ch.endtime)
            end
            
            if length(filenames)==1;
                if debug,
                    fprintf('Initiating AND Ending (DEBUG)\n')
                end
                t=td; 
                break; 
            end
            
            fstart=f.firstTime;
            fend=f.lastTime;
            
            ts_verbose(1,'%s: fstart=%s,  fend=%s,  startAt=%s, readLen=%d',...
                filenames{i},char(fstart),char(fend),char(startAt),readLen);
            
            if ifn==1,  % Reading first files w.r.t. time
                if fstart>startAt, % the data starts after startAt
                    ts_warning('Data starts %g seconds after requested start',fstart-startAt);
                end
            end
            if firstRead,
                t=td; firstRead=0;
                if debug,
                    fprintf('Initiating  ...')
                end
            else
                if debug,
                   fprintf('Adding  ...')
                end
                t=t+td;
            end
            if debug,
                fprintf('%d total channels \n',t.nChannels);
                t
                t
            end
        end
        if ifn==1,  % First (or only) file (group) time-wise
            tdata=t;
        else
            tdata=tdata+t;	% Concatenates channels with same network/station/component/location
                            % and whose data are consecutive
                            % Makes a new channel for other data
        end
    end
    ts_debug(2,'Input data = %s',char(tdata))

    
    %% Cut data down to size and synchronize
    tdata=tdata.cut(startAt,readLen);
    
    %% Standard end for TiSKit functions %%%%%%%%%%%%%
    ts_debug(1,'Exiting after %.1f seconds',toc); %
    %%
end % end of function


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function filenames=getFileList(fname)
    %% CHECKS FOR EXISTENCE OF INPUT FILE, EXPANDS TO CELL ARRAY OF MATCHED FILES
    
    if ~ischar(fname),
        error('fname is not a string');
    end
    %foundfnames=ls(fname);  % Old style, gives different output on PCs (no directory) and Mac/Linux (directory)
    d=rdir(fname); % New style, always gives output with directory names, but in structure field
    foundfnames=sprintf('%s ',d.name); % Makes a text list, so that regexp will create a simple cell array 
    filenames=regexp(foundfnames,'(\S+)','match'); % Extract non-empty entries (still necessary with dir?)
    if isempty(filenames), error('No files matched "%s"',fname); 
    else
        ts_verbose(2,'%d files matched "%s"',length(filenames),fname);
        % NEWLY ADDED BECAUSE FORCED TO USE DIR (which gives no pathname in output)
%         pathname=fileparts(fname);
%         for i=1:length(filenames),
%             filenames{i}=fullfile(pathname,filenames{i});
%         end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%function [fnames,startAts,readLens]=getTimeNames(fname, startAt,readLen,fileDelta,fileOffset)
function [fnames]=getTimeNames(fname, startAt,readLen,fileDelta,fileOffset,slop)
    % returns a list of filenames with the date/time parameters filled in
    % so that the requested time frame will be contained inside
    % Input:
    %	fname=Filename containing time symbols to fill in (see ts_datetime/char)
    %   startAt: datetime of first data to read (ts_datetime object)
    %   readLen: length of data to read in seconds
    %   fileDelta: number of seconds per file
    %   fileOffset: offset in seconds of start of file from start that
    %               would allow some file starts Jan 1 at midnight (00:00:00)
    %   slop: allow for this much offset (seconds) in the data start/end time from
    %           that expected from the filename and expected length
    %
    if ~isa(startAt,'ts_datetime'),
        error('startAt must be a ts_datetime object if the filename has time symbols');
    end
    
    i=0;
    % Expand "times to retrieve to account for imperfect alignment of
    % file start and end times
    startAt=startAt-slop;
    readLen=readLen+2*slop;
    
    tm=floor(startAt,fileDelta,fileOffset);
    while tm < (startAt+readLen),
        i=i+1;
        fnames{i}=char(tm,fname); %#ok<AGROW>
%         The following assumes there is no slop, which we can't afford to do
%         if tm < startAt,
%             startAts{i}=startAt;
%         else
%             startAts{i}=tm;
%         end
%         readLens{i}=min([readLen,fileDelta-(startAts{i}-tm),startAt+readLen-tm]);    %% Allows to read less than fileDelta on first and last files
        ts_verbose(2,'%s',fnames{i});
        tm=tm+fileDelta;
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function str=expandParameters(fmt,network,station,location,component)
    % Replace formatting characters with parameter values
    % Usage: outfname = expandParameters(infname,network,station,location,component)
    %   where infname can contain characters and any of
    %          the following symbols:
    %           '%n' network
    %           '%s' station
    %           '%l' location
    %           '%c' component
    
    %  W Crawford 4/2012
    
    ifmt=1; nfmt=length(fmt);
    str='';
    
    while ifmt<=nfmt,
        if fmt(ifmt)=='%',
            fsymb = fmt(ifmt+1);
            ifmt = ifmt+2;
            switch fsymb,
                case 'n',
                    str = sprintf('%s%s',str,network);
                case 's',
                    str = sprintf('%s%s',str,station);
                case 'l',
                    str = sprintf('%s%s',str,location);
                case 'c',
                    str = sprintf('%s%s',str,component);
                otherwise,
                    % Just stuff unknown formats back into string
                    str = [str '%' fsymb];
            end
        else
            % Stuff non-format characters into string
            str = [str fmt(ifmt)];
            ifmt = ifmt+1;
        end
    end
end

%%%%
function [network,station,location,component]=getStationParms(filename,fnameFormat)
    % Gets station parameters from filename and stuffs them into the ts_data object
    
    ts_debug(1,'filename=%s, fnameFormat=%s',filename,fnameFormat);
    network='';station='';location='';component='';
    % REGEXP METHOD FOR READING NETWORK, ETC 
    fnameFormat=strrep(fnameFormat,'\','\\');   % For Windows machine paths
    fnameFormat=strrep(fnameFormat,'%s','(?<station>\S+)');
    fnameFormat=strrep(fnameFormat,'%c','(?<component>\S+)');
    fnameFormat=strrep(fnameFormat,'%n','(?<network>\S+)');
    fnameFormat=strrep(fnameFormat,'%l','(?<location>\S+)');
    fnameFormat=strrep(fnameFormat,'.','\.');
    temp=regexp(filename,fnameFormat,'names');
    % Read parameters
    station=temp.station;
    component=temp.component;
    if isfield(temp,'network'),  network=temp.network;  end
    if isfield(temp,'location'),location=temp.location; end

end

function text=getParm(C,isort,type)
    % returns the text found at the first index in isort and compares with
    % those at the other indices (if any).  If isort is empty, returns an
    % empty string
    text='';
    if ~isempty(isort)
        text=C{isort(1)};
        if length(isort)>1
            for i=2:length(isort)
                if ~strcmp(text,C{isort(i)})
                    ts_warning('discrepancy between %s names in database: %s and %s', ...
                                type,text,C{isort(i)})
                end
            end
        end
    end;
end
