classdef ts_file
    % PRIMARY TISKIT CLASS: Time series data files
    %
    % ts_file properties:
    %   filename    - Full pathname of file containing data  (string)
    %   format      - Data file format (SAC, MSEED, ...)     (string)
    %   custom      - Parameters specific to the file format (cell vector)
    %   readchans   - Which channels to store and display    (numeric vector)
    %   ch_metadata - Channel information                    (ts_data_channel_metadata vector)
    %   event       - Event information                      (ts_event)
    %   firstTime   - READONLY: First time in the data file  (ts_datetime)
    %   lastTime    - READONLY: Last time in the data file   (ts_datetime)
    %   clockCorrs  - READONLY: [tzoffset clockdrift]        (numeric 2x1)
    %   nChannels   - READONLY: Number of channels in the file (numeric)
    %
    % ts_file methods:
    %     ts_file             - Creator
    %     read                - Read data
    %     save                - Save information to an executable file
    %  Static
    %     ts_file.readDB      - Read data from database (organized hierarchy)
    %     ts_file.listFormats - Returns a list of waveform file formats that can be read
    %
    % EXAMPLE:
    % f=ts_file('mafile','MSEED')  % Explicitly give file type
    % d=f.open()
    % d.plot()
    %
    % f=ts_file('mafile.msd')      % Determines file type from suffix
    %
    % ts_file.listFormats			% Lists the formats whith read*() functions
    %
    % f=ts_file('mafile.msd')
    % f.firsttime
    % f.read(0,0)					% Interogates file for information
    % f.firsttime
    %
    % d=ts_file.readDB('SDS','/SDS','2011/10/14','1h','network=VU')
    %     							% Reads data from network VU in SeisComp database
    % plot(d);						% Same as d.plot()
    %
    
    %% PUBLIC PROPERTIES
    properties
        filename='';	% Full pathname of file containing data
        format='';	    % Format of data file (SAC, MSEED, ...)
        readchans=[];   % Which channels to store and display.  Implemented in ts_file:read()
        custom={};      % Cell array of parameters specific to a given format
        % Array of data channel metadata
        ch_metadata=ts_data_channel_metadata; 
        event=ts_event; % Event information (time, position)
        % datatype='';    % ????? int32 vs int16 vs ???????
    end
    %% PRIVATE PROPERTIES
    properties (SetAccess = private)    % Hard to set structures correctly, make specific routines
        privateFirstTime
        privateLastTime
    end
    %% DEPENDENT PROPERTIES
    properties (Dependent = true)
        firstTime       % First time in the data file (READ ONLY)
        lastTime        % Last time in the data file (READ ONLY)
        clockCorrs;     % 2-element array containing [tzoffset clockdrift]
        nChannels=0;    % Number of channels in the file.
    end
    
    %% PUBLIC METHODS
    methods
        %% CREATOR
        function obj = ts_file(varargin)
            % Create a ts_file instance
            %
            % Usage:
            %   obj = ts_file(filename,format)
            %   obj = ts_file();
            %   obj = ts_file(dfile);		% makes a copy
            % Inputs:
            %     filename [''] Name of the file containing the data.
            %     format [''] Data format. ts_file.read() calls the private function
            %                   ap_readFORMAT() to read the data.  If not
            %                   specified, ts_file tries to determine the
            %                   file type from the filename extension.
            
            ts_init;	% Sets up global variables
            % Choose between overloaded argument lists
            numargs = length(varargin);
            if numargs == 0,
                return;
            elseif numargs == 1 && isa(varargin{1},'ts_file'),
                obj = createCopy(varargin{1});
            else
                obj.filename = varargin{1};
                if numargs>1,
                    obj.format = varargin{2};
                else
                    obj = guessFileFormat(obj);
                end
            end
        end
        
        %% SET METHODS (ELIMINATE BAD INPUTS)
        function obj = set.filename(obj,value)
            if ~isa(value,'char'); error('value must be a character string'); end
            obj.filename=value;
        end
        function obj = set.format(obj,value)
            if ~isa(value,'char'); error('value must be a character string'); end
            obj.format=value;
        end
        function obj = set.readchans(obj,value)
            if ~isa(value,'numeric'); error('value must be numeric'); end
            if ~isequal(value,round(value)), error('value must be integers'); end
            if any(value>obj.nChannels),
                %obj,value,obj.nChannels,
                error('values must all be less than nChannels');
            end
            readall=1;
            if isempty(value)         % Read all channels in order
                obj.readchans=[];       % makes it always read all channels
                readall=0;
            elseif length(value)==obj.nChannels
                if all(value==1:obj.nChannels)      % Read all channels in order
                    obj.readchans=[];   % makes it always read all channels
                    readall=0;
                end
            end
            if readall,
                j=0;
                obj.readchans=[];
                temp=[];    % Use temp instead of obj.readchans because
                % obj.readchans=[] runs this function recursively
                % and automatically populates obj.readchans with
                % all of the channels (
                for i=1:length(value),
                    if value(i)<=0,
                        ts_warning('Invalid channel number: %d.  Ignored',value(i));
                    elseif value(i)>obj.nChannels,
                        ts_warning('readchannel (%d) > nChannels (%d)!  Ignored',value(i),obj.nChannels);
                    else
                        j=j+1;
                        temp(j)=value(i); %#ok<AGROW>
                    end
                end
                obj.readchans=temp; % Now, if temp is empty, we're happy that the
                % recursive call populates it!
            end
        end
        function obj = set.custom(obj,value)
            if ~isa(value,'cell'), error('value must be a cell array'); end
            obj.custom=value;
        end
        function obj = set.privateFirstTime(obj,value)
            value=ts_datetime(value);   % Takes care of data checking and conversion
            obj.privateFirstTime=value;
        end
        function obj = set.privateLastTime(obj,value)
            value=ts_datetime(value);   % Takes care of data checking and conversion
            obj.privateLastTime=value;
        end
        function obj = set.ch_metadata(obj,value)
            if ~isa(value,'ts_data_channel_metadata'), 
                error('value must be a ts_data_channel_metadata object');
            end
            obj.ch_metadata=value;
        end
        
        %% GET METHOD FOR READCHAN (RETURNS LIST EVEN IF READCHANSIS EMPTY)
        function value = get.readchans(obj)
            % Returns all channels if readchans is empty
            value=obj.readchans;
            if isempty(obj.readchans),
                value=1:obj.nChannels;
            end
        end
        
        %% ACCESS METHODS FOR DEPENDENT PROPERTIES
        function obj = set.clockCorrs(obj,val)  % Sets ALL clock drifts to same
            [M,N]=size(val);
            if M~=1 || N ~= 2,
                error('input should be a 1x2 matrix containing [tzoffset clockdrift]');
            else
                for i=1:length(obj.ch_metadata)
                    obj.ch_metadata(i).tzoffset=val(1);
                    obj.ch_metadata(i).clockdrift=val(2);
                end
            end
        end
        function val = get.clockCorrs(obj)
            val=[obj.ch_metadata(1).tzoffset obj.ch_metadata(1).clockdrift];
        end
        function obj = set.nChannels(obj,value)
            % Set the number of channels
            % Creates or destroys channel info array elements as necessary
            if length(value)>1, error('value must be a scalar');
            elseif ~isa(value,'numeric'), error('value must be an integer');
            elseif value~=round(value), error('value must be an integer');
            elseif value<1,
                error('value must be greater than 0');
            end
            oldchans=obj.nChannels;
            newchans=value;
            if (newchans > oldchans)     % Must create new chan instances
                for i=oldchans+1:newchans
                    obj.ch_metadata(i)=obj.ch_metadata(1);
                end
            elseif newchans < oldchans % Must remove chan instances
                obj.ch_metadata=obj.ch_metadata(1:newchans);
            end
        end
        function val = get.nChannels(obj)
            val=length(obj.ch_metadata);
        end
        function val = get.firstTime(obj)
            val=obj.privateFirstTime;
        end
        function val = get.lastTime(obj)
            val=obj.privateLastTime;
        end
        
        %% DISPLAY AND CHAR METHODS
        function txt=char(obj,padding)
            % padding pads the left side, to give an offset
            if ~exist('padding','var'), padding=0; end
            pformat=sprintf('%%%ds',padding);
            pad=sprintf(pformat,'');
            
            txt=     sprintf('%s%12s: ''%s''\n',           pad,'filename',obj.filename);
            txt=[txt sprintf('%s%12s: ''%s''\n',           pad,'format',obj.format)];
            %txt=[txt sprintf('%s%12s: %g\n',               pad,'samprate',obj.samprate)];
            [M,N]=size(obj.custom);
            txt=[txt sprintf('%s%12s: %dx%d cell array \n',pad,'custom',M,N)];
            txt=[txt sprintf('%s%12s: %s\n',               pad,'firstTime',char(obj.firstTime))];
            txt=[txt sprintf('%s%12s: %s\n',               pad,'lastTime',char(obj.lastTime))];
            txt=[txt sprintf('%s%12s: %s object\n',        pad,'event',class(obj.event))];
            txt=[txt sprintf('%s\n',char(obj.event,        padding+10))];
            txt=[txt sprintf('%s%12s: %d\n',               pad,      'nChannels',obj.nChannels)];
            txt=[txt sprintf('%s%12s: %s\n',               pad,      'readchans',sprintf('%3d ',obj.readchans))];
            txt=[txt sprintf('%s%12s: %d element array of %s objects\n',pad,'ch_metadata',length(obj.ch_metadata),class(obj.ch_metadata))];
            txt=[txt sprintf('%s\n',char(obj.ch_metadata,padding+10))];
        end
        function disp(obj)
            % Displays 'char' text when object is not followed by a semicolon
            disp(char(obj));
        end
        
        %% OTHER METHODS
        %  I don't remember why we need createCopy, try to live without
%         function out_obj = createCopy(in_obj)
%             % CREATE A COPY (NECESSARY FOR VALUE CLASSES)
%             out_obj=ts_file();
%             propnames=properties(in_obj);
%             for i=1:length(propnames),
%                 p=propnames{i};
%                 set(out_obj,p,get(in_obj,p));
%             end
%         end
        %
        [tdata,firstOut,secondOut] = read(obj,start,theLength);  % DEFINED IN SEPARATE FILE
        %
        save(obj)              % DEFINED IN SEPARATE FILE, Saves ts_file info to a executable script
        %
        %
        function [startAddress,readsamps]=calcReadAddresses(obj,startAt,endAt)
            % Calculate start address and number of samples to read
            % given the startAt (numeric or ts_datetime) and readLen
            % (numeric or ts_datetime) parameters
            chanObj=ts_data_channel(obj);    % Has to set startime and clockdrift
            % Note: set starttime to timezero
            % in absense of other info
            startAddress=chanObj.calcAddress(startAt);
            if isa(endAt,'numeric')
                if endAt==0; readsamps=0; return; end
            elseif isa(endAt,'ts_datetime')
                if ~isa(startAt,'ts_datetime'),
                    error('endAt is ts_datetime, but startAt isn''t!!');
                else
                    endAt=endAt-startAt;
                end
            end
            readsamps=chanObj.calcNumSamps(endAt);
        end
    end % Public methods
    
    %% STATIC METHODS
    methods (Static)
        function mylist=listFormats()
            % Return cell array of available data file formats
            st=dbstack('-completenames'); % Get current filename and directory
            [path]=fileparts(st(1).file);
            d=dir(fullfile(path,'private','read*.m'));
            %d=dir('private/read*.m')
            %d=dir('*'); d.name
            %fprintf('Readable formats = ');
            for i=1:length(d);
                %fprintf('%s ',d(i).name(5:length(d(i).name)-2));
                mylist{i}=d(i).name(5:length(d(i).name)-2);
            end
            fprintf('\n');
        end
        [tdata,f] = readDB(format,basedir,startAt,readLen,varargin);    % DEFINED IN SEPARATE FILE
    end % Private methods
    
    %% PRIVATE METHODS
    methods (Access = private)
        obj = guessFileFormat(obj); % DEFINED IN SEPARATE FILE
    end % Private methods
    
end



