classdef ts_response
    % Class for instrument responses
    %
    % RESPONSE
    %   The response specifies how your data was transformed from
    %   physical ("input") values to the stored ("output") values:
    %
    %   outUnits = inUnits * resp, where:
    %
    %                    (s-zero1)*(s-zero2)*(s-zero3)*...
    %	resp(s) = pzgain*----------------------------------
    %                    (s-pole1)*(s-pole2)*(s-pole3)*...
    %
    % 	where s=1i*2*pi*frequency
    %
    %   The reponse is the transfer function FROM the input units TO
    %   the output units.
    %   We calculate spectra and convert time series to input units by
    %   dividing the stored data by this response.
    %
    % GAIN VERSUS "PASSBAND" GAIN
    %   The pole-zero gain stored in this class is not the "passband gain"
    %   given for standard lowpass, highpass or bandpass filters, because
    %   transfer functions do not necessarily have a flat passband in which
    %   this gain makes sense.
    %   If you want to specify the passband gain, create the object
    %     1) using pole-zero notation and specifying omega_ref, or
    %     2) using Butterworth filter paramters
    %   for details, see the help for the creator method.
    %
    % ts_response properties:
    %   gain     - Instrument pole-zero gain
    %   poles    - Instrument poles (radians)
    %   zeros    - Instrument zeros (radians)
    %   inUnits  - Units of data at the instrument's input
    %              (e.g., physical units)
    %   outUnits - Units of stored data (e.g., digital units)
    %
    % ts_response methods:
    %   Creator
    %      ts_response    - Creator
    %   Miscellaneous
    %      getNorm        - Return normalization factor and gain at a frequency
    %      plot           - Plot the response
    %      response       - Return response at given frequencies
    %   Reading from standard format
    %      Use the ts_response creator with the filetype as the first
    %      argument
    %   Writing to standard formats
    %      writeGSE2_PZ   - Write the response in GSE pole-zero format (v2.0)
    %      writeSAC_PZ    - Write the response to a SAC pole-zero file
    %      writeSEISAN_PZ - Write the response to a SEISAN pole-zero file
    %   Overloaded Matlab methods
    %      eq             - Implements "="
    %      ne             - Implements "~="
    %      mtimes         - Implements "*"
    %
    % EXAMPLES:
    %   % Bandpass filter with 2 poles at 4.5 Hz and 2 at 100 Hz
    %   tsr=ts_reponse('BW','m/s^2','du',1000,[100 100],[4.5 4.5]);
    %   plot(tsr);
    %
    %   % The same thing in pole-zero notation
    %   tsr=ts_response('PZ','m/s^2','du',3.94784e8,-2*pi*[4.5 4.5 100 100],[0 0]);
    %   plot(tsr)
    %
    %   % Read in from a SAC pole-zero file
    %   tsr=ts_response('SACPZ','','','MYCHANNEL.SACPZ');
    %   plot(tsr)
    %
    %   % ALMOST the same thing in pole-zero notation, specifying gain at a frequency
    %   % in the passband (but the passband isn't perfectly flat, so the response
    %   % isn't exactly the same
    %   tsr=ts_response('PZ','m/s^2','du',1000,-2*pi*[4.5 4.5 100 100],[0 0],2*pi*20);
    %   plot(tsr)
    
    %% PROPERTIES
    properties
        gain=1;              % Instrument pole-zero gain (NOT passband gain)
        poles=[];            % Poles
        inUnits='';         % Units of data that you want to see
        outUnits='';        % Units of data as they are stored
    end
    properties (Dependent=true)
        zeros               % zeros
    end
    
    properties (Access=private, Hidden=true)
        privateZeros=[];           % True storage of zeros (necessary so that
        % matlab doesn't confuse with the function zeros())
    end
    
    %% PUBLIC METHODS
    methods
        %% CREATOR
        function [obj,extra]=ts_response(inpType,iu,ou,argA,argB,argC,argD,argE)
            % Create a ts_response object
            %
            % Usage:
            %   obj= ts_response();                                              % Unity response (gain = 1)
            %   obj= ts_response('SACPZ',inUnits,outUnits,filename);             % SAC Pole-Zero File
            %   obj= ts_response('GSE2PZ',filename);                             % GSE2 Pole-Zero File
            %   obj= ts_response('PZ',inUnits,outUnits,gain,poles,zeros);        % Pole-Zero Notation
            %   obj= ts_response('PZ',inUnits,outUnits,gain,poles,zeros,om_ref); % ""    ""   ""
            %   obj= ts_response('BW',inUnits,outUnits,gain,nHP,hpFrq,nLP,lpFrq);% Butterworth Filter
            %
            % Inputs:
            %   type	Input type:
            %               'SACPZ': SAC Pole-Zero file        %
            %               'GSE2PZ': GSE2 Pole-Zero file
            %               'PZ':    Pole-Zero Notation
            %               'BW':    Butterworth Filter
            %               'Units': Unit Conversion
            %   inUnits	The input units of the data, that is,
            %                   the units that you want spectra, etc to be
            %                   calculated in
            %   outUnits The units in which the data are stored;
            %
            %  SAC POLE-ZERO FILE
            %   filename        Name of a SAC pole-zero file (poles & zeros in rad/s)
            %   inUnits         Automatically set to 'nm' if empty
            %  GSE2 POLE-ZERO FILE
            %   filename        Name of a GSE2 pole-zero file (poles & zeros in rad/s)
            %  POLE-ZERO NOTATION
            %   gain            Pole-zero gain (NOT the passband gain, unless omega_ref is specified)
            %   poles           Pole-zero poles (radians/s)
            %   zeros           Pole-zero zeros (radians/s)
            %   om_ref          Frequency (radians/s) at which the gain applies
            %                    (useful for implementing a "passband gain")
            %  BUTTERWORTH FILTER
            %   gain            Passband gain
            %   nLP             Number of highpass poles
            %   lpFrqs          Highpass corner frequency (Hz)
            %   nLP             Number of lowpass poles
            %   lpFrqs          Lowpass corner frequency (Hz)
            %
            % Output:
            %   obj           the ts_response object
            %
            % Following convention, poles and zeros are expressed in radians
            %         and Butterworth filter frequencies are in Hz.  Here are some
            %         conversion factors.
            %            radians = 2*pi*Hz = 2*pi/Period = 1/RC
            %            Hz = radians/2pi = 1/Period = 1/(2*pi(RC)
            %         where Period is the instrument "natural period" and RC is the
            %         resistance*capicitance for each stage of an analog electric
            %         filter.
            %
            %
            % Examples:
            %	resp = ts_response('Units','m/s^2','nm');   % Data are stored as nm, you want to look at m/s^2
            %	resp = ts_response('SACPZ','','','myPZs');  % Data stored as nm in SAC Pole-Zero file 'myPZs'
            %	resp = ts_response('BW','Pa','V',200,1,50); % Data stored as volts after 1-50 Hz bandpass Butterworth filter, 200 Volts/Pa
            
            %% Check input arguments
            ts_debug(1,'Starting with %d arguments\n',nargin);
            extra=[];
            if nargin==0;  return;                             % 0 arguments => Create empty object
            elseif ((nargin==1) && isa(inpType,'ts_response')) % 1 argument: ts_response object => copy
                obj=inpType; return;
            elseif ~ischar(inpType),
                error(' The first argument is not a string (type ''doc ts_response'')');
                
            end
            if nargin >=4,
                obj.inUnits = iu;
                obj.outUnits = ou;
            end
            
            %% Create poles, zeros and gain according to the input format
            normalizer=1;
            switch upper(inpType),
                case 'SACPZ'
                    if nargin ~= 4
                        error('ts_response(''SACPZ'',...) needs 4 arguments, you gave %d',nargin)
                    end
                    [g,ps,zs] = readSACPZfile(argA);
                    if isempty(obj.inUnits)
                        obj.inUnits='nm';    % Standard for SAC PZ files
                    end
                case 'GSE2PZ'
                    if nargin ~= 1 && nargin ~=2,
                        error('ts_response(''GSE2PZ'',...) needs 1 or 2 arguments, you gave %d',nargin)
                    end
                    if ~exist('iu','var'); argA=[];
                    else argA=iu;
                    end
                    [g,ps,zs,~] = readGSE2PZfile(argA);
                    
                    obj.inUnits='nm'; obj.outUnits='counts';
                case 'PZ',
                    if nargin~=6 && nargin~=7
                        error('ts_response(''PZ'',...) needs 6 or 7 arguments, you gave %d',nargin)
                    end
                    g=argA;
                    ps=argB(:)';
                    zs=argC(:)';
                    if exist('argD','var')
                        omega_norm = argD;
                        s=1i*omega_norm;
                        for p = ps, normalizer = normalizer*(s-p); end
                        for z = zs, normalizer = normalizer/(s-z); end
                        normalizer=abs(normalizer);
                        ts_verbose(1,'Calculated normalizer = %.2f at %g radians',abs(normalizer),omega_norm);
                        g=g*normalizer;
                    end
                case {'BADBW'}, % Not a real butterworth, because all poles are real
                    if nargin~=6
                        error('ts_response(''badBW'',...) needs 6 arguments, you gave %d',nargin)
                    end
                    hpOmega=2*pi*argC;
                    lpOmega=2*pi*argB;
                    nLP = length(lpOmega);
                    nHP = length(hpOmega);
                    ps=zeros(1,nLP+nHP); zs=zeros(1,nHP);
                    for i = 1:length(hpOmega),
                        ps(i) = -hpOmega(i);
                        zs(i) = 0;
                    end
                    for i=1:length(lpOmega),
                        ps(i+nHP) = -lpOmega(i);
                        normalizer = normalizer*lpOmega(i);
                    end
                    normalizer=abs(normalizer);
                    g = argA*normalizer;
                case {'BW','BUTTERWORTH'},
                    assert(nargin==8,'ts_response(''BW'',...) needs 8 arguments, you gave %d',nargin);
                    nHP=argB;           assert(length(nHP)==1,'length(nHP)~=1');
                    assert(isnumeric(nHP),'nHP is not numeric');
                    hpOmega=2*pi*argC;  
                    if nHP==0, assert(isempty(hpOmega),'~isempty(hpOmega) for nHP==0');
                    else assert(length(hpOmega)==1,'length(hpOmega)~=1'); end
                    assert(isnumeric(hpOmega),'hpOmega is not numeric');
                    nLP=argD;           assert(length(nLP)==1,'length(nLP)~=1');
                    assert(isnumeric(nLP),'nLP is not numeric');
                    lpOmega=2*pi*argE;  
                    if nLP==0, assert(isempty(lpOmega),'~isempty(lpOmega) for nLP==0');
                    else assert(length(lpOmega)==1,'length(lpOmega)~=1');end
                    assert(isnumeric(lpOmega),'lpOmega is not numeric');
                    
                    ps=[butterPoles(nHP,hpOmega) butterPoles(nLP,lpOmega)];
                    zs=zeros(1,nHP);
                    if ~isempty(lpOmega)
                        normalizer=abs(lpOmega^nLP);
                    else normalizer=1;
                    end
                    g = argA*normalizer;
                otherwise,
                    error('Invalid type: %s.  Valid types are ''PZ'', ''SACPZ'', ''BW'' and ''UNITS''',inpType);
            end
            
            %% Save poles, zeros and gain
            obj.gain = g;
            if length(obj.gain) ~= 1,
                error('gain must be a scalar');
            end
            obj.poles = ps(:)';
            obj.privateZeros = zs(:)';
            ts_debug(1,'created ts_response object:\n%s',char(obj));
            
        end    % Creator function ts_response
        
        %% SET METHODS (ELIMINATE BAD INPUTS)
        function obj = set.gain(obj,value)
            if ~isa(value,'numeric'), error('value must be numeric'); end
            obj.gain=value;
        end
        function obj = set.poles(obj,value)
            if ~isa(value,'numeric'), error('value must be numeric'); end
            [M,N]=size(value);
            if M~=1 && N~=1, error('value must be one-dimensional'); end
            obj.poles=value;
        end
        function obj = set.zeros(obj,value)
            if ~isa(value,'numeric'), error('value must be numeric'); end
            [M,N]=size(value);
            if M~=1 && N~=1, error('value must be one-dimensional'); end
            obj.privateZeros=value;
        end
        function obj = set.outUnits(obj,value)
            if ~isa(value,'char'), error('value must be a character string'); end
            obj.outUnits=value;
        end
        function obj = set.inUnits(obj,value)
            % Convert input (physical) units to a different value
            % If both newunits and obj.inUnits are one of the
            % DEFINED UNITS (see following), will change gain, poles
            % and zeros as appropriate.
            % DEFINED UNITS: 'm', 'm/s', 'm/s^2', 'nm', 'nm/s' nm/s^2'
            %                'gal', 'mgal', 'ugal'
            if ~isa(value,'char'), error('value must be a character string'); end
            if ~isempty(obj.inUnits)
                conv = unitConversion(obj.inUnits,value);
                if ~isempty(conv),
                    obj = obj*conv; % Adds zero-value poles or zeros
                    % Remove redundant zero-value poles and zeros
                    while any(obj.poles==0) & any(obj.privateZeros==0) % remove canceling poles and zeros at 0 Hz
                        nulls=obj.poles==0;
                        iNulls=find(nulls);
                        if length(iNulls)>1
                            for i=2:length(iNulls)
                                nulls(iNulls(i))=0;
                            end
                        end
                        obj.poles=obj.poles(find(~nulls));
                        
                        nulls=obj.privateZeros==0;
                        iNulls=find(nulls);
                        if length(iNulls)>1
                            for i=2:length(iNulls)
                                nulls(iNulls(i))=0;
                            end
                        end
                        obj.privateZeros=obj.privateZeros(find(~nulls));
                    end    
                end                
            end
            obj.inUnits=value;
        end
        
        %% ACCESS METHODS FOR DEPENDENT PROPERTIES
        function val = get.zeros(obj)
            val=obj.privateZeros;
        end
        
        %% DISPLAY AND CHAR METHODS
        function txt = char(obj,padding)
            % Make character string describing the object
            % padding pads the left side, to give an offset
            if ~exist('padding','var'), padding=0; end
            pformat=sprintf('%%%ds',padding);
            pad=sprintf(pformat,'');
            txt=[];
            if length(obj)==1,  % Just one ts_response
                txt = [txt sprintf('%s%8s: %-12g\n', pad,'gain',obj.gain)];
                txt = [txt sprintf('%s%8s: %s\n',    pad,'poles',sprintf(' %.4g%+.4gi',[real(obj.poles(:)) imag(obj.poles(:))]'))];
                txt = [txt sprintf('%s%8s: %s\n',    pad,'zeros',sprintf(' %.4g%+.4gi',[real(obj.zeros(:)) imag(obj.zeros(:))]'))];
                txt = [txt sprintf('%s%8s: ''%s''\n',pad,'inUnits',obj.inUnits)];
                txt = [txt sprintf('%s%8s: ''%s''',pad,'outUnits',obj.outUnits)];
            else  % Array of ts_responses
                tmp=[]; for i=1:length(obj), tmp=[tmp sprintf('%-13g ',obj(i).gain)]; end
                txt=[txt sprintf('%s%8s: %s\n',pad,'gain',tmp)];
                for i=1:length(obj), N(i)=length(obj(i).poles); end
                txt=[txt sprintf('%s%8s: %s elements\n',pad,'poles',sprintf('%-13d ',N))];
                for i=1:length(obj), N(i)=length(obj(i).zeros); end
                txt=[txt sprintf('%s%8s: %s elements\n',pad,'zeros',sprintf('%-13d ',N))];
                tmp=[]; for i=1:length(obj), tmp=[tmp sprintf('%-13s ',['''' obj(i).inUnits ''''])]; end
                txt=[txt sprintf('%s%8s: %s\n',pad,'inUnits',tmp)];
                tmp=[]; for i=1:length(obj), tmp=[tmp sprintf('%-13s ',['''' obj(i).outUnits ''''])]; end
                txt=[txt sprintf('%s%8s: %s\n',pad,'outUnits',tmp)];
            end
        end
        function disp(obj)
            % Displays 'char' text when object is not followed by a semicolon
            disp(char(obj));
        end
        
        %% OTHER METHODS
        function result = eq(a,b)
            % Implements a==b
            % returns 1 if a is the same as b, 0 otherwise
            if (~isa(b,'ts_response')),
                error('second object must also be ts_response');
            end
            result=1;
            
            if a.gain ~= b.gain,					result=0; return; end
            if length(a.poles) ~= length(b.poles),	result=0; return; end
            if length(a.zeros) ~= length(b.zeros),	result=0; return; end
            for i=1:length(a.poles),
                if a.poles(i) ~= b.poles(i),		result=0; return; end
            end
            for i=1:length(a.zeros),
                if a.zeros(i) ~= b.zeros(i),		result=0; return; end
            end
            
        end
        function result = ne(a,b)
            % Implements obja ~= objb
            % returns 1 if a not the same as b, 0 otherwise
            
            result = ~(a==b);
        end
        plot(varargin)                    % DEFINED IN SEPARATE FILE
        writeGSE2_PZ(obj,fname,sd,rf,st,cmp,inS,outS,styp,dt,dg,cm); % Defined in separate file
        writeSAC_PZ(obj,fname,convert,norm_freq,inst_gain); % Defined in separate file
        writeSEISAN_PZ(obj,fname);     % Defined in separate file
        function [normalizer, gain] = getNorm(obj,freq)
            % Return normalization factor and gain at a given frequency (Hz)
            % 
            % Usage: [normalizer, gain] = getNorm(obj,freq)
            omega_norm = freq*2*pi;
            s=sqrt(-1)*omega_norm;
            normalizer=1;
            for p = obj.poles, normalizer = normalizer*(s-p); end
            for z = obj.privateZeros, normalizer = normalizer/(s-z); end
            gain=obj.gain/normalizer;
        end
        function fpout = mtimes(fpa,fpb)
            % Implements obja * objb
            %
            % Example: newfilt=filta*filtb;
            %
            % Multiplies the gains, concatenates the poles and zeros
            %
            if isa(fpa,'ts_response') && isa(fpb,'ts_response'),
                fpout=ts_response();
                fpout.gain=fpa.gain*fpb.gain;
                fpout.poles=[fpa.poles fpb.poles];
                fpout.privateZeros=[fpa.privateZeros fpb.privateZeros];
                if strcmp(fpa.inUnits,fpb.outUnits)
                    fpout.inUnits=fpb.inUnits;
                    fpout.outUnits=fpa.outUnits;
                elseif strcmp(fpb.inUnits,fpa.outUnits)
                    fpout.inUnits=fpa.inUnits;
                    fpout.outUnits=fpb.outUnits;
                elseif isempty(fpa.inUnits)&&isempty(fpa.outUnits)   % One filter does not not change units, use units from other
                    fpout.inUnits=fpb.inUnits;
                    fpout.outUnits=fpb.outUnits;
                elseif isempty(fpb.inUnits)&&isempty(fpb.outUnits)   % One filter does not not change units, use units from other
                    fpout.inUnits=fpa.inUnits;
                    fpout.outUnits=fpa.outUnits;
                else
                    ts_warning('One response''s input Units does not match the other response''s output Units.  Setting input and output Units to UNKNOWN');
                    fpout.inUnits='UNKNOWN';
                    fpout.outUnits='UNKNOWN';
                end
                %fpout=ts_response('PZ',fpa.gain * fpb.gain, [fpa.poles fpb.poles], [fpa.privateZeros fpb.privateZeros]);
            elseif isa(fpa,'double') && length(fpa)==1,
                fpout=fpb; mulfact=fpa;
                fpout.gain=fpout.gain*mulfact;
            elseif isa(fpb,'double') && length(fpb)==1,
                fpout=fpa; mulfact=fpb;
                fpout.gain=fpout.gain*mulfact;
            else
                error('Both objects must be ts_response objects, or one a scalar');
            end
        end
        function resp = response(obj,fvect)
            % Return instrument response for a range of frequencies
            %
            % Usage:
            %	resp = response(filtp,freqs);
            %	Calculates response of a set of filter parameters
            %		filtparm is an ts_response object
            %		freqs is a vector of frequencies (1/s)
            if ~exist('fvect','var'),
                maxf=100;fstep=maxf/100; fvect=fstep:fstep:maxf;
            end
            onevect = ones(size(fvect)); omega = 2*pi*fvect;
            i = sqrt(-1);
            % Gain
            resp = obj.gain * onevect;
            % Poles
            for p = obj.poles,
                resp = resp ./ (i*omega - p);
            end
            % Zeros
            for z = obj.privateZeros,
                resp = resp .* (i*omega - z);
            end
        end
    end % public methods
    
    %% PRIVATE METHODS
    methods (Access = private)
        obj = readSACPZfile(fname);     % Defined below
        obj = readGSE2PZfile(fname);     % Defined below
        obj = clearBWvalues(obj)
    end
    
    %% STATIC PRIVATE METHODS (don't need object, call using ts_response.function())
    methods (Static, Access = private)
        [gain, poles, zs] = unitConversion(inUnits,outUnits) % Defined below
    end
    %% STATIC  METHODS (don't need object, call using ts_response.function())
    methods (Static)
        function [obj,GSE] = readGSE(filename)
            % Returns ts_response object plus structure with other information
            % U sage: [obj,GSE]=readGSE2(filename)
            %   GSE is a structure with the following fields:
            %       startDate   Date from which these values apply (ts_datetime object)
            %       refFreq     Frequency at which to give the system response (nm/c)
            %       station     Station name
            %       comp        Component name (up to three characters)
            %       inSPS       Sample rate at digitizer input
            %       outSPS      Sample rate at digitizer output
            %       sensorType  Sensor type (i.e., 'STS2' 'T240' 'L-28')
            %       digType     Digitizer type (i.e. 'DM-24', 'L-CHEAPO')
            %       digGain     Digitizer gain (counts/V)
            %       comments    Any comments in GSE2 file
            [gain,ps,zs,GSE]=readGSE2PZfile(filename);
            obj=ts_response('PZ','nm','counts',gain,ps,zs);
            return
        end
    end
    
end	% classdef

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [gain,ps,zs] = readSACPZfile(fname)
    % read in pole-zero information from a SAC pole-zero file:
    % Frequencies are specified in omega format
    % Poles are specified by the line POLES n, where n is the number of poles,
    % followed by m lines with the real and imaginary parts of each pole.
    % If m<n, remaining poles are assumed to be at the origin
    % Zeros are specified in the same way using the line ZEROS n
    % A multiplicative constant to add is specified using CONSTANT x
    % Up to 15 poles and 15 zeros, I THINK SAC convention is that the response
    % is in velocity (m/s).  AnyPlot doesn't assume anything, you  enter the
    % units as text in the "chCorrectedUnits" variable
    
    t.gain = 1;
    t.poles = [];
    t.privateZeros = [];
    
    inPZ=0;	% set to 1 if reading poles, set to 2 if reading zeros
    nPZ=0; iPZ=0;
    fid = fopen(fname,'r');
    if fid==0, error('Pole-zero file %s not found',fname); end
    ts_debug(1,'Reading pole-zero file %s\n',fname);
    while 1,
        tline=fgetl(fid);
        if ~ischar(tline), break, end
        if length(deblank(tline))<=1, break,end
        ts_debug(1,tline);
        if tline(1)=='*',        % comment line
            continue
        end
        A = strread(tline,'%s');
        %A{1}
        %A = textscan(tline,'%s')
        %A{1}
        %ts_debug(1,'A=%s\n',celldisp(A));
        switch A{1},
            case 'POLES',
                t = cleanUpOld(t,inPZ,nPZ,iPZ);
                inPZ = 1;
                nPZ = str2double(A{2});
                iPZ = 0;
            case 'ZEROS',
                t = cleanUpOld(t,inPZ,nPZ,iPZ);
                inPZ = 2;
                nPZ = str2double(A{2});
                iPZ = 0;
            case 'CONSTANT'
                t = cleanUpOld(t,inPZ,nPZ,iPZ);
                t.gain = sscanf(tline,'CONSTANT %f');
            case '*'
                t = cleanUpOld(t,inPZ,nPZ,iPZ);
                t.gain = sscanf(tline,'CONSTANT %f');
            otherwise;
                omega = str2double(A{1}) + sqrt(-1)*str2double(A{2});
                % f = omega/(2*pi);
                if inPZ==1,
                    t.poles=[t.poles omega];
                    iPZ=iPZ+1;
                elseif inPZ==2,
                    t.privateZeros=[t.privateZeros omega];
                    iPZ=iPZ+1;
                else
                    error('a non-command line occured while not in list mode')
                end
        end
    end
    
    ps = t.poles(:)';
    zs = t.privateZeros(:)';
    gain=t.gain;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [gain,ps,zs,GSEprops] = readGSE2PZfile(fname)
    % read response from a GSE 2.0 pole-zero file
    % Usage: [gain,ps,zs,GSEprops] = readGSE2PZfile(fname)
    %
    %   gain,ps,zs are the gain, poles and zeros
    %   GSEprops is a structure with the following fields:
    %           startDate   Date from which these values apply (ts_datetime object)
    %           refFreq     Frequency at which to give the system response (nm/c)
    %           station     Station name
    %           comp        Component name (up to three characters)
    %           inSPS       Sample rate at digitizer input
    %           outSPS      Sample rate at digitizer output
    %           sensorType  Sensor type (i.e., 'STS2' 'T240' 'L-28')
    %           digType     Digitizer type (i.e. 'DM-24', 'L-CHEAPO')
    %           digGain     Digitizer gain (counts/V, object's gain will be divided by this
    %                           to calculate the sensor gain)
    % The GSE 2.0 response format is:
    % "CAL2" station component sensorType refgain(nm/c) refFreq(Hz) outputSPS startDate
    % "PAZ2  1 V"  sensorNormConst(V/nm)   nPoles  nZeros  "Laplace transform"
    % real(Pole1)  imag(Pole1)
    % real(Pole2)  imag(Pole2)
    % ....
    % real(Zero1)  imag(Zero1)
    % read(Zero2)  imag(Zero2)
    % ...
    % "DIG2  2" digitizerGain(c/V)  inputSPS  digitizerType
    %
    % Frequencies are specified in radians
    %
    % The file specifications come from
    % http://www.seismo.ethz.ch/prod/autodrm/manual/CRP243.OpsAnx3.A4.pdf
    %
    % Note that the GSE2.1 format is different enough to be unreadble by a
    % 2.0 reader, but has no tag to distinguish from 2.0!
    % http://www.seismo.ethz.ch/prod/autodrm/manual/provisional_GSE2.1.pdf
    
    
    if ~exist('fname','var')
        fname=[]; end
    if isempty(fname);
        [FileName,PathName]=uigetfile({'*_GSE','GSE PZ files'},'Select a GSE 2.0 Pole-Zero File');
        fname=fullfile(PathName,FileName);
    end
    fid=fopen(fname,'r');
    if fid==0, error('GSE2.0 PZ file %s not found',fname);
    else
        ts_debug(1,'Reading pole-zero file %s\n',fname);
    end
    
    GSEprops.comments='';
    line=fgetl(fid);
    % LINE 1: GENERAL INFORMATION (none of this is stored, but total gain
    % is compared with component gains
    if ~strcmp(line(1:4),'CAL2'),
        error('Line 1 does not start with CAL2');
    else
        station=strtrim(line(6:10));
        comp=   strtrim(line(12:14));
        %auxID=  strtrim(line(16:19));
        instType=strtrim(line(21:26));
        calib=   str2double(line(28:37));    % system sensitivity (nm/count) at reference period
        %calib=   str2double(line(28:42));    % GSE 2.1
        calPer=  str2double(line(39:45));    % calibration reference period (seconds)
        %calPer=  str2double(line(44:50));    % GSE2.1
        outSPS=  str2double(line(47:56));
        %outSPS=  str2double(line(52:62));    % GSE 2.1
        onDate=  strtrim(line(58:67));
        %onDate=  strtrim(line(64:73));    % GSE 2.1
        onTime=  strtrim(line(69:73));
        %onTime=  strtrim(line(75:79));    % GSE 2.1
        %offDate= strtrim(line(75:84));
        %offDate= strtrim(line(81:90));    % GSE 2.1
        %offTime= strtrim(line(86:90));
        %offTime= strtrim(line(92:96));    % GSE 2.1
        
        ts_verbose(0,'CAL2 line: station comp instType refGain(nm/cnt) @Hz   outSPS   startDate');
        ts_verbose(0,'           %-5s   %-3s  %-7s  %-12g    %-5g %-8g %s %s', ...
            station,comp,instType,calib,1/calPer,outSPS,onDate,onTime);
    end
    
    %% READ IN RESPONSE INFORMATION: earth input is assumed to be in nm
    %% displacement
    line=fgetl(fid);
    pazDone=0; digDone=0;
    while 1,
        if isempty(line), continue
        else
            c=strtrim(line); % remove white spaces
            if c(1)=='('     % Comment line
                if c(end)~=')'
                    error('Comment not closed by parentheses: "%s"',line);
                end
                ts_verbose(0,'COMMENT: %s',c);
                comment=strtrim(c(2:end-1));
                if isempty(GSEprops.comment)
                    GSEprops.comment=comment;
                else
                    if ~iscell(GSEprops.comment)
                        GSEprops.comment={GSEprops.comment};
                    end
                    GSEprops{length(GSEprops.comment)+1}=comment;
                end
                continue
            end
        end
        switch(line(1:4))
            case 'FIR2', error('Finite Impulse response not implemented')
            case 'GEN2', error('Generic Response not implemented')
            case 'FAP2', error('Frequency, Amplitude, Phase Response not implemented')
            case 'PAZ2'     % POLE-ZERO RESPONSE (usually SEISMOMETER) [Same for GSE 2.0 and 2.1]
                if pazDone, error('This code doesn''t handle Multiple Pole-Zero responses'); end
                %sNum=str2double(line(6:7));    % Stage sequence number (unused)
                oUnits=line(9);
                if oUnits ~= 'V', error('outUnits are %s, function only written for "V"',oUnits); end
                sFactor=str2double(line(11:25));
                %deci=str2double(line(27:30));      % Unused
                %gCorr=str2double(line(32:39));   % Group correction applied (seconds)
                nPoles=str2double(line(41:43));
                nZeros=str2double(line(45:47));
                descript=strtrim(line(49:end));
                % sFactor = ounits/nm
                j=sqrt(-1);
                ps=zeros(1,nPoles);
                for i = 1:nPoles,
                    line=fgetl(fid);
                    val=sscanf(line,'%f %f');
                    ps(i)=val(1)+j*val(2);
                end
                zs=zeros(1,nZeros);
                for i = 1:nZeros,
                    line=fgetl(fid);
                    val=sscanf(line,'%f %f');
                    zs(i)=val(1)+j*val(2);
                end
                ts_verbose(0,'PAZ2 line: %d Poles, %d Zeros, "%s"',nPoles, nZeros, descript);
                pazDone=1;
            case 'DIG2',    % Digitizer response (only one) [Same for GSE 2.0 and 2.1]
                if digDone, error('Multiple DIG2 lines'); end
                %sNum=str2double(line(6:7));    % Stage sequence number
                digGain=str2double(line(9:23));
                inSPS=  str2double(line(25:35));
                descrip=strtrim(line(37:end));
                ts_verbose(0,'DIG2 line: input samprate = %g, digitizer type = ''%s''', inSPS,descrip);
                digDone=1;
            otherwise
                ts_warning('Unknown GSE2 line: ''%s''',line(1:4));
        end
        line=fgetl(fid);
        if ~ischar(line), break, end
    end
    
    % Calculate and verify gain
    % We want gain in counts/nm, digGain is counts/V, sensNormConst is V/nm
    % Check using refGain (nm/count) at refFreq
    gain = digGain*sFactor;
    tester=ts_response('PZ','nm','counts',gain,ps,zs);
    testGain=abs(tester.response(1/calPer));
    if testGain*calib >1.01 || testGain*calib < 0.99;
        ts_warning('Product of stage gains (%g nm/c) disagrees with calibration gain (%g nm/c) at ref freq (%g Hz)', ...
            1/testGain,calib,1/calPer);
    end
    
    % Fill the GSEprops structure
    %onDate
    %onTime
    GSEprops.startDate  = ts_datetime([onDate ' ' onTime]);
    GSEprops.refFreq    = 1/calPer;
    GSEprops.station    = station;
    GSEprops.comp       = comp;
    GSEprops.inSPS      = inSPS;
    GSEprops.outSPS     = outSPS;
    GSEprops.sensorType = instType;
    GSEprops.digType = descrip;
    GSEprops.digGain = digGain;
    
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function t = cleanUpOld(t,inPZ,nPZ,iPZ)
    if inPZ==0; return; end
    while iPZ < nPZ,
        if inPZ==1,
            t.poles=[t.poles 0];
        elseif inPZ==2,
            t.privateZeros=[t.privateZeros 0];
        else
            error('invalid inPZ value: %d',inPZ);
        end
        iPZ = iPZ+1;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function conv = unitConversion(inUnits,outUnits)
    [igain,inz,isuccess]=refMeter(inUnits);
    [ogain,onz,osuccess]=refMeter(outUnits);
    
    if ~isuccess || ~osuccess,
        ts_warning('Did not convert');
        conv=[];
        return
    end
    
    gain = ogain/igain;
    % Everything here is flipped because we are changing the denominator
    % ("desired") in the response
    relnz = inz-onz;
    if relnz>0,     zs=zeros(1,relnz);  ps=[];
    elseif relnz<0, ps=zeros(1,-relnz); zs=[];
    else            ps=[];              zs=[];
    end
    conv=ts_response('PZ',outUnits,inUnits,gain,ps,zs);
end

function [g,nz,success]=refMeter(units)
    switch lower(units)
        case 'm',       g=1;    nz=0;  success=1;
        case 'm/s',     g=1;    nz=1;  success=1;
        case 'm/s^2',   g=1;    nz=2;  success=1;
        case 'nm',      g=1e-9; nz=0;  success=1;
        case 'nm/s',    g=1e-9; nz=1;  success=1;
        case 'nm/s^2',  g=1e-9; nz=2;  success=1;
        case 'gal',     g=1e-2; nz=2;  success=1;   % 1 cm/s^2
        case 'mgal',    g=1e-5; nz=2;  success=1;
        case 'ugal',    g=1e-8; nz=2;  success=1;
        otherwise, ts_warning('Invalid unit for conversion: %s',units);
            g=1; nz=0; success=0; % don't change anything
    end
end

function ps=butterPoles(nPoles,om)
    if length(nPoles)>1, error('length(nPoles) > 1'); end
    if nPoles==0,ps=[]; return; end
    if length(om)>1, error('length(om) > 1'); end
    phase=((2*(1:nPoles)+nPoles-1)*pi)/(2*nPoles);
    normPoles=exp(1i*phase);
    %phase90=phase*180/pi   % [phase in degrees for debugging
    ps=om*normPoles;
    ps=ps(:)';  % Force row matrix
end
