function xferfun = calcXF(obj,varargin)
    % CALCXF Calculate the transfer function between 2 channels
    %			of a ts_spectra object
    % xfobj = calcXF(obj,drivechan,respchan,...)
    %	output:
    %		xfobj is an ts_xferfun object
    %	input:
    %		obj is an ts_spectra object
    %		drivechan is the number of the driving channel
    %		respchan is the number of the response channel
    %	Optional inputs
    %		'noisechan'		next argument says which channel to assume contains all noise
    %			'response'	[default] Assume that all noise is on the response channel
    %			'driving'	Assume that all noise is on the driving channel
    %			'equal'		Assume the same signal/noise on the driving  and response channels
    %			'unknown'	Make no assumption about noise
    %		'setnoiselevels'	Define the noise levels for each channel.
    %						Next argument is a cell array containing noise power spectral density
    %						(PSD) levels for the driving and response channels as follows:
    %						{[drivefreqs drivelevels],[respfreqs resplevels]}
    %						The noise PSD units are the same as the data PSDs
    %						Values are linearly interpolated in the log(PSD) domain
    %       'output_compliance'
    %           wdepth:     Calculate normalized seafloor compliance
    %                       Assumes that the driving channel is ocean surface
    %                       gravity waves and the response channel
    %                       is seafloor motion.  Also removes the
    %                       calculated gravitational attraction of the
    %                       ocean surface gravity waves.
    %		'leaveGravAttract'
    %             1:        Do NOT remove the grav attraction of ocean
    %                       surface gravity waves.  Only matters if
    %                       'compliance' is set
    numargs = length(varargin);
    if numargs < 2,
        error('You must specify an ts_spectra object, a drive chan, and a response chan');
    end
    dch = varargin{1};
    rch = varargin{2};
    if length(dch) ~=1 || length(rch) ~= 1,
        error('drive and response channels must be scalar');
    end
    
    
    % First, set  defaults
    noisechan = 'response';
    calcCompliance=0;
    removeGravAttract=1;    % Only used if compliance=1
    
    % read in data parameters
    for i = 3:2:length(varargin);
        switch(lower(varargin{i}))
            case 'noisechan',
                noisechan = varargin{i+1};
            case 'setnoiselevels',
                noisechan = 'set';
                driveNoise = varargin{i+1}{1};
                respNoise = varargin{i+1}{2};
            case 'output_compliance',
                wdepth = varargin{i+1};
                if ~strcmp(obj.sinfo(dch).origunits,'Pa')
                    error('To calculate compliance, drive channel units must be "Pa" (they are "%s")',obj.singo(dch).origunits)
                end
                respUnits=obj.sinfo(rch).origunits;
                if ~any(strcmp(respUnits,{'m/s^2','m/s','m'})),
                    error('To calculate compliance, response channel units must be "m/s^2", "m/s" or "m" (they are "%s")',respUnits)
                end
                calcCompliance=1;
            case 'leaveGravAttract'
                removeGravAttract=0;
            otherwise,
                error(['Unknown parameter: ' varargin{i}]);
        end
    end
    
    ts_verbose(1,'Calculating transfer function between channels %d and %d, assumed noise channel is: %s',dch,rch,noisechan);
    
    coh =   squeeze(obj.coherdata(rch,dch,:))';
    cohmagsq =   abs(coh).*abs(coh);
    drive_spect = obj.spectdata(dch,:);
    resp_spect =  obj.spectdata(rch,:);
    nwinds =  obj.nWindows;
    
    gooddata = abs(coh) > obj.cohsigniflevel;
    onevect = ones(size(cohmagsq));
    errbase = sqrt((onevect - cohmagsq)./(2*nwinds*cohmagsq));
    RespOverDrive = sqrt(resp_spect ./ drive_spect);
    
    switch noisechan,	% The following are from Bendat&Piersol "Random Data" 1986,
        %						pp 176-181 (xfs) and pp 317, Table 9.6
        case 'response',
            xf= RespOverDrive .* coh;
            xferr = abs(xf) .* errbase;
        case 'driving',
            xf= RespOverDrive ./ coh;
            xferr = abs(xf) .* errbase;
        case 'equal',
            xf    = RespOverDrive;
            xferr = abs(xf .* errbase);
        case 'unknown',
            xf    = RespOverDrive;
            % This is a VERY ad-hoc error estimate
            maxerr =  abs(coh.^(-1)) + errbase;
            minerr =  abs(coh)      - errbase;
            xferr = abs(xf .* (maxerr-minerr)/2);
        case 'set',
            xf = setXF_FromNoiseLevels(obj.freqs,coh, ...
                drive_spect,resp_spect, ...
                driveNoise,respNoise);
            xferr = abs(xf .* errbase);
        otherwise, 		error(['Invalid: noisechan = ' noisechan]);
    end
    
    drive_info=struct('name',obj.sinfo(dch).name,'units',obj.sinfo(dch).origunits);
    resp_info =struct('name',obj.sinfo(rch).name,'units',obj.sinfo(rch).origunits);
    
    if calcCompliance,
        [multiplier,drive_info,resp_info,gooddata]=compliance_conversion(obj.freqs, ...
            wdepth,gooddata, ...
            drive_info,resp_info);
        xf=xf.*multiplier;
        xferr=xferr.*multiplier;
        
        if removeGravAttract,
            a_w=calc_gravAttract(obj.freqs, wdepth,drive_spect);
            % Compliance should be (a_meas-a_w)/(om^2*p_meas)
            % = xf-a_w/(om^2*p_meas)
            % note that xf is negative, so subtracting (positive) a_w
            % actually increases the compliance amplitude!
            if strcmp(noisechan,'response')
                xf=xf-a_w/(4*pi*pi*obj.freqs.^2.*sqrt(drive_spect));
            else
                disp('BEWARE!  grav attraction calculation is not corrected for errors in pressure spectrum')
                xf=xf-a_w/(4*pi*pi*obj.freqs.^2.*sqrt(drive_spect));
            end
            % xferr may need to be adjusted, too
        end
    end
    
    
    xferfun=ts_xferfun({dch,drive_info.name,drive_info.units},...
        {rch, resp_info.name, resp_info.units}, ...
        xf, xferr,...
        obj.freqs, gooddata);
end

function gravAttract=calc_gravAttract(freqs, H,Pspect)
    % Calculate the gravitational attraction of ocean surface waves
    % Compliance will be HIGHER at low frequencies after removal
    % H is the water depth
    % Uses two equations from Crawford et al., 1998 (on corrected in Zha et
    % al., 2014):
    %   a_w = 2*pi*G*rho_w*exp(-k*H)*h_w; % Equation 2, Zha et al. 2014
    %   P_sf=rho_w*g*h_w/cosh(k*H)
    % => a_w = (pi*G/g)*P_sf*(1+exp(-2*k*H))
    G=6.673e-11;
    g=9.8;
    k=gravd(2*pi*freqs,H);
    gravAttract = (pi*G/g)*sqrt(Pspect).*(1+exp(-2*k*H));
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [multiplier,drive,resp,gooddata]=compliance_conversion(freqs,...
        wdepth,gooddata,drive,resp)
    % Calculate values to convert transfer function to compliance
    k=gravd(2*pi*freqs,wdepth);
    if ~strcmp(drive.units,'Pa')
        error('Driving channel units are not "Pa"!')
    end
    if strcmp(resp.units,'m'),
        multiplier=k;
    elseif strcmp(resp.units,'m/s')
        multiplier=k./(2*pi*freqs);
    elseif strcmp(resp.units,'m/s^2')
        multiplier=k./(2*pi*freqs).^2;
    else
        error('Unuseable respUnits: %s',respUnits)
    end
    resp.name='Displacement*wavenumber';
    resp.units='1';
    drive.name='Pressure';
    gooddata=and(gooddata,2*pi./k>wdepth);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xf = setXF_FromNoiseLevels(freqs,coh,drive,resp,estDriveNoise,estRespNoise)
    assert( estDriveNoise(1,1) <= min(freqs), ...
        'Lowest driving Noise freq (%g Hz) is above lowest data frequency (%g Hz)', ...
        estDriveNoise(1,1),min(freqs));
    assert(estDriveNoise(end,1) >= max(freqs), ...
        'Highest driving Noise freq (%g Hz) is below highest data frequency (%g Hz)', ...
        estDriveNoise(end,1),max(freqs));
    assert(estRespNoise(1,1) <= min(freqs),  ...
        'Lowest resp Noise freq ( %g Hz) is above lowest data frequency (%g Hz)', ...
        estRespNoise(1,1),min(freqs));
    assert(estRespNoise(end,1) >= max(freqs), ...
        'Highest resp Noise freq (%g Hz) is below highest data frequency (%g Hz)', ...
        estRespNoise(end,1),max(freqs));
    
    %whos, log10(estDriveNoise(:,1)), log10(estDriveNoise(:,2)), log10(freqs)
    % Interpolate noise for all PSD freqs
    driveNoiseB = 10.^( interp1(log10(estDriveNoise(:,1)),log10(estDriveNoise(:,2)),log10(freqs)) );
    respNoiseB =  10.^( interp1(log10(estRespNoise(:,1)), log10(estRespNoise(:,2)),log10(freqs))  );
    
    % Calculate factor to multiply estimated noise by to matches coherence
    % From Bendat and Piersol pages 180-181 and my notebook D9, pages 113-114
    cohsq = abs(coh).^2;	% I use complex coherence function, they use coherence ("coherency squared") function (see section 5.2.4)
    a = -driveNoiseB.*respNoiseB;  b = driveNoiseB.*resp + respNoiseB.*drive; c=drive.*resp.*(cohsq-1);
    multfact(1,:) = (-b + sqrt(b.*b - 4*a.*c))./(2*a);
    multfact(2,:) = (-b - sqrt(b.*b - 4*a.*c))./(2*a);
    for iFreq= 1:length(freqs),
        [~,i]=min(abs(log(multfact(:,iFreq))));		% find multiplier that changes the least;
        noiseMult(iFreq) = multfact(i,iFreq);
    end;
    corrDriveNoise = driveNoiseB.*noiseMult;
    corrRespNoise = respNoiseB.*noiseMult;
    driveSignal = drive - corrDriveNoise;
    respSignal  = resp - corrRespNoise;
    
    checker=(1+corrDriveNoise./driveSignal + corrRespNoise./respSignal + (corrDriveNoise.*corrRespNoise)./(driveSignal.*respSignal)).^(-1);
    
    xf = sqrt(respSignal./driveSignal);
    
    figure(20); clf
    subplot(2,2,1);
    loglog(freqs,drive,'b-'); hold on
    loglog(freqs,driveSignal,'b--');
    loglog(freqs,driveNoiseB,'r-');
    loglog(freqs,corrDriveNoise,'r--');
    title('Driving channel'); grid on
    axis([min(freqs) max(freqs) .9*min([min(drive) min(driveNoiseB)]) 1.1*max([max(drive) max(driveNoiseB)]) ]);
    subplot(2,2,2);
    loglog(freqs,resp,'b-'); hold on
    loglog(freqs,respSignal,'b--');
    loglog(freqs,respNoiseB,'r-');
    loglog(freqs,corrRespNoise,'r--');
    title('Resp channel'); grid on
    axis([min(freqs) max(freqs) .9*min([min(resp) min(respNoiseB)]) 1.1*max([max(resp) max(respNoiseB)]) ]);
    subplot(2,2,4);
    semilogx(freqs,sqrt(cohsq),'-'); hold on
    h=semilogx(freqs,sqrt(checker),':'); set(h,'LineWidth',2);
    ylabel('Coherency (+(\gamma^2_{rd}(f)))^{0.5}','Interpreter','Latex');    % Latex avoids backwards text
    axis([min(freqs) max(freqs) 0 1]);
    drawnow,
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function K = gravd(W,h)
    %GRAVD Gravity wave wavenumber determination
    % k = rad/meter
    % k = GRAVD(w,h)
    % w= ang freq (can be a vector), h = water depth (meters)
    
    if (length(W) < 1)
        error('You must specify at least one frequency');
    end
    G = 9.79329;
    N = length(W);
    W2 = W.*W;  kDEEP = W2/G;  kSHAL = W/sqrt(G*h);
    erDEEP = ones(size(W)) - G*kDEEP.*dtanh(kDEEP*h)./W2;
    one = ones(size(W));
    d = one; done = zeros(size(W));	nd = find(done == 0);
    k1 = kDEEP;  k2 = kSHAL; e1 = erDEEP; ktemp = done; e2 = done;
    while 1 > 0
        e2(nd) =  one(nd) - G * k2(nd) .* dtanh(k2(nd)*h)./W2(nd);
        d = e2.*e2;
        done = d < 1e-20;
        if done
            K = k2; break;
        end
        nd = find(done == 0);
        ktemp(nd) = k1(nd) - e1(nd).*(k2(nd)-k1(nd))./(e2(nd)-e1(nd));
        k1(nd)=k2(nd);  k2(nd) = ktemp(nd);  e1(nd) = e2(nd);       % Reset values
    end
end

function y = dtanh(x)
    %DTANH	DTANH(X) is the more stable hyperbolic tangent of x.
    
    a = exp(x.*(x<=50));	% Make sure nothing blows up to NaN
    one = ones(size(x));
    y = (abs(x) > 50) .* (abs(x)./x) + (abs(x) <= 50).*((a-one./a)./(a+one./a));
end

