function [axhand,linehand] = plotcoher(obj,varargin)
    %PLOTCOHER Plot coherency stored in an ts_spectra object
    % [ha,hl] = plotcoher(obj, ...)
    %  Inputs:
    % 	obj is an ts_spectra object
    %	... are ts_figureproperties or ts_figureproperties_subplot field, value pairs
    %       (type 'doc ts_figureproperties' or 'doc ts_figureproperties_subplot' for options)
    %  Outputs:
    %	ha is an array of handles to all the subplot axes.
    %	hl is an array of handles to all the plotted lines
    
    clf
    if ~isempty(varargin),
        [obj,figInfo]=setPlotProperties(obj,varargin);
    else
        figInfo=obj.figureprops;
    end
   
    %% Set up variables    
    nChannels = obj.nChannels;
    slev = obj.cohsigniflevel;
    plotasmatrix = figInfo.plotasmatrix;
    counter = 0;
    linehand = [];
    fontsize = figInfo.fontsize;

    %% Plot
    clf
    for i = 1:nChannels,
        % Calculate frequency bounds to plot
        [fmin,fmax]=getFMinMax(obj,i,figInfo);
        for j = i+1:nChannels,
            counter = counter+1;
            [nSubplots,iplot]=getPlotPosition(i,j,counter,nChannels,plotasmatrix);
            [pbox,info] = figInfo.calcplotpos(nSubplots,iplot,'ylabelLeftOnly');

            x=pbox(1); y=pbox(2); w=pbox(3); hght=pbox(4);
 
            % PLOT COHERENCE MAGNITUDE
            pos=[x y+.31*hght w .60*hght];
            h=plotCohMag(obj,figInfo,i,j,pos,fmin,fmax,slev,info);
            linehand=[linehand;h];
            axhand(2*iplot-1) = gca;

            % PLOT COHERENCE PHASE
            pos=[x y w .30*hght];
            h=plotCohPhase(obj,figInfo,i,j,pos,fmin,fmax,slev,info);
            linehand = [linehand; h];
            axhand(2*iplot) = gca;
        end
    end
    
    %drawnow
    
    % ADD TITLE
    figInfo.addtitle('Coherence')
    set(gcf,'Name','plotcoher(ts_spectra)');
end


function [obj,figInfo]=setPlotProperties(obj,varargin)
    figInfo=obj.figureprops;     % Change plot values locally
    for i=1:2:length(varargin),
        if isprop(figInfo,varargin{i})
            eval(sprintf('figInfo.%s = varargin{i+1};',varargin{i}));
        elseif isprop(figInfo.subplot,varargin{i})
            for j=1:figInfo.nSubplots,
                eval(sprintf('figInfo.subplot(j).%s = varargin{i+1};',varargin{i}));
            end
        else
            ts_warning('property ''%s'' not found in figureprops or figureprops.subplot', varargin{i})
        end
    end
end
 
function [fmin,fmax]=getFMinMax(obj,i,figInfo)
    out=figInfo.subplot(i).xyminmax;
    freqs = obj.freqs;
    if isnan(out(1)), fmin = min(freqs); else fmin=out(1); end
    if isnan(out(2)), fmax = max(freqs); else fmin=out(2); end
end

function numcohs=calcNumCohs(numchans)
    numcohs = 0;
    for i = 1:numchans, for j = i+1:numchans, numcohs=numcohs+1; end; end;
end

function [nSubplots,iplot]=getPlotPosition(i,j,counter,numchans,plotasmatrix)
    if plotasmatrix,
        nSubplots=(numchans-1)^2;
        iplot = (numchans-i-1)*(numchans-1)+(j-1);
    else
        nSubplots = calcNumCohs(numchans);
        iplot=counter;
    end
end

function h=plotCohMag(obj,figInfo,i,j,pos,fmin,fmax,slev,info)
    fontsize=figInfo.fontsize;
    axes('position',pos);
    coh = squeeze(obj.coherdata(i,j,:));
    semilogx([fmin;fmax]*ones(1,9),[1;1]*(.1:.1:.9),':k'); hold on	%grid lines
    semilogx([fmin fmax], [slev slev],'r');	% signif level
    h = semilogx(obj.freqs,abs(coh)); 	hold off
    if ~figInfo.plotasmatrix,
        titleText=[obj.sinfo(i).name '-' obj.sinfo(j).name];
        %title(titleText,'FontSize',fontsize);   
        text(0.5,1.0,titleText,'FontSize',fontsize,'Units','Normalized','HorizontalAlignment','center','VerticalAlignment','Baseline');
        if ~info.isLeft,
            set(gca,'YTickLabel',[]); 
        else
            ylabel('Amplitude');
        end
    else
        if info.isInvDiag,
            title(obj.sinfo(j).name,'FontSize',fontsize);
            ylabel(obj.sinfo(i).name,'FontSize',fontsize); 
        else set(gca,'YTickLabel',[]);
        end
    end
    set(gca,'XTickLabel',[]);
    axis([fmin fmax 0 1.01]);
    set(gca,'FontSize',fontsize);
end

function h=plotCohPhase(obj,figInfo,i,j,pos,fmin,fmax,slev,info)
    axes('position',pos);
    coh = squeeze(obj.coherdata(i,j,:));
    fontsize=figInfo.fontsize;
    gooddata = abs(coh) > slev;
    warning off
    NaNArray= double(gooddata)./gooddata;	% Force to double for Matlab 6
    warning backtrace
    h = semilogx(obj.freqs,(180/pi)*angle(coh).*NaNArray);
    if pos(4)<.1, set(gca,'YTick',-180:180:180);
    else
        set(gca,'YTick',-180:90:180);
    end
    set(gca,'YGrid','on');
    if ~figInfo.plotasmatrix,
        if ~info.isLeft,
            set(gca,'YTickLabel',[]);
        else
            ylabel('Phase','FontSize',fontsize);
        end
    else
        if ~info.isInvDiag, set(gca,'YTickLabel',[]); end
    end
    if info.isBottom,
        xlabel('Frequency (Hz)','FontSize',fontsize);
    else
        set(gca,'XTickLabel',[]);
    end
    axis([fmin fmax -181 181]);
    set(gca,'FontSize',fontsize);
end