classdef ts_spectrogram
    % Class for spectrograms
    %  Spectrograms are created by ts_data_channel.calcSpectrogram()
    %
	% ts_spectrogram properties:
    %     data       - Spectrogram data
    %     freqs      - Spectrogram frequencies
    %     starttime  - time of first spectrum     (ts_datetime)
    %     toffsets   - vector of offsets from starttime of each PSD  (seconds)
    %     units      - PSD data units
    %     name       - Channel name
    %     endtime    - READONLY: time of last spectrum
    %     nSpectra   - READONLY: number of spectra in spectrogram
	%
    % ts_spectrogram methods:
    %     ts_spectrogram - Constructor
    %     plus           - Combine two spectrograms
    %     plot           - Plot spectrogram
    %     PPSD           - Plot as probabilistic power spectral density
    %
    % EXAMPLES:
    % % Set up time scale
    % sps=1; days = 30; 
    % firstDay='2007/07/04'; 
    %
    % % Make data
    % t=1:1/sps:days*86400;                 % Set up time vector
    % d=sin(t/(5*86400/pi)).*rand(size(t)); % Make a modulated time signal
    %
    % % Put data into a ts_data channel object and plot 
    % tsc = ts_data_channel(d,sps);         % Create ts_data_channel object
    % tsc.starttime=firstDay;               % Set the data start time
    % figure(1); plot(tsc);                 % Plot time series
    %
    % % Calculate and plot spectrogram
    % sgram=tsc.calcSpectrogram(40000,86400); % Make spectrogram: daily spectra
    %                                         % using 40000-second windows
    % figure(2); plot(sgram);
    
    properties
        data=[];    % Spectrogram data
        freqs;      % Spectrogram frequencies
        starttime;  % time of first spectrum
        toffsets;   % NEW! offsets in seconds of each PSD from starttime
        %timedelta;  % spacing between spectra (seconds)
        units;      % data units
        name;       % name of channel
    end
    properties (Dependent=true)
        endtime;    % time of last spectrum
        nSpectra;   % number of spectra in spectrogram
        %startyear;  % year of first data
        %days;       % vector of julian days for data
    end
    methods
        % Creator
        function obj=ts_spectrogram(name,units,starttime,timedelt,numtimes,freqs)
            % ts_spectrogram constructor
            %
            % Usage:
            %   obj=ts_spectrogram(name,units,starttime,timedelt,numtimes,freqs)
            % where timedelt is in seconds
            if nargin==0; return;
            elseif nargin~=6,
                error('ts_spectrogram() requires 6 arguments:  name,units,starttime, timedelt,numtimes,freqs')
            end
            obj.name=name;
            obj.units=units;
            obj.starttime=ts_datetime(starttime);
            obj.toffsets=(timedelt*(0:numtimes-1))';
            %obj.timedelta=timedelt;
            obj.freqs=freqs;
            obj.data=zeros(numtimes,length(freqs));
        end
        
        function et=get.endtime(obj)
            %et=obj.starttime+(obj.nSpectra - 1)*obj.timedelta;
            et=obj.starttime+max(obj.toffsets);
        end
        
        function ns=get.nSpectra(obj)
            ns=length(obj.data(:,1));
        end
        
        %function sy=get.startyear(obj)
        %    sy=obj.starttime.year;
        %end
        
        %function days=get.days(obj)
        %    st=obj.starttime;
        %    startday = st.jday+(st.hour+(st.minute+st.second/60)/60)/24;
        %    days=startday+(0:obj.nSpectra-1)*obj.timedelta/86400;
        %end
        
        function obj=plus(objA,objB)
            % Combine two spectrograms
            % Spectrograms must not overlap, must have same name,
            % frequencies
            quitError=0;
            if ~isa(objB,'ts_spectrogram'), 
                error('second argument is not a ts_spectrogram object');
            end
            if objA.starttime<objB.starttime,
                if objA.endtime >= objB.starttime,
                    error('spectrograms overlap: first ends at %s and second starts at %s',objA.endtime,objB.endtime); 
                end
            elseif objA.starttime > objB.starttime,
                if objB.endtime >= objA.starttime,
                    error('spectrograms overlap: first ends at %s and second starts at %s',objB.endtime,objA.endtime); 
                end
            else
                error('spectrograms start at same time!'); 
            end
            if ~strcmp(objA.name,objB.name),
                error('spectrograms have different names: "%s" and "%s"',objA.name,objB.name);
            end
            %if ~isequal(objA.freqs,objB.freqs)
            if ~isequal(size(objA.freqs),size(objB.freqs))
                error('freq arrays are not equal length: %d versus %d elements',objA.freqs,objB.freqs);
            end
            
            if objA.starttime< objB.starttime
                ABoffset=objB.starttime-objA.starttime;
                obj=objA;
                obj.data=[obj.data; objB.data];
                obj.toffsets=[obj.toffsets(:);objB.toffsets(:)+ABoffset ];
            else
                ABoffset=objA.starttime-objB.starttime;
                obj=objB;
                obj.data=[obj.data; objA.data];
                obj.toffsets=[obj.toffsets(:);objA.toffsets(:)+ABoffset];
            end
            
        end
        
        function obj=toLogFreqs(obj,nFreqs)
            % Convert spectogram from linearly spaced to log-spaced
            % frequencies
            % Usage: newSGram=sGram.toLogFreqs(nFreqs)
            %
            %   nFreqs = number of log Freqs to use [default=128];
            
            if ~exist('nFreqs','var'), nFreqs=128; end
            
            origLogFreqs=log10(obj.freqs);
            newLogFreqs=min(origLogFreqs) + (0:nFreqs-1)*(max(origLogFreqs)-min(origLogFreqs))/(nFreqs-1);
            newData=interp1(origLogFreqs,obj.data',newLogFreqs);
            obj.data=newData';
            obj.freqs=10.^(newLogFreqs);
         end
        
        function plot(obj)
            %% Plot results
            colormap jet
            %c=clock;
            clf

            [header,ticks,labels]=maketimeticks(obj.starttime,obj.endtime);
            pcolor(obj.freqs,obj.toffsets,10*log10(obj.data)); 
            shading flat,set(gca,'XScale','log')
            set(gca,'YTick',ticks);
            set(gca,'YTickLabel',labels);
            ylabel(header);
            title(obj.name);
            xlabel('Frequency (Hz)')
            
            grid on; set(gca,'Layer','Top')
            h=colorbar();
            set(get(h,'YLabel'),'String',sprintf('PSD (dB ref 1 %s)',obj.units));
            %fname = sprintf('spectrograms.f%d.%04d%02d%02d_%02d%02d.eps',gcf,c(1),c(2),c(3),c(4),c(5));
            %ts_verbose(1,'Saving spectrogram plot to %s',fname);
            %print('-depsc2',fname);
            title(obj.name,'Interpreter','none');
            set(gcf,'Name','ts_Spectrogram.plot()');
        end
        
        function PPSD(obj)
            %% Plot results as probabilistic power spectral density
            colormap hot
            %c=clock;
            clf

            logmin=min(min(10*log10(obj.data(obj.data~=0))));
            logmax=max(max(10*log10(obj.data(obj.data~=0))));
            logdel=(logmax-logmin)/50;
            zscale=logmin:logdel:logmax;
            PPSD=zeros(length(zscale),length(obj.freqs));
            for i=1:obj.nSpectra
                for j=1:length(obj.freqs),
                    zval=10*log10(obj.data(i,j));
                    igrid=round((zval-logmin)/logdel);
                    if igrid<1, igrid=1;
                    elseif igrid>length(zscale), igrid=length(zscale);
                    end
                    PPSD(igrid,j)=PPSD(igrid,j)+1;
                end
            end
            PPSD(PPSD>0)
            whos
            pcolor(obj.freqs,zscale,PPSD); 
            shading flat,set(gca,'XScale','log')
            %set(gca,'YTick',ticks);
            %set(gca,'YTickLabel',labels);
            ylabel(sprintf('PSD (db ref 1 %s)',obj.units));
            title(obj.name);
            xlabel('Frequency (Hz)')
            
            grid on; set(gca,'Layer','Top')
            set(gcf,'Name','ts_Spectrogram.PPSD()');
        end
        
        function out=mean(obj)
            % Return the mean spectra of the spectragram
            spect=mean(obj.data,1);
            out=ts_spectra(obj.freqs,spect,[]);
        end
        
        function out=median(obj)
            % Return the median spectra of the spectragram
            spect=median(obj.data,1);
            out=ts_spectra(obj.freqs,spect,[]);
        end
        
        function out=logmean(obj)
            % Return the mean of the logarithm of the spectragrams
            spect=10.^(mean(log10(obj.data),1));
            out=ts_spectra(obj.freqs,spect,[]);
        end
            
        function obj=cut(obj,starttime,endtime)
            % Returns spectrogram data within a given time range
            %   starttime=ts_datetime object, or seconds from spectrogram start
            %   end time =ts_datetime object or number of seconds from starttime
            %               default= to end of spectrogram (0)
            
            %% VERIFY STARTTIME
            if ~exist('starttime','var')
                error('You must specify a starttime');
            end
            if ischar(starttime)
                starttime=ts_datetime(starttime);
            end
            if isa(starttime,'ts_datetime')
                if starttime < obj.starttime,
                    error('Requested starttime (%s) is before the spectrogram start (%s)', ...
                                starttime.char(),obj.starttime.char())
                elseif starttime > obj.endtime
                     error('Requested starttime (%s) is after the spectrogram end (%s)', ...
                                starttime.char(),obj.endtime.char())
                end
            elseif isnuumeric(starttime)
                if starttime<0,
                    error('Requested a negative starttime! (%g)',starttime);
                elseif starttime > (obj.endtime-obj.starttime)
                    error('Requested a starttime (%g) greater than the total spectrogram length (%g)', ...
                            starttime,(obj.endtime-obj.starttime))
                end
                starttime=obj.starttime+starttime;   % Convert to ts_datetime
            else
                error('starttime is neither numeric nor ts_datetime (%s)',class(starttime));
            end
            
            %% VERIFY ENDTIME
            if ~exist('endtime','var')
                endtime=obj.endtime;
            end
            if ischar(endtime)
                endtime=ts_datetime(endtime);
            end
            if isa(endtime,'ts_datetime')
                if endtime <= starttime,
                    error('endtime (%s) is not after starttime (%s)!',endtime.char(), starttime.char());
                elseif endtime > obj.endtime,
                    error('endtime (%s) is after spectrogram endtime (%s)',endtime.char(),obj.endtime.char())
                end
            elseif isnumeric(endtime)
                if endtime < 0,
                    error('Requesting a negative endtime! (%g)',endtime)
                elseif endtime == 0,
                    endtime=obj.endtime-starttime;
                elseif starttime+endtime > obj.endtime
                    error('starttime (%s) + endtime(%g) = %s: after spectrogram end! (%s)', ...
                            starttime.char(), endtime, char(starttime+endtime),obj.endtime.char())
                end
                endtime=starttime+endtime;  % Convert to ts_datetime
            end    
        
            %% CUT
            [~,iStart]=min(abs(obj.toffsets-(starttime-obj.starttime)));
            if obj.toffsets(iStart)>(starttime-obj.starttime), iStart=iStart-1; end
            if iStart<1, iStart=1;
            elseif iStart> obj.nSpectra; iStart=nSpectra;
            end
            
            [~,iEnd] = min(abs(obj.toffsets-(endtime-obj.starttime)));
            if obj.toffsets(iEnd)<(endtime-obj.starttime), iEnd=iEnd-1; end
            if iEnd<iStart, iEnd=iStart;
            elseif iEnd> obj.nSpectra; iEnd=nSpectra;
            end
            obj.data=obj.data(iStart:iEnd,:);
            obj.starttime=obj.starttime+obj.toffsets(iStart);
            obj.toffsets=obj.toffsets(iStart:iEnd) - obj.toffsets(iStart);
           
            ts_verbose(1,'Extracted spectrogram data from %s to %s',obj.starttime.char(),obj.endtime.char());
        end
    end
end