%% ts_response
% Instrument Response Class
%
%% Description
% *RESPONSE*
%  The response specifies how your data was transformed from
%  physical ("input") values to the stored ("output") values:
%
%  outUnits = inUnits * resp, where:
%
%                   (s-zero1)*(s-zero2)*(s-zero3)*...
%  resp(s) = pzgain*----------------------------------
%                   (s-pole1)*(s-pole2)*(s-pole3)*...
%
% where s=1i*2*pi*frequency
%
%  The reponse is the transfer function FROM the input units TO
%  the output units.
%  We calculate spectra and convert time series to input units by
%  dividing the stored data by this response.
%
% *GAIN VERSUS "PASSBAND" GAIN*
%  The pole-zero gain stored in this class is not the "passband gain"
%  given for standard lowpass, highpass or bandpass filters, because
%  transfer functions do not necessarily have a flat passband in which
%  this gain makes sense.
%  If you want to specify the passband gain, create the object
%    1) using pole-zero notation and specifying omega_ref, or
%    2) using Butterworth filter paramters
%  for details, see the help for the creator method.
%
%% Properties:
%  gain     - Instrument pole-zero gain
%  poles    - Instrument poles (radians)
%  zeros    - Instrument zeros (radians)
%  inUnits  - Units of data at the instrument's input
%              (e.g., physical units)
%  outUnits - Units of stored data (e.g., digital units)
%
%% Methods:
%  Creator
%      ts_response    - Creator
%  Miscellaneous
%      getNorm        - Return normalization factor and gain at a frequency
%      plot           - Plot the response
%  Writing to standard formats
%      writeGSE2_PZ   - Write the response in GSE pole-zero format (v2.0)
%      writeSAC_PZ    - Write the response to a SAC pole-zero file
%      writeSEISAN_PZ - Write the response to a SEISAN pole-zero file
%  Overloaded Matlab methods
%      eq             - Implements "="
%      ne             - Implements "~="
%      mtimes         - Implements "*"
%
%% Examples
% Bandpass filter with 2 poles at 4.5 Hz and 2 at 100 Hz
tsr=ts_reponse('BW','m/s^2','du',1000,[100 100],[4.5 4.5]);
plot(tsr);
%%
% The same thing in pole-zero notation
tsr=ts_response('PZ','m/s^2','du',3.94784e8,-2*pi*[4.5 4.5 100 100],[0 0]);
plot(tsr)
%%
% ALMOST the same thing in pole-zero notation, specifying gain at a frequency
% in the passband (but the passband isn't perfectly flat, so the response
% isn't exactly the same
tsr=ts_response('PZ','m/s^2','du',1000,-2*pi*[4.5 4.5 100 100],[0 0],2*pi*20);
plot(tsr)
