function lcpo2seisan(fname,starttime,endtime,step,buffer,dfiles,actions,destdir)
% LCPO2SEISAN converts LCHEAPO data to SEISAN format
% Usage: lcpo2seisan(starttime,endtime,step,dfiles,actions)
%
% Inputs:
%	startime: start of data to save (ts_datetime object)
%	endtime: end of all data to save (ts_datetime object)
%	step:	length in seconds of each file created
%	buffer:	time in seconds of data to read before and after output times (only
%			useful if you are using processing actions with edge effects
%			you want to remove)
%	dfiles: cell array of ts_file objects (see below)
%	actions: processing actions on each channel before saving
%	destdir: directory in which to put the SEISAN files ('.')
%
% dfiles: each ts_file object contains a lot of information about
%		the instrument.  You should at least do the following:
%	d_a = ts_file(filename,'LCPO');	% creates the object, specifies the filename
%	d_a = set(d_a,'position',[lat lon elev depth] % first 2 decimal degrees,
%												  % last two meters
%	d_a = set(d_a,'synctime',time);		% ts_datetime object
%	d_a = set(d_a,'clockdrift',drift);	% (INSTelapsed-GPSelapsed)/GPSelapsed
%	d_a = set(d_a,'chName',1,'P');		% or whatever
%	d_a = set(d_a,'chName',2,'HX');		% and so on for all the channels
%	
%	you can also specify that you only want to read some of the channels:
%	d_a = set(d_a,'readChannels',[1 2 4]);
%
%	then repeat as necessary for other files, and put them all together	like:
%	dfiles={d_a,d_b,d_c,d_d,...};
%	Even if you only use one datafile, you still need to put it in a cell
%	array:
%		dfiles={d_a};
%
% actions: allows you to process individual channels before saving.  For
%			now, only filtering is allowed.  You must make a cell array
%			with two columns: the first column will be the channel # in the
%			data, the second is a text string that will be processed on that
%			channel as if that channel where an ts_data object named 'mychannel'
%
%		For example: to highpass, 2-pole Butterworth filter the data from channels 11 and 13 at 4.5 Hz:
%			actions = {11, 'filter(mychannel,'BwHigh',2,4.5)'; ...
%					   13, 'filter(mychannel,'BwHigh',2,4.5)'};
%			Note that the channel number corresponds to the absolute
%			channel number in the assembled data.  So if each instrument
%			has 4 channels and you read in all 4 channels from each
%			instrument, channel 11 corresponds to the 3rd instrument's 3rd
%			channel.
%
% BUGS: - does no error checking (times within data range, valid commands,
%			etc)
%		- would be nice if it read a little before and after if it filters
%			channels, to get rid of edge effects.

if starttime+step > endtime, error('starttime + step > endtime'); end
if ~exist('actions'), actions = {}; end
if ~exist('destdir'), destdir = '.'; end

ttime = starttime,
while ttime < endtime-step,
	% READ IN ALL OF THE DATA
	for i = 1:length(dfiles)
		td=ap_readLCPO(dfiles{i},ttime-buffer,step+2*buffer);
		if i==1, tdata=td;
		else, tdata=merge(tdata,td); end
	end
	% MODIFY THE CHANNELS IF NECESSARY
	for i = 1:length(actions),
		ich = actions{i,1};
		cmd = ['mychannel = ' actions{i,2} ';'];
		mychannel = extract(tdata,ich);
		eval(cmd);
		tdata.data(ich,:) = mychannel.data(1,:);
	end
	if buffer>0,
		tdata=cut(tdata,buffer,step);
	end
	nCh = tdata.nChannels;
	fn=sprintf('%04d-%02d-%02d-%02d-%02d-%02d_%03d.SEI', ttime.year,ttime.month,ttime.day, ...
				ttime.hour,ttime.minute,floor(ttime.second),nCh);
	outfname=fullfile(destdir,fn);
	ap_writeSEISAN(tdata,outfname);
	ttime=ttime+step
end
