function convertLCPO_toSDS(dfile,station,network,location,starttime,endtime,synctime,GPSEnd,instEnd,pos,destDir,actions)
    % convertLCPO_toSDS converts LCHEAPO data to SeisComp3 Data Structure
    % Usage: convertLCPO(dfile,station,network,location,starttime,endtime,synctime,GPSEnd,instEnd,pos,destDir,actions);
    %
    % Inputs:
    %	dfile: ts_file object with the filename and component information
    %	station: station name
    %	network: network name
    %	location: location name (2 characters)
    %	starttime: first date to write (ts_datetime object, or '0' for start of data)
    %	endtime: date for which to stop writing (ts_datetime object, or '0' for end of data)
    %	synctime: time at which LCPO was synced to GPS [string, or ts_datetime object]
    %	GPSend: GPS time at which end synchronization was obtained
    %   instEnd: time given by the instrument at the GPSend sync.
    %	pos: OBS position [latitude(decimal degrees), longitude(decimal degrees), depth(meters)]
    %	destDir: root SDS directory.  Default = ./SDS
    %	actions: A text string that allows you to process the data before saving.
    %           The string will be processed on data stored in a ts_data object named 'd'
    %
    %		Examples:
    %			'd=d.filter(''BwHigh'',2,4.5)';                       % hipass filter data at 4.5 Hz:
    %			'd.channel(1)=d.channel(1).filter(''BwHigh'',2,4.5)'; % filter only channel 2
    %			'd=d.resample(''/25'')';                              % resample data at 25-times lower rate
    %
    
    tStart=tic;
    if ts_get('verbose') > 0,
        ts_warning('verbose level is %d, you might want to set it to 0',ts_get('verbose'));
    end;
    if ts_get('debug') > 0,
        ts_warning('debug level is %d, you might want to set it to 0',ts_get('debug'));
    end;
    
    %%% CHECK INPUT PARAMETERS
    if ~exist('station','var'), error('You must specify a station'); end
    if ~exist('network','var'), error('You must specify a network'); end
    if ~exist('location','var'), error('You must specify a location (e.g. ''00'')'); end
    if ~exist('dfile','var'), error('You must specify a dfile'); end
    if ~exist('synctime','var'), error('You must specify synctime'); end
    if ~exist('GPSEnd','var'), error('You must specify GPSend'); end
    if ~exist('instEnd','var'), error('You must specify instEnd'); end
    if ~exist('pos','var'), error('You must specify instrument postion'); end
    if ~exist('starttime','var'), starttime=0; end
    if ~exist('endtime','var'), endtime=0; end
    if ~exist('destDir','var'), destDir='./SDS';
    elseif isempty(destDir), destDir='./SDS';
    end
    if ~exist('actions','var'), actions=''; end
    step=86400;
    
    % Create a diary file and write intro info
    a=clock();
    dfname=sprintf('LCPOtoSDS_%s_%04d%02d%02dT%02d%02d%02.0f.txt',station,a);
    fprintf('Creating diary file: %s\n',dfname)
    diary(dfname)
    fprintf('dfile = %s\n',char(dfile));
    fprintf('station = %s\n',station);
    fprintf('network = %s\n',network);
    fprintf('location = %s\n',location);
    fprintf('synctime = %s\n',char(synctime));
    fprintf('GPSEnd = %s\n',char(GPSEnd));
    fprintf('instEnd = %s\n',char(instEnd));
    fprintf('pos = [%g, %g, %g]\n',pos);
    if isnumeric(starttime),
        fprintf('startExtract = %g\n',starttime);
    else
        fprintf('startExtract = %s\n',char(starttime));
    end
    if isnumeric(endtime),
        fprintf('endExtract = %g\n',endtime);
    else
        fprintf('endExtract = %s\n',char(endtime));
    end
    fprintf('destDir = %s\n',destDir);
    
    % CALCULATE AND PRINT INSTRUMENT DRIFT RATE
    synctime=ts_datetime(synctime); GPSEnd=ts_datetime(GPSEnd); instEnd=ts_datetime(instEnd);
    if (GPSEnd-synctime) < 0,
        error('The GPS end time (%s) is before your sync time (%s)!',char(GPSEnd),char(synctime));
    end
    driftrate=(instEnd-GPSEnd)/(GPSEnd-synctime);
    fprintf('Calculated driftrate = %.5e\n',driftrate);
    
    % PUT STATION, NETWORK AND CLOCK INFO INTO TS_FILE OBJECT
    for i=1:dfile.nChannels,
        dfile.ch_metadata(i).station=station;
        dfile.ch_metadata(i).network=network;
        dfile.ch_metadata(i).coordinates=[pos(1) pos(2) -pos(3) pos(3)];
        dfile.ch_metadata(i).timezero=synctime;
        dfile.ch_metadata(i).clockdrift=driftrate; % Added 2011/12: prior runs did not integrate drift!!!
    end
    %dfile.readchans=readchans;
    
    [starttime,endtime] = getStartEnd(dfile,starttime,endtime,step);
    
    tt=starttime;
    workdir=cd;
    while tt < endtime,
        startday=tt.floor('day'); nextday=startday+step;
        if endtime < nextday,
            endread=endtime;
        else
            endread=nextday;
        end
        % READ THE DATA
        %d = read(dfile, tt,step);
        d = read(dfile, tt,endread-tt);
        % MODIFY THE DATA IF REQUESTED
        if ~isempty(actions)
            eval([actions ';']);
        end
        % WRITE ALL OF STATION'S CHANNELS FOR THIS DAY
        for i=1:d.nChannels,
            ch=d.channel(i);
            ch.metadata.network=network;
            ch.metadata.location=location;
            ch.metadata.station=station;
            
            mydir=fullfile(destDir,sprintf('%d',tt.year),network,station,[ch.metadata.component '.D']);
            if ~isdir(mydir)
                mkdir(mydir);
            end
            fullfname=fullfile(mydir, ...
                sprintf('%s.%s.%s.%s.D.%s', ...
                network,station,location, ...
                ch.metadata.component,ch.starttime.char('%y.%j')));
            d_ch=ts_data(ch);
            [s,w]=system('/usr/local/bin/sac2mseed -h');    % sac2mseed is distributed by IRIS
            if (s ~= 0)
                ts_warning('sac2mseed not found, using writeMSEED (SLOWWWW!)');
                d_ch.writeMSEED(fullfname);
            else
                d_ch.writeSAC('temp.SAC');
                [~,w]=unix(sprintf('/usr/local/bin/sac2mseed -n %s -t %s -c %s -l %s -r 4096 -e 10 -o %s temp.SAC', ...
                    network, station, ch.metadata.component, location, fullfname));
                ts_verbose(0,'Converting to miniSEED: %s',fullfname);
                ts_verbose(1,w);
                unix('rm temp.SAC');
            end
            ts_verbose(1,'Wrote %s',fullfname);
        end
        tt = nextday;
    end;
    te=toc(tStart);
    th=floor(te/3600);ts=te-3600*th; tm=floor(ts/60); ts=ts-tm*60;
    fprintf('Station %s Took %.0f:%02d:%02d\n',station,th,tm,floor(ts));

    %ts_set('verbose',tsv);
    diary off
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [s,e]=getStartEnd(dfile,s,e,step)
    
    [d,dfile] = read(dfile,0,1);	% LCPO file info is returned in custom parameter
    %A = dfile.custom, site_s=A{1}; site_e=A{2};
    site_s=dfile.firstTime; site_e=dfile.lastTime;
    
    if isa(s,'char'), s=ts_datetime(s); end
    if isa(e,'char'), e=ts_datetime(e); end
    
    if isa(s,'ts_datetime'),
        if s < site_s-1/d.channel(1).samprate,
            olds=s;
            saut=step*floor((site_s-s)/step);
            s=s+saut;
            while s < site_s-1/d.channel(1).samprate,
                s=s+step;
            end
            fprintf('Advanced starttime from %s to %s (data starttime=%s)\n', ...
                char(olds),char(s),char(site_s));
        end
        if s > site_e,
            error('Your starttime (%s) is after the data endtime (%s)',char(s),char(site_e));
        end
    else
        if s==0,
            day_s=ts_datetime(site_s.year,site_s.jday,0,0,0);
            offset = site_s - day_s;
            nsteps=ceil(offset/step);
            s=day_s + nsteps*step;
        else
            error('Unknown format for starttime');
        end
    end
    
    if isa(e,'ts_datetime'),
        if e<s,
            error('endtime (%s) is before starttime )%s)',e,s);
        elseif e>site_e
            ts_warning('Requested endtime (%s) is beyond data end (%s), adjusting...', ...
                e,site_e);
            e=site_e;
        end
    else
        if e==0,
            e=site_e;
        else
            etest=s+e;
            if etest > site_e,
                ts_warning('Requested endtime (%d seconds after %s) is beyond data end (%s), adjusting...', ...
                    etest,s,site_e);
                e=site_e;
            else
                e=etest;
            end
        end
    end
end
