function convertUTIG( tfile, startTime, endTime, saveLength, format )
%CONVERTUTIG Convert UTIG OBS data to another data file format
% Usage:
%   convertUTIG(tfile,startTime,endTime,saveLength,format)
% Inputs:
%   tfile      is a ts_file object, should have filenames, synctime,
%               clockdrift, network and instname defined as follows:
%               tfile=ts_file(filenames,'UTIG')
%               tfile=set(tfile,'synctime',...
%               tfile=set(tfile,'network', ...
%               etcetera, type 'doc ts_file' for details
%               filenames is a string with wildcards matching all of the UTIG
%                   OBS filenames (e.g., '*.10'), or a cell array of all of the
%                   filenames (the routine should run faster if they are listed in
%                   chronological order)
%   startTime  is the start time of data to save (may be a string of the form
%                   'YYYY/MM/DD HH:mm:SS')
%   endTime    is the end of data to save
%   saveLength is the length in seconds of each file to output
%   format     is a valid output data format (there most exist a program
%               ts_writeFORMAT).  Valids are 'SAC', 'MSEED'.  Default is
%               MSEED.

tic;
%% PUT DATA INTO APPROPRIATE VARIABLES
startTime=ts_datetime(startTime);   % Convert string to ts_datetime
endTime=  ts_datetime(endTime);
if ~exist('format','var'), format='MSEED'; end

% If filenames is a cell array, force it to be row-oriented
% if iscell(filenames), filenames=filenames{:}'; end

% tfile=ts_file(filenames,'UTIG');
% tfile=set(tfile,'syncTime',syncTime);
% tfile=set(tfile,'clockdrift',clockdrift);

%% CHECK FOR PROBLEMS WITH REQUESTED TIMES
if endTime-startTime < saveLength,
    error('Requested end of data is less than one saveLength greater than requested start of data');
end
[td,fStart,fEnd]=read(tfile,-1);
DEBUG=0;
if DEBUG, fprintf('convertUTIG: fStart=%s, fEnd=%s\n',char(fStart),char(fEnd)); end
if fStart > startTime,
    diffBlocks=ceil((fStart-startTime)/saveLength);
    reqStartTime=startTime;
    startTime = startTime+diffBlocks*saveLength;
    fprintf('You asked for starttime=%s\n',char(reqStartTime));
    fprintf('         Data starts at %s\n', char(fStart));
    fprintf('      Shifting start to %s\n', char(startTime));
end
if fEnd < endTime,
    reqEndTime=endTime;
    endTime=fEnd;
    fprintf('You asked for end time  %s\n', char(reqEndTime));
    fprintf('           Data ends at %s\n', char(fEnd)); 
    fprintf('   Shifting end time to %s\n', char(endTime));
end
if endTime-startTime < saveLength,
    error('End of data is less than one saveLength greater than start of data');
end

%% READ AND WRITE
readTime=startTime;
station=get(tfile,'instname');
while readTime <= endTime-saveLength,
    tdata=read(tfile,readTime,saveLength);
    ts_write(tdata,format,[format '/%y/%m/%d/%y%m%d_%H%M%S_' station]);
    readTime=readTime+saveLength;
end

fprintf('%.1f seconds',toc);
end     % End of function

