function resp=tsresp_LCPO_DPG(calibration)
% Creates a ts_response instance for the specified differential pressure
% gauge and logger
%
% Usage: resp=tsresp_DPG(calibration)
%
% Where calibration is a text string specifying the logger, DPG and test year.
%   valid values are:
%       'L21_DPG5004_2007'  (2007 calibration of DPG 5004 on logger 21)
%       'L22_DPG5018_2007'  (2007 calibration of DPG 5018 on logger 22)
%       'L23_FROM_MOMAR_APG'  (calibration of logger 23 DPG using 2007-8 seafloor APG data)

    switch calibration,
        case 'L21_DPG5004_2007';
            % Simple based on my measurements
            resp=ts_response('PZ','Pa','counts',495,2*pi*[-0.0029,-0.0077],[0,0],2*pi*1);
            % Complicated based on my measurements and Scripps dataless
            % resp=ts_response('PZ','Pa','counts',495,[2*pi*[-0.0029,-0.0077],-57.471,-100,-1655.6],[0,0,-11656.0],2*pi*1);
        case 'L21_LSo7_Rayleigh';
            % Lab measurements corrected for Lso7 Rayleigh wave ratio
            resp=ts_response('PZ','Pa','counts',595,2*pi*[-0.0029,-0.0077],[0,0],2*pi*1);
        case 'L22_DPG5018_2007';
            % Simple based on my measurements
            resp=ts_response('PZ','Pa','counts',510,2*pi*[-0.0022,-0.01],[0,0],2*pi*1);
            % Complicated based on my measurements and Scripps dataless
            %resp=ts_response('PZ','Pa','counts',510,[2*pi*[-0.0022,-0.01],-57.471,-100,-1655.6],[0,0,-11656.0],2*pi*1);
        case 'L22_LSo8_Rayleigh';
            % Lab measurements corrected for Rayleigh wave ratio
            resp=ts_response('PZ','Pa','counts',660,2*pi*[-0.0022,-0.01],[0,0],2*pi*1);
        case 'L22_LSc7_Rayleigh';
            % Lab measurements corrected for LSc7 Rayleigh wave ratio (data
            % very weird/bad)
            resp=ts_response('PZ','Pa','counts',8250,2*pi*[-0.0022,-0.01],[0,0],2*pi*1);
        case 'L23_FROML22_LOWGAINHIGHFREQ';
            % To eye-fit LSb7 data, took L22 response, changed gain 
            % (by 25x!) and the highpass card cornerfreq (from 100 to 10 seconds)
            % WE CHECKED THE CARD AND IT DID NOT HAVE CORRESPONDING STRANGE R-C
            resp=ts_response('PZ','Pa','counts',20,2*pi*[-0.0022,-0.1],[0,0],2*pi*1);
        case 'L23_TRY_MOMAR_APG';
            % Initial value fit LSb7 spectra to JPPW and JPPE spectra, setting card
            % highpass corner frequency to  0.05 rad/s (126 seconds) in
            % agreement with RxC = 2Mx10?F
            resp=ts_response('PZ','Pa','counts',3.3,-[2*pi/455,1/20],[0,0],2*pi*1);
        case 'L23_FROM_MOMAR_APG';
            % Best value Fit LSb7 spectra to JPPW and JPPE spectra, setting card
            % highpass corner frequency to  0.05 rad/s (126 seconds) in
            % agreement with RxC = 2Mx10?F
            resp=ts_response('PZ','Pa','counts',4.8,-2*pi*[1/600,1/126],[0,0],2*pi*1);
        case 'L23_LSb7_Rayleigh_APG';
            % Set card HP corner freq to 0.05 rad/s (126s, R*C = 2Mx10?F)
            % Fit LF Rayleigh wave pressure/accel ratio
            % Then Best Fit JPPW and JPPE IF spectra
            resp=ts_response('PZ','Pa','counts',7.0,-2*pi*[1/600,1/126],[0,0],2*pi*1);
        case 'NONE';
            % Unity gain, no poles or zeros
            resp=ts_response('PZ','Pa','counts',1,[],[],2*pi*1);
        otherwise,
            error('Unknown DPG-logger combination "%s"',calibration);
    end
end

