function tsplot(varargin)
%TSPLOT Graphical user interface for TiSKit
% Syntax:
%  Form 1:
%   tsplot;               Asks for datafile(s) information, then displays data
%  Form 2:
%   tsplot(datafile,readAt,readLen,displayOptions);
%   tsplot({datafile1,datafile2,datafile3, ...},readAt,readLen,displayOptions);
%  Form 3:
%   tsplot(timedata,displayOptions);
%
%  In the first form, the program asks for information about the input data
%  		files, reads them, and displays them
%  In the second form, the program reads in the data and displays it
%  In the third form, the program displays the data (should disable the
%  		dataStart field)
% Inputs:
%   timedata    An ts_data object
%	datafile	An ts_file object
%	readAt		Either a record number (if scalar) or time (ts_datetime) to read at
%	readLen		Data length (in seconds) to read
%   displayOptions Set startup tsplot values for 
%           	'Y Range', 'Segment Length', 'Address', or 'Title'
%              	Example: tsplot(tdata,'Address','1000','Y Range','-10:10')
%
% A guide to the interface: (just notes for now)
%  Clicking on a data line shows the address, time, and y-value
%  Dragging a box from left to right in a plot window zooms in
%  Dragging a box from right to left zooms out to all of the loaded data
%  Y range:
%    Click "Auto" to allow the y axis to fit all the displayed data
%    Enter a number (e.g. '2000') to fix the y-axis range
%    Enter a range (e.g. '-1000:1000') to fix the y-axis min and max
%  Segment length:
%    Enter a length of time ('10m','30s','5h', etc.) or a number of points to
%     display ('2400')
%  "Option click" (Ctrl-click on the Mac, right button(?) on UNIX/PC)
%    lets you change the linetype or demultiplex the time series channel
%
% BUGS: The tsplot window's ui "children" aren't available until
%       after you click in the window, so the startup segment length, etc, aren't
%       drawn on startup
% W Crawford 10/99

%  "Option click" (Ctrl-click on the Mac, right button(?) on UNIX/PC)
%    lets you pick P-, S-, W-waves, change the linetype, or demultiplex
%     the channel
%

ts_init;	% Sets up global variables
ts_set('dialogs',1);

ts_verbose(1, 'Starting...');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
initTSglobals();    % Clear all globals and initialize TSPLOT_VARIABLES
global TSPLOT_DATA TSPLOT_VARIABLES
global TEMP_READCHANS TEMP_QUITSTATE          
global TSPLOT_DATAFILE                        % Is this still useful?  Only used if nargin==0
											  % and TSPLOT_VARIABLES seems
											  % to have equivalent

datareadin = 0;

if nargin==0,	% tsplot() called, ask for datafile information
	TSPLOT_DATAFILE = ts_file;		% Initialize/reset the global
	devnum=1; chnum = 0;
	% Commenting out the following gets rid of the datafile GUI and
	% relies on each read* to figure out its parameters
	while devnum>0,
		fFormat = menu('Data file format',ts_file.listFormats());
        df=ts_file('',fFormat);
		td=read(df);
% 		h = tsp_DatafileGUI;
% 		% If the file is already defined, load it in
 		if length(TSPLOT_VARIABLES.files) >= devnum,
 			if ~isempty(TSPLOT_VARIABLES.files(devnum).readchans),
 				df = TSPLOT_VARIABLES.files(devnum);
 			end
 		end
% 
 		TSPLOT_VARIABLES.numdevs = TSPLOT_VARIABLES.numdevs+1;	
 		TSPLOT_VARIABLES.files(devnum) = df;	
 		for i = chnum+1:chnum+length(TEMP_READCHANS),
 			TSPLOT_VARIABLES.chan(i) = TSPLOT_VARIABLES.chan(1);
 		end
 		chnum = chnum+length(TEMP_READCHANS);
		resp=input('Read in more datafiles? ','s');
		if lower(resp(1))=='n', devnum=0;
        else devnum=devnum+1;
		end
 	end
else
	if isa(varargin{1},'ts_data'),	% tsplot(tdata,...) called, just display the data
		TSPLOT_VARIABLES.showFileInfo=0;
		td = varargin{1};
		numchans = td.nChannels;
		for i = 1:numchans;
            chan=td.channel(i);
			instName =  deblank(chan.metadata.station); %instName = instName(:)'; % make sure its a row
			if i==1,
				dfiles=ts_file();
				numInsts = 1;	
                dfiles.ch_metadata(1).station = instName;	
				instNameList{1}=instName;
				dfiles.readchans = chan.sourcefile.readchans;
                dfiles.nChannels = chan.sourcefile.nChannels;
            else
				for instnum = 1:numInsts,
					foundInst=0;
					if strcmp(instName,instNameList{instnum}),	% already have this station
						foundInst = 1;
						dfiles(instnum).readchans  = ...
									[dfiles(instnum).readchans chan.sourcefile.readchans];
						break;
					end
				end
				if ~foundInst,	% Need to assign a new instrument
					numInsts =  numInsts +1;
					instnum =  numInsts;
					dfiles(instnum).ch_metadata(1).station = instName;	
					dfiles(instnum).readchans = chan.sourcefile.readchans;
					instNameList{instnum}=instName;
				end
			end
		end
		for i = 1:numInsts,
 			dfiles(i).ch_metadata(1).information = [dfiles(i).filename ', channels' sprintf(' %d',dfiles(i).readchans)];
		end
		TSPLOT_DATA = td; clear td
		TSPLOT_VARIABLES.numdevs = numInsts;
		TSPLOT_VARIABLES.files = dfiles;
		datareadin = 1;
		if nargin > 1,
			options = {varargin{2:nargin}};
		end
		for i = 2:TSPLOT_DATA.nChannels,
			TSPLOT_VARIABLES.chan(i) = TSPLOT_VARIABLES.chan(1);
		end
	else % tsplot(dfile,...) or tsplot({dfile1,dfile2,...},...)
		chnum = 0;
		if ~isa(varargin{1},'cell'),
			dfilelist = {varargin{1}};
		else
			dfilelist = varargin{1};
		end
		for devnum=1:length(dfilelist),
			dfile = dfilelist{devnum};
			TSPLOT_VARIABLES.files(devnum) = dfile;
%
			readchans = dfile.readchans;
			temp = dfile.filename;
			if isa(temp,'cell'),
				TSPLOT_VARIABLES.files(devnum).ch_metadata(1).information = ['multiple files, channels' sprintf(' %d',readchans)];
            else
				TSPLOT_VARIABLES.files(devnum).ch_metadata(1).information = [temp ', channels' sprintf(' %d',readchans)];
			end
			for j = chnum+1:chnum+length(readchans),
				TSPLOT_VARIABLES.chan(j) = TSPLOT_VARIABLES.chan(1);
			end
			chnum = chnum+length(readchans);
		end
		TSPLOT_VARIABLES.numdevs = devnum;
		%numchans = chnum;
		if nargin >= 2,
			TSPLOT_VARIABLES.dataStart = varargin{2};
			if nargin>=3,
				TSPLOT_VARIABLES.dataLen = varargin{3};
				if nargin>=4,
					options = {varargin{4:nargin}};
				end
			end
		end
		
	end
end

ts_debug(1, '%d files\n',TSPLOT_VARIABLES.numdevs);
fig = tsp_TimeGUI;  % Open GUI
figure(fig);
if ~datareadin,
	tsp_TimeGUIMod('readData',TSPLOT_VARIABLES.dataStart,'noplot');
	ts_debug(1, 'reading data');
	datareadin = 1;
end
if exist('options','var')
	for i=1:2:length(options),
		field = options{i}; value = options{i+1};
		switch(field),
			case 'Y Range', tsp_TimeGUIMod('setyrange',value,'noplot');
			case 'Segment Length', tsp_TimeGUIMod('setseglen',value,'noplot');
			case 'Address', tsp_TimeGUIMod('setaddress',value,'noplot');
			case 'Title', tsp_TimeGUIMod('setTitle',value,'noplot');
			case {'hilite','highlight','hilites','highlights'},TSPLOT_VARIABLES.highlights=value;
			otherwise, warning('Invalid tsplot field:''%s''',field);
		end
	end
end
TSPLOT_VARIABLES.timefigureprops = TSPLOT_DATA.figureprops;
TSPLOT_VARIABLES.timefigureprops.nSubplots = TSPLOT_DATA.nChannels;
tsp_plottime();
ts_debug(1, 'Exiting');
end

function initTSglobals()
clear global TSPLOT_DATA
clear global TSPLOT_VARIABLES TSPLOT_DATAFILE TEMP_READCHANS TEMP_QUITSTATE   % Get rid of these, if they exist already
global TSPLOT_VARIABLES

% "USER-CHANGEABLE" VARIABLES %%%%%%%%%%%%%
phases    = {'P','S','W'};	% Shows onscreen info only for the first 3
phasecols = {'g','r','m'};

TSPLOT_VARIABLES.dataStart = 0;	% offset (seconds) from start of file
TSPLOT_VARIABLES.dataLen = 0;	% length (seconds) to read (0=read to end of file)
TSPLOT_VARIABLES.numdevs = 0;
TSPLOT_VARIABLES.datafileformats = ts_file.listFormats(); % Used to call read*
TSPLOT_VARIABLES.highlights=[];
TSPLOT_VARIABLES.tinfo = struct('filt',struct(	'state','off',...
												'hipasscorner',[], ...
												'lopasscorner',[], ...
												'npoles',4, ...
												'foreandback','no'), ...
									'whichtoplot','unfiltered');
TSPLOT_VARIABLES.files(1) = ts_file;			
TSPLOT_VARIABLES.spectinfo = struct('startrec',0,'endrec',0, ...
									'startaddr',1,'endaddr',1, ...
									'windowsize',2048, ...
									'windowtype','', ...
									'customaddrs','no', ...
									'whichspectplot','spect',...
									'infoline', '');
TSPLOT_VARIABLES.title = 'tsplot';

phase = [];		
for i = 1:length(phases),
	phase{i} = @eventarrival;
end
TSPLOT_VARIABLES.eqpick = struct('filename','',...
									'numsaved',0,...
									'phasename',{phases}, ...	
									'phasecolor',{phasecols}, ...	
									'qualitydefs',[2 4 8 16 32]);	
TSPLOT_VARIABLES.chan(1).phase	= phase;
TSPLOT_VARIABLES.timefigureprops = ts_data_figureproperties;
TSPLOT_VARIABLES.freqfigureprops = ts_figureproperties;
TSPLOT_VARIABLES.showFileInfo=1;
TSPLOT_VARIABLES.timeplot=struct('start',0,'length',0);
end


